<?php
namespace Kuoqi{
    /**
     * Class Params 系统层参数控制对象
     * @package Kuoqi
     * 针对系统的参数对象的相关信息的控制
     */
    class Params{
        /**
         * 初始化
         */
        static function __initialize(){
            defined('SAFE_CODE')||define('SAFE_CODE',true);//强制定义安全字段，防止文件在服务器被异常执行
            self::$APP_CONFIG = new \stdClass();
            self::$ROOT = dirname(__FILE__);//ROOT系统根目录的路径信息
            self::$APP_ROOT = (isset($_SERVER['DOCUMENT_ROOT'])&&$_SERVER['DOCUMENT_ROOT'])?$_SERVER['DOCUMENT_ROOT']:(self::$ROOT);//APP系统根目录的路径信息
            //home为app下面的项目的根目录路径
            self::$HOME = '{@APP}/web';
            self::$METHOD = isset($_SERVER['REQUEST_METHOD'])?$_SERVER['REQUEST_METHOD']:'option';
            self::$HOST = isset($_SERVER['HTTP_HOST'])?$_SERVER['HTTP_HOST']:'';
            self::$REQUEST_URL = isset($_SERVER['REQUEST_URI'])?$_SERVER['REQUEST_URI']:"/";
            if(strlen(\Phar::running()) ){
                self::$IS_PHAR = true;
            }else{
                self::$IS_PHAR = false;
            }
            //判定php的模式，是哪种
            if(strtolower(PHP_SAPI)=='cli'){
                self::$PHP_MODE = self::$CODE_MODE_CLI;
            }else{
                self::$PROTOCOL = (!empty($_SERVER['HTTPS']) && $_SERVER['HTTPS'] !== 'off' || (isset($_SERVER['SERVER_PORT'])&&($_SERVER['SERVER_PORT'] == 443))) ? "https" : "http";
                self::$WEB = self::$PROTOCOL .'://'. self::$HOST;
                //判定输入模式，并进行一个基础的解析
                self::$INDEX = htmlentities((isset($_SERVER['SCRIPT_NAME']))?$_SERVER['SCRIPT_NAME']:Params::$DEFAULT_INLET);
                self::$HOME = isset($_SERVER['SCRIPT_FILENAME'])?dirname($_SERVER['SCRIPT_FILENAME']):'{@APP}/web';
            }
//            self::$AUTOLOAD_ROUTE_LIST = array();//清空加载路由列表
//            self::$AUTOLOAD_FILE_LIST = array();//清空加载文件列表
            self::$INPUT_GETS = array();
            self::$INPUT_POSTS = array();
            self::$INPUT_JSONS = array();
//            self::$PROCESS_LOGS = array();
            self::$LOG_DATA = array();
        }
        static $PHP_MODE = 1;//基础的php模式
        static $CODE_MODE_WEB = 1;//使用web模式
        static $CODE_MODE_CLI = 2;//使用cli模式
        static $APP_CONFIG = null;//APP的基础配置信息
        static $ROOT = '';//系统目录，对应项目中，kuoqi.php的文件目录
        static $APP_ROOT = '';//软件目录，对应项目中，主入口文件位置的目录
        static $HOME = '';//应用目录，对应项目中应用的根目录，例如web项目为web的根目录，CLI模式中，为当前命令执行的目录等
        static $HOST = '';//主机信息
        static $WEB = '';//WEB
        static $PROTOCOL = 'http';//协议，请使用小写
        static $METHOD = '';//请求方式
        static $VERSION = '1.04';//当前系统版本信息
        static $INPUT_GETS = array();
        static $INPUT_POSTS = array();
        static $INPUT_JSONS = array();
        //static $PROCESS_LOGS = array();
        static $AUTOLOAD_ROUTE_LIST = array();//加载的路由列表
        static $AUTOLOAD_FILE_LIST = array();//加载的文件列表
        static $LOG_DATA = array();//日志的数据列表
        static $LOG_ROOT = '{@APP}/runtime/logs';
        static $MODEL_DATA_ROOT = '{@APP}/runtime/data';
        static $PHP_VERSION = PHP_VERSION;//php版本号信息
        static $TEMPLATE_EXT = '.template';//模板文件后缀名
        static $VIEW_EXT = '.html';// view模板的文件后缀名
        static $CTL_EXT = '.class.php';// view模板的文件后缀名
        static $IS_PHAR = false;//是否在php压缩包里面运行
        static $INDEX= '';//默认的主入口文件地址
        static $PHP_SAPI = PHP_SAPI;//获取PHP的使用方式
        static $PHP_SAPI_CLI = 'cli';//获取php的使用方式-cli模式
        static $PHP_SAPI_FPM = 'FPM';//获取PHP的使用方式-fpm模式
        static $DEFAULT_INLET = "index.php";//默认的主入口的信息
        //静态资源的后缀名的列表,如果当这个资源不存在的时候，使用伪静态的内容替换
        static $STATIC_EXTS = array('.js','.css');
        //自生携带的静态数据，意味着，只有整个请求完成这个才消失，放置到内存里面为了快速获取，优化系统性能
        static $DOC_WEB = 'http://www.kuoqps.com';//文档的主机地址
        static $_SYSTEM_CACHE = array();
        static $REQUEST_URL = "";

    }
    /**
     * Class Route 路由对象基础信息
     * @package Kuoqi
     *
     */
    class Route{
        /**
         * 初始化
         */
        static function __initialize(){}

        /**
         * 根据字符串转换为路径信息,{@ROOT}/library/api/php-excel/PHPExcel.php
         * @param $root
         * @return mixed
         */
        static public function str2rootpath($root,$isRealpath = false){
            $root = str_replace('{@HOME}',Params::$HOME,$root);
            $root = str_replace('{@WEB}',Params::$WEB,$root);
            $root = str_replace('{@APP}',Params::$APP_ROOT,$root);
            $root = str_replace('{@ROOT}',Params::$ROOT,$root);
            if($isRealpath){
                $root = realpath($root);
            }
            $root = str_replace('\\','/',$root);
            return $root;
        }

        /**
         * 路径数组转字符串
         * @param $url
         * @return array
         */
        static public function query2arr($url){
            $arr = array();
            if(is_array($url)){
                $arr = $url;
            }else{
                foreach (explode('&',$url) as $v){
                    $tmp = explode('=',$v);
                    if(count($tmp)==2){
                        $arr[$tmp[0]] = $tmp[1];
                    }
                }
            }
            return $arr;
        }

        /**
         * 路径字符串转路径数组
         * @param $arr
         * @return string
         */
        static public function arr2query($arr){
            $context = $arr;
            if(is_array($arr)){
                $tmp = array();
                foreach ($arr as $k=>$v){
                    $tmp[] = sprintf('%s=%s',$k,$v);
                }
                $context = implode('&',$tmp);
            }
            return $context;
        }

        /**
         * 标准驼峰写法，首字母大写中间的大写字母转换为_加小写的方式,包含了中划线的写法
         * @param $code
         * @return string
         * User kuoqi
         * Datetime 2022/3/6 0006 10:35
         * Remark
         */
        static public function code2hump($code){
            //首先第一步，将首字母大写
            $code = str_replace('-','_',$code);
            $arr = explode('_',$code);
            foreach($arr as $k=>$v){
                $arr[$k] = ucfirst($v);
            }
            $code = implode('',$arr);
            return $code;
        }

        /**
         * 将驼峰的方式转换为代码的方式 首字母大写中间的大写字母转换为_加小写的方式
         * @param $hump
         * @return string
         */
        static public function hump2code($hump){
            $opt = '';
            for($i=0;$i<strlen($hump);$i++){
                if((($ascii = ord($hump[$i]))>=65)&&($ascii<=90)){
                    $opt .= $opt?('_'.chr($ascii+32)):chr($ascii+32);
                }else{
                    $opt .= $hump[$i];
                }
            }
            return $opt;
        }


        /**
         * 任意文件路径转项目内文件路径信息，并会优化路由符号为路径符号
         * @param $path
         * @param bool $extLast 是否包含最后一个字段
         * @return string
         */
        static public function topath($path,$extLast = true){
            $path = str_replace("\\",'/',$path);
            $arr = explode("/",$path);
            $opath = "";
            foreach ($arr as $k=>$item){
                if((count($arr)==$k+1)&&$extLast){
                    $opath .= $opath?("/".$item):$item;
                }else{
                    $opath .= $opath?("/".self::hump2code($item)):self::hump2code($item);
                }
            }
            return $opath;
        }

        public $method;//请求方式
        public $iptarr = array();//输出路由的数组信息
        public $route = '';// 输入的基础路由信息
        public $scene = '';// 路由的第一个字段信息，也是基础路由模块信息
        public $path = '';// 字路由路径
        protected $_is_system = false;//是否系统的参数模块信息

        /**
         * Route constructor.
         * @param string $route
         * @param bool $extLast
         */
        public function __construct($route = '',$extLast = true)
        {
            //如果路由第一个为\\，则取消第一个这个参数的数据内容
            if(strpos($route,"\\")===0){
                $route = substr($route,1);
            }

            //综合处理数据内容
            $this->method = strtoupper(Params::$METHOD);//请求方式的规定
            $this->iptarr = explode('\\',$route);
            $code = stripos($route,'\\');
            $this->route = $route;
            $this->scene = ucwords(substr($route,0,$code));
            //对于字段的解析，然后进行一个返回信息绑定
            $this->path = ucwords(substr($route,$code+1));
            if($this->scene=='Kuoqi'){
                $this->_is_system = true;
                //$this->path = 'library/'.$this->path;
                $this->path = sprintf('library/%s',$this->path);
                //$this->path = sprintf('library/%',$this->path);
            }else if($this->scene=='App'){
                $this->path = sprintf('application/%s',$this->path);
            }else{
                $this->path = sprintf('%s/%s',$this->scene,$this->path);
            }
            $this->path = self::topath($this->path,$extLast);//文件路径转项目路径
        }
        //是否为系统的路由
        public function isSystem(){ return $this->_is_system;}
    }
    class Logs
    {
        /**
         * 临时的数据监听
         * @var array
         */
        static $tmp_listen = array();

        //是否开启
        /**
         * 切片文件的大小
         * @var string
         */
        protected $file_size = '2097152';//2M = 2097152
        //当前的日志头
        protected $file_prefix = '';

        /**
         * 是否cli模式
         * @var bool
         */
        protected $iscli = false;

        /**
         * 使用日志模块，默认为info的文件信息
         * Logs constructor.
         * @param string $filename
         */
        public function __construct($filename = 'info')
        {
            if(strtolower(PHP_SAPI)=='cli'){
                $this->iscli = true;
            }else{
                $this->iscli = false;
            }
            $this->file_prefix = $filename.'_';
            //监听开始
            $this->iscli||self::listenStart('request');
        }

        /**
         * 记录一条日志数据信息
         * @param $msg
         * @param string $code
         * @param int $timelen
         */
        public function log($msg,$code = 'info',$timelen = 0){
            if($this->iscli){
                //这种模式下面，只有直接写入日志发的份
                $stream = sprintf('[%s]'.date('Y-m-d H:i:s').' %s runtime %s'.PHP_EOL
                    ,$code
                    ,print_r($msg,true)
                    ,$timelen
                );
                $fopen = $this->fopen();
                fwrite($fopen,$stream);
                fclose($fopen);
            }else{
                Params::$LOG_DATA[] = self::initItem($msg,$code,$timelen);
            }
        }

        public function fopen(){
            $root = Route::str2rootpath(Params::$LOG_ROOT);
            if(!(is_dir($root)||mkdir($root,'0777',true))){
                die('log model:make '.Params::$LOG_ROOT.' error'.PHP_EOL);//如果创建失败，则直接进入到异常流程
            }
            //将内容写入到日志里面去
            if(!is_writable($root)){ return false;}
            $file = $root.'/'.$this->file_prefix.date('Ymd',time()).'.log';
            if(is_file($file)&&(filesize($file)>$this->file_size)){
                //进行文件转存
                $tfile = $root.'/'.$this->file_prefix.date('Ymd',time()).'_'.date('His').'.log';
                file_put_contents($tfile,file_get_contents($file));
                $fopen = @fopen($file,'w');
            }else{
                $fopen = @fopen($file,'a');
            }
            if(!$fopen){
                die('log model:make '.$fopen.' error'.PHP_EOL);
            }
            return $fopen;
        }

        /**
         * 对象摧毁触发
         */
        public function __destruct()
        {
            if(Params::$IS_PHAR){ return;}
            if($this->iscli){ return;}
            $fopen = $this->fopen();
            if($fopen){
                fwrite($fopen,$this->getHead());
                fwrite($fopen,$this->getConts());
                fwrite($fopen,$this->getFoot());
                fclose($fopen);
            }
        }

        /**
         * 获取每条日志的头部信息
         */
        final function getHead(){
            return sprintf('%s[request]%s on %s ips %s'.PHP_EOL
                ,"=============================================\n"
                ,Params::$METHOD
                ,isset($_SERVER['REQUEST_URI'])?$_SERVER['REQUEST_URI']:'(noset)'
                ,isset($_SERVER["REMOTE_ADDR"])?$_SERVER["REMOTE_ADDR"]:'(noset)'
            );
        }
        final function getConts(){
            $stream = '';
            foreach (Params::$LOG_DATA as $item){
                $stream .= sprintf('[%s] %s at %s runtime %s'.PHP_EOL
                    ,$item['code']
                    ,$item['msg']
                    ,$item['time']
                    ,$item['timelen']
                );
            }
            return $stream;
        }

        /**
         * 获取尾部相关信息
         */
        final function getFoot(){
            return sprintf('== END at %s runtime %s'.PHP_EOL
                ,date('Y-m-d H:i:s'),
                self::listenEnd('request')
            );
        }

        /**
         * 获取当前的唯一识别数组信息
         * @param $msg
         * @param $code
         * @param int $timelen
         * @return array
         */
        static public function initItem($msg,$code,$timelen = 0){
            return array('msg'=>$msg,'code'=>$code,'timelen'=>$timelen,'time'=>date('Y-m-d H:i:s'));
        }

        /**
         * 静态的方式初始化一个日志
         * @param $msg
         * @param string $code
         * @param int $timelen
         */
        static public function initLog($msg,$code = 'info',$timelen = 0){
            \Kuoqi::$logs&&(Config::get('APP_DEBUG')||Config::get('DEBUG'))&&\Kuoqi::$logs->log($msg,$code,$timelen);
        }

        /**
         * 创建一个时间监测点,输入自定义的值
         * @param string $code
         */
        static public function listenStart($custom_code = 'normal'){
            self::$tmp_listen[$custom_code] = microtime(true);
        }

        /**
         * 结束一个时间监测点,并返回当前的数据信息
         * @param string $code
         * @return int|mixed|string
         */
        static public function listenEnd($custom_code = 'normal'){
            if(isset(self::$tmp_listen[$custom_code])){
                $custom_code = microtime(true) - self::$tmp_listen[$custom_code];
                unset(self::$tmp_listen[$custom_code]);
                return $custom_code;
            }else{
                return -1;
            }
        }
    }
    class Config{
        //初始化
        static function __initialize($opts = array()){
            require_once(\Kuoqi\Route::str2rootpath('{@ROOT}/function.php'));
            $config = require_once(\Kuoqi\Route::str2rootpath('{@ROOT}/config.php'));
            if(is_array($config)){
                foreach($config as $k=>$v){
                    self::set($k,$v);
                }
            }
            if(is_array($opts)){
                foreach($opts as $k=>$v){
                    self::set($k,$v);
                }
            }
        }
        static function set($n,$v){
            if(isset(Params::$$n)){Params::$$n = $v;}
            return Params::$APP_CONFIG->$n = $v;
        }
        static function get($n){
            return isset(Params::$APP_CONFIG->$n)?Params::$APP_CONFIG->$n:(isset(Params::$$n)?Params::$$n:null);
        }
        //配置get方式的参数信息
        static function setInputget($m,$v = null){
            if(is_array($m)){
                //如果是数组，则判定为多数据写入
                foreach ($m as $k=>$item) {
                    self::setInputget($k,$item);
                }
            }else{
                //如果不是数组，判定为常规数据写入
                Params::$INPUT_GETS[$m] = $v;
            }
            return null;
        }
        //写入post数据信息到数据序列
        static function setInputpost($m,$v = null){
            if(is_array($m)){
                //如果是数组，则判定为多数据写入
                foreach ($m as $k=>$item) {
                    self::setInputpost($k,$item);
                }
            }else{
                //如果不是数组，判定为常规数据写入
                Params::$INPUT_POSTS[$m] = $v;
            }
            return null;
        }
        //使用json字符串的方式传入
        static function setInputjson($m,$v = null){
            if(is_array($m)){
                //如果是数组，则判定为多数据写入
                foreach ($m as $k=>$item) {
                    self::setInputjson($k,$item);
                }
            }else{
                //如果不是数组，判定为常规数据写入
                Params::$INPUT_JSONS[$m] = $v;
            }
            return null;
        }
        // 获取inputRequest的所有信息
        static function getInputrequest(){
            $arr = array_merge(Params::$INPUT_GETS,Params::$INPUT_POSTS,Params::$INPUT_JSONS);
            return $arr;
        }

        /**
         * 使用自定义的配置文件
         * @param $file
         * @throws Error
         */
        static public function useCustomfile($file){
            $fl = \Kuoqi\Route::str2rootpath($file);
            if(is_file($fl)&&is_readable($fl)){
                $arr = @include ($fl);
                if(is_array($arr)){
                    foreach($arr as $k=>$v){
                        self::set($k,$v);
                    }
                }
            }else{
                 \Kuoqi::error(sprintf("引用配置文件失败，配置文件不存在或不可读",$file));
            }
            return;
        }
    }
    class KuoqiException{
        protected $errcode = 0;//错误异常code
        protected $errmsg = '';//错误异常代码
        protected $infomsgs = array();//获取当前对象保存的所有的输出内容的信息
        public function getError(){ return $this->errmsg;}
        public function setError($errmsg){
            if($errmsg){
                $this->errmsg = $errmsg;$this->errcode = -1;
            }else{
                $this->errmsg = $errmsg;$this->errcode = 0;
            }
        }
        public function isValied(){ return !(bool)$this->errcode;}
        public function setInfo($msg){
            $this->infomsgs[] = $msg;
        }
        public function getInfo(){
            if($this->infomsgs){
                return $this->infomsgs[count($this->infomsgs)-1];
            }else{
                return "(no message)";
            }

        }
    }
    class Error extends \Exception {
        static $can_die = true;
        public $trace_stream = '';//被注入的错误信息
        public function __construct($message = "")
        {
            parent::__construct($message);
        }

        /**
         * quit_cli 2024-05-30 cli模式下的显示内容
         */
        final function quit_cli(){
            echo "errinfo:".$this->getMessage().PHP_EOL;
            echo sprintf('errmsg file: %s  on line %s', self::echofilter($this->getFile()), $this->getLine()).PHP_EOL;
            echo $this->getTraceAsString().PHP_EOL;
        }
        final function _autoRoutes(){
            $stream = '';
            for($i=count(\Kuoqi\Params::$AUTOLOAD_ROUTE_LIST)-1;$i>-1;$i--){
                $stream .=  sprintf(
                    '<pre style="margin: 0;padding: 5px 15px 5px 30px; font-family: Arial,Verdana,sans-serif; font-size: inherit;color:#666666">%s</pre>'
                    ,\Kuoqi\Params::$AUTOLOAD_ROUTE_LIST[$i]);
            }
            $stream = sprintf('<details style="font-size: 0.85em;"><summary style="margin: 0;padding: 5px 15px; font-family: Arial,Verdana,sans-serif; font-size: inherit;color:#666666">路由信息</summary>%s</details>',$stream);
            return $stream;
        }
        //自动显示已加载的文件列表信息
        final function _autoFiles(){
            $stream = '';
            for($i=count(\Kuoqi\Params::$AUTOLOAD_FILE_LIST)-1;$i>-1;$i--){
                $stream .=  sprintf(
                    '<pre style="margin: 0;padding: 5px 15px 5px 30px; font-family: Arial,Verdana,sans-serif; font-size: inherit;color:#666666">%s</pre>'
                    ,\Kuoqi\Params::$AUTOLOAD_FILE_LIST[$i]);
            }
            $stream = sprintf('<details style="font-size: 0.85em;"><summary style="margin: 0;padding: 5px 15px; font-family: Arial,Verdana,sans-serif; font-size: inherit;color:#666666">加载文件信息</summary>%s</details>',$stream);
            return $stream;
        }
        static function process($message){
            Params::$LOG_DATA[] = \Kuoqi\Logs::initItem($message,'process');
        }
        final function echofilter($stream){
            $stream = str_replace(array(Params::$ROOT,Params::$APP_ROOT),array('${ROOT}','${APP}'),$stream);
            return $stream;
        }
        final function getTraceStream(){
            if($this->trace_stream){
                return $this->trace_stream;
            }else{
                return $this->getTraceAsString();
            }
        }
        final function getErrornow(){
            //基础异常信息
            $stream = '<div style="font-family: Arial,Verdana,sans-serif; font-size: inherit; background-color: #4a5a80; margin: 0 0 0 0; padding: 0.5em 0.75em;font-weight: bold;color:#ffffff;">'.$this->getMessage().'</div>';
            //基础页面信息
            $stream .= sprintf('<p style="font-family: Arial,Verdana,sans-serif; font-size: inherit; margin: 0.5em; font-weight: 600;color: #376a56;">errmsg file: %s  on line %s</p>',
                self::echofilter($this->getFile()),
                $this->getLine()
            );
            return $stream;
        }

        /**
         * createby kuoqi at 2022/11/15 16:24
         * @param $code 配置code
         */
        public function setCode($code){
            $this->code = $code;
        }

        /**
         * createby kuoqi at 2022/11/15 16:24
         * @param $file 配置file
         */
        public function setFile($file){
            $this->file = $file;
        }

        /**
         * createby kuoqi at 2022/11/15 16:24
         * @param $line 配置line
         */
        public function setLine($line){
            $this->line = $line;
        }
        //问题回溯列表信息展示
        final function debugBacktrace(){
            $stream = '';
            $list = debug_backtrace();
            foreach ($list as $item){
                if(isset($item['type'])){
                    if($item['type']=='::'){
                        $stream .= sprintf('<details style="font-size: 0.85em;"><summary style="margin: 0;padding: 5px 15px; font-family: Arial,Verdana,sans-serif; font-size: inherit;color:#666666">%s%s%s</summary><pre style="font-family: Arial,Verdana,sans-serif; font-size: inherit; margin: 0.5em; font-weight: 600;color: #376a56;padding: 5px 15px;">%s</pre></details>'
                            ,$item['class']
                            ,$item['type']
                            ,$item['function']
                            ,print_r($item,true)
                        );
                    }elseif($item['type']=='->'){
                        $stream .= sprintf('<details style="font-size: 0.85em;"><summary style="margin: 0;padding: 5px 15px; font-family: Arial,Verdana,sans-serif; font-size: inherit;color:#666666">%s%s%s</summary><pre style="font-family: Arial,Verdana,sans-serif; font-size: inherit; margin: 0.5em; font-weight: 600;color: #376a56;padding: 5px 15px;">%s</pre></details>'
                            ,$item['class']
                            ,$item['type']
                            ,$item['function']
                            ,print_r($item,true)
                        );
                    }else{
                        $stream .= '-';
                    }
                }else{
                    $stream .= '---------------';
                }
            }
            return $stream;
        }
        final function quit_debug(){
            $stream = '';
            $stream .= '<meta http-equiv="Content-Type" content="text/html; CHARSET=utf-8">';
            $stream .= '<meta name="viewport" content="initial-scale=1, width=device-width, maximum-scale=1, user-scalable=no">';
            $stream .= '<style>body{ margin: 0;padding: 0;background-color: #f2efe7;}</style>';
            $stream .= $this->getErrornow();//获取当前异常信息
            $stream .= sprintf('<pre style="margin: 0;padding: 5px 15px; font-family: Arial,Verdana,sans-serif; font-size: inherit;color:#666666">%s</pre>',self::echofilter($this->getTraceStream()));
            //展示问题回溯列表
            $stream .= $this->debugBacktrace();
            //这个是致死的进程
            //如果是debug模式，展示app_root与root数据，这两个很关键
            //展示路由信息
            $stream .= self::_autoRoutes();//显示已自动加载的路由列表
            $stream .= self::_autoFiles();//显示已自动加载的文件列表
            $stream .= '<p style="margin: 0;padding: 5px 15px; font-family: Arial,Verdana,sans-serif; font-size: inherit;font-size: small;color: #888888;">ROOT->'.Params::$ROOT.'</p>';
            $stream .= '<p style="margin: 0;padding: 5px 15px; font-family: Arial,Verdana,sans-serif; font-size: inherit;font-size: small;color: #888888;">APP_ROOT->'.Params::$APP_ROOT.'</p>';
            $stream .= '<p style="margin: 0;padding: 5px 15px; font-family: Arial,Verdana,sans-serif; font-size: inherit;font-size: small;color: #888888;">HOST->'.Params::$HOST.'</p>';
            $stream .= '<p style="margin: 0;padding: 5px 15px; font-family: Arial,Verdana,sans-serif; font-size: inherit;font-size: small;">';
            $stream .= 'VERSION&nbsp;:&nbsp;'.Params::$VERSION;
            $stream .= '&nbsp;,&nbsp;PHP_VERSION&nbsp;:&nbsp;'.Params::$PHP_VERSION;
            $stream .= sprintf('&nbsp;,&nbsp;is_phar&nbsp;:&nbsp;%s',Params::$IS_PHAR?'true':'false');
            $stream .= sprintf('&nbsp;,&nbsp;DEBUG&nbsp;:&nbsp;%s',Config::get('DEBUG')?'true':'false');
            $stream .= sprintf('&nbsp;,&nbsp;APP_DEBUG&nbsp;:&nbsp;%s',Config::get('APP_DEBUG')?'true':'false');
            $stream .= '</p>';
            $stream .= '<p style="margin: 0;padding: 5px 15px; font-family: Arial,Verdana,sans-serif; font-size: inherit;font-size: small;">KuoqiPHP-oop -->{PHP依旧是世界上最好的语言}</p>';
            //kill进程
            return $stream;
        }
        final function quit_template(){
            $stream = '';
            $stream .= '<meta http-equiv="Content-Type" content="text/html; CHARSET=utf-8">';
            $stream .= '<meta name="viewport" content="initial-scale=1, width=device-width, maximum-scale=1, user-scalable=no">';
            $stream .= '<h1 style="text-align: center;margin: 0.67em;">404 Not Found</h1>';
            $stream .= '<hr />';
            if(Config::get('APP_DEBUG')){
                $stream .= '<p style="text-align: center;"> '.$this->getMessage().'</p>';
            }else{
                $stream .= '<p style="text-align: center;"><a style="text-decoration: unset;color: initial;" href="'.Config::get('DEBUG_REDIRECT_URL').'">'.Config::get('DEBUG_REDIRECT_TEXT').'</a></p>';
            }
            $stream .= '<p style="text-align: center;"><a target="_blank" style="text-decoration: unset;color: initial;" href="'.\Kuoqi\Params::$DOC_WEB.'/index/doc/index">KuoqiPHP '.Params::$VERSION.'</a></p>';
            $stream .= '<p style="text-align: center;">PHP_VERSION '.Params::$PHP_VERSION.'</p>';
            return $stream;
        }
        //退出
        public function quit(){
            if(strtolower(Params::$PHP_SAPI)==Params::$PHP_SAPI_CLI){
                $this->quit_cli();
            }else{
                if(Config::get('DEBUG')){
                    $stream = $this->quit_debug();
                }else{
                    $stream = $this->quit_template();
                }
                //$stream = mb_convert_encoding($stream,"UTF-8","GBK");
                $flush = new \Kuoqi\Flush();
                $flush->to404($stream);
            }
        }

        /**
         * createby kuoqi at 2022/11/15 10:23
         * set_error_handler
         * handler( int $errno, string $errstr[, string $errfile[, int $errline[, array $errcontext]]] )
         * E_ERROR,E_PARSE,E_NOTICE,E_CORE_ERROR,E_CORE_WARNING,E_COMPILE_ERROR,E_COMPILE_WARNING,E_STRICT
         * 以下级别的错误不能由用户定义的函数来处理： E_ERROR、 E_PARSE、 E_CORE_ERROR、 E_CORE_WARNING、 E_COMPILE_ERROR、 E_COMPILE_WARNING，和在调用 set_error_handler() 函数所在文件中产生的大多数 E_STRICT。
         */
        static public function ErrorHandler($errno,$errstr,$errfile,$errline,$errcontext){
            //var_dump($errno,$errstr,$errfile,$errline,$errcontext,error_get_last());
            //var_dump(E_ERROR,E_PARSE,E_ERROR,E_PARSE,E_NOTICE,E_CORE_ERROR,E_CORE_WARNING,E_COMPILE_ERROR,E_COMPILE_WARNING,E_STRICT);exit();
            if(in_array($errno,array(
                E_WARNING,E_NOTICE,E_CORE_WARNING,
            ))){
                \Kuoqi::log(sprintf('errmsg %s and file is %s on line %s',$errstr,$errfile,$errline),'error');
            }else{
                $error = new self($errstr);
                $error->setFile($errfile);
                $error->setCode($errno);
                $error->setLine($errline);
                return $error->quit();
            }
        }

        /**
         * createby kuoqi at 2022/11/15 10:24
         * set_exception_handler
         */
        static public function ExceptionHandler($err){
            //error_log("You messed up!", 3, __DIR__."/errors.log".PHP_EOL);
            $error = new self($err->getMessage());
            $error->setFile($err->getFile());
            $error->setCode($err->getCode());
            $error->setLine($err->getLine());
            $error->quit();
        }
    }
    /**
     * 增加一个专门推送到浏览器的模块
     * Class flush
     * @package Kuoqi
     */
    class flush{
        /**
         * @var array 绑定的额外的头部信息
         */
        protected $code = 200;//code
        protected $status = 'OK';//状态值
        protected $_headers = array();
        protected $content_type = 'text/html';
        protected $stream = '';//输出的内容信息
        static protected $_code_status = array(
            '200'=>'OK',
            '301'=>'Redirect',
            '302'=>'Found',
            '404'=>'Not Found',
        );

        /**
         * flush constructor.
         */
        public function __construct()
        {
            $this->setHead('X-Powered-By','KuoqiPHP.'.\Kuoqi\Params::$VERSION);
        }

        /**
         * 设置内容
         * @param $stream
         */
        public function setContext($stream){
            $this->stream = $stream;
        }
        /**
         * 设置head
         * @param $n head名
         * @param $v head值
         */
        public function setHead($n,$v){
            $this->_headers[$n] = $v;
        }

        /**
         * to404
         */
        public function to404($stream = ''){
            if($stream){
                $this->stream = $stream;
                //ob_clean();
            }else{
                $this->stream = ob_get_clean();
            }
            $this->code = 404;
            $this->toFlush();
        }

        /**
         * to 301
         * @param $redirect
         */
        public function to301($redirect){
            $this->setHead('location',$redirect);
            $this->stream = ob_get_clean();
            $this->code = 301;
            $this->toFlush();
        }

        /**
         * to 302
         * @param $redirect
         */
        public function to302($redirect){
            $this->setHead('location',$redirect);
            $this->stream = ob_get_clean();
            $this->code = 302;
            $this->toFlush();
        }

        /**
         * 推送数据给客户端
         * @param string $stream
         */
        public function toHtml($stream = ''){
            if($stream){
                $this->stream = $stream;
                ob_clean();
            }else{
                $this->stream = ob_get_clean();
            }
            $this->content_type = sprintf('text/html;charset=%s',Config::get('CHARSET'));
            $this->code = 200;
            $this->toFlush();
        }

        /**
         * 推送数据给客户端
         * @param string $stream
         */
        public function toJson($stream = ''){
            if($stream){
                $this->stream = $stream;
                ob_clean();
            }else{
                $this->stream = ob_get_clean();
            }
            $this->content_type = sprintf('application/json;charset=%s',Config::get('CHARSET'));
            $this->code = 200;
            $this->toFlush();
        }

        /**
         * 输出内容区域
         */
        public function toFlush(){
            $errstr = ob_get_clean();
            if($errstr){
                \Kuoqi::log($errstr,"error");
            }
            header(sprintf("HTTP/1.1 %s %s",$this->code,self::$_code_status[$this->code]));
            header("Date:".gmdate('D,dMYH:i:sT'));
            header("Content-Type:{$this->content_type}");
            header(sprintf("Content-Length: %s",strlen($this->stream)));
            foreach ($this->_headers as $k=>$v){
                header(sprintf('%s: %s',$k,$v));
            }
            echo $this->stream;
            flush();//刷新输出缓冲
        }
    }
}
namespace {
    use \Kuoqi\Error;
    use Kuoqi\Logs;
    use \Kuoqi\Params;
    use \Kuoqi\Config;
    use \Kuoqi\Route;
    /**
     * Class Kuoqi
     * 对象都使用首字母大写的驼峰形式标识
     */
    class Kuoqi{
        static $logs = null;// 日志的相关模块信息
        static $app = null;// 当前使用的app
        /**
         * 系统初始化部分
         * @param array $opts
         * @return \Kuoqi\App|\Kuoqi\Cli
         */
        static public function __initialize($opts = array()){
            //error_reporting(0);
            //error_reporting(-1);
            Params::__initialize();//内部param的初始化
            Config::__initialize(isset($opts['config'])?$opts['config']:array());//配置初始化
            set_error_handler(array('Kuoqi\Error','ErrorHandler'));
            set_exception_handler(array('Kuoqi\Error','ExceptionHandler'));
            spl_autoload_register(array('Kuoqi','__autoload'));//注册自动加载函数信息
            //self::$logs = new Logs();//初始化日志模块的相关信息
            Error::process('kuoqi::__initialize');
            if(Params::$PHP_MODE === Params::$CODE_MODE_WEB){
                Error::process('use core with web');
                self::$logs = new \Kuoqi\Logs('web');
                return self::$app =  new \Kuoqi\App();
            }else{
                Error::process('use core with cli');
                self::$logs = new \Kuoqi\Logs('cli');
                return self::$app = new \Kuoqi\Cli();
            }
        }
        /**
         * 自动加载的部分
         * @param $route
         * @throws Error
         */
        static public function __autoload($route){
            Params::$AUTOLOAD_ROUTE_LIST[] = $route;
            $route = new Route($route);
            $file = ($route->isSystem()?'{@ROOT}':'{@APP}').'/'.$route->path.'.class.php';
            if(self::__autoloadfile($file)){
                if(class_exists($route->route,false)){
                    if(method_exists($route->route,'__initialize')){
                        eval('$route->route::__initialize();');
                    }
                }elseif(interface_exists($route->route)){}
                else{
                    throw new Error("Kuoqi::__autoload failed " . 'Class['.$route->route.'] Non-existent');
                }
            }else{
                throw new Error("Kuoqi::__autoload failed " .$file." is not file or can't readable");
            }
        }
        /**
         * 自动加载文件
         * @param $file
         * @param bool $once
         * @return bool|mixed
         */
        static public function __autoloadfile($file,$once = true){
            Params::$AUTOLOAD_FILE_LIST[] = $file;
            $file = \Kuoqi\Route::str2rootpath($file);
            if(is_file($file) && is_readable($file)){
                if($once){
                    return include_once($file);
                }else{
                    return include($file);
                }
            }
            return false;
        }
        /**
         * 日志信息
         * @param $msg
         * @param string $code
         * @param int $timelen
         */
        static public function log($msg,$code = 'info',$timelen = 0){
            self::$logs&&self::$logs->log($msg,$code,$timelen);
        }
        static public function error($msg){
            if(\Kuoqi\Params::$PHP_MODE==\Kuoqi\Params::$CODE_MODE_CLI){
                echo $msg.PHP_EOL;
                //判断是否有GUI，如果没有还是需要使用日志的方式进行
            }else if(\Kuoqi\Params::$PHP_MODE==\Kuoqi\Params::$CODE_MODE_WEB){
                self::log($msg,"error");
            }else{
                throw new Error("need develop");
            }
        }
        static public function warn($msg,$iswrap = true){
            echo $msg.($iswrap?PHP_EOL:"");
        }
        /**
         * 获取缓存内容,只针对一次请求生效
         * @param $n
         * @return mixed|null
         */
        static public function getAskcache($n){
            return isset(Params::$_SYSTEM_CACHE[$n])?Params::$_SYSTEM_CACHE[$n]:null;
        }
        /**
         * 写入缓存内容，只针对一次请求生效
         * @param $n
         * @param $any
         * @return mixed
         */
        static public function setAskcache($n,$any){
            return Params::$_SYSTEM_CACHE[$n] = $any;
        }
        //获取当前对象的唯一识别，可追加唯一识别
        static final function getClassid($class,$ext = ''){
            $classname = get_class($class);
            return sprintf('%s::%s',$classname,$ext);
        }

        /**
         * 使用简单的数组合并
         * @param $inputs
         * @param $defs
         * @return mixed
         */
        static final function opts($inputs,$defs){
            foreach ($defs as $k=>$v){
                if(isset($inputs[$k])){
                    if(is_array($inputs[$k])&&count($inputs)){
                        $defs[$k] = $inputs[$k];
                    }else if(strlen($inputs[$k])>0){
                        $defs[$k] = $inputs[$k];
                    }
                }
            }
            return array_merge($inputs,$defs);
        }

        /**
         * 判断对象是否存在
         * classExists 2024-09-30
         * @param $route
         * @return bool
         */
        static final function classExists($route){
            $route = new Route($route);
            $file = ($route->isSystem()?'{@ROOT}':'{@APP}').'/'.$route->path.'.class.php';
            if(self::__autoloadfile($file) && class_exists($route->route,false)){
                return true;
            }
            return false;
        }
    }
}
//release
//释放，将已经使用完成之后的资源进行一个释放，减少服务器的占用