<?php
namespace Kuoqi;
use Kuoqi;
class App{
    //模块初始化
    static public function __initialize(){}
    protected $url_driver = '';//url的驱动
    public $index = "";//当前APP应用的INDEX内容
    public $route = null;//输入路由部分信息
    public $controller = null;//控制器对象信息
    protected $format = 'web';//app的格式类型
    public $layout_id = false;//是否启用layout模块,启用之后为启用之后的路由
    public $use_view = false;//是否启用view模块,启用之后为view的路由
    public $is_flush = false;//是否已经输出，如果没有输出的情况下，最后会执行一次输出
    protected $_assigns = array();//内置的保存当前数据的对象信息
    protected $is_custom_error = false;//自定义错误
    public $action_return = "";//action的相应内容

    public $view = null;//view的对象信息
    public $response = null;//响应模块的对象信息
    public $behavior = null;//当前app的古怪行为合集

    /**
     * 运行核心代码
     * running 2024-08-14
     * @throws Error
     */
    public function running(){
        if(strlen(\Phar::running()) ){
            //判断是否在phar中
            if(preg_match('/\.phar/',\Kuoqi\Params::$APP_ROOT)){
                \Kuoqi\Params::$IS_PHAR = true;
            }else{
                \Kuoqi\Params::$IS_PHAR = false;
            }
        }else{
            \Kuoqi\Params::$IS_PHAR = false;
        }
        Error::process('app running');
        $this->__initCommonConfig();
        $this->__initInput();//输入部分的初始化
        $this->__initBehavior();//初始化用户行为部分behavior
        $this->__initPrivateConfig();
        //将当前app的信息注入到控制器中，让后面可以使用
        //本操作只会将本app信息写入到 \Kuoqi\Controller的静态中备用，不进行任何其他操作
        \Kuoqi\Controller::useApp($this);
        \Kuoqi\Helper::useApp($this);//下发给helper
        $this->view = new App\View($this);
        $this->response = new \Kuoqi\App\Response($this);

        //这里进入behavior的部分
        //这里执行一些app的方法，给APP安排上在这上面的显示和初始化这些
        //行为的解析，使用多一个函数的目的是让这个信息可以被手动变更，而不是直接一次性固定死了
        $this->execController();//解析控制器的内容
        $this->execActioner();//解析动作的内容
        //根据不同格式封装输出
        switch($this->getFormat()){
            case 'json':$this->response->toJson();break;
            case 'json_string':$this->response->toJsonString();break;
            case 'web':$this->response->toWeb();break;
            case 'error':$this->response->toCustonError();break;//输出自定义异常
            default:throw new Error('not any outputs with .'.$this->getFormat());
        }
        $this->response->flush();//已web的行为方式输出 //将app的需要输出的信息，都要放置到response中进行输出

    }
    public function __initCommonConfig(){
        Error::process('include common config');
        $file = new FileDriver(sprintf('{@APP}/%s/common/function.php'
            ,\Kuoqi\Route::hump2code(Config::get('APP_PATH'))
        ));
        if($file->exist()){
            $file->includeOnce();
        }else{
            \Kuoqi::$logs->log('no common function','warn');
        }
        $file->setBasename('config.php');
        if($file->exist()){
            $config = $file->includeMore();
            if(is_array($config)){
                foreach($config as $k=>$v){
                    Config::set($k,$v);
                }
            }
        }else{
            \Kuoqi::$logs->log('no common config','warn');
        }
    }
    public function __initPrivateConfig(){
        Error::process('include private config');
        $file = new FileDriver(sprintf('{@APP}/%s/%s/function.php'
            ,\Kuoqi\Route::hump2code(Config::get('APP_PATH'))
            ,\Kuoqi\Route::hump2code($this->route->m)
        ));
        if($file->exist()){
            $file->includeOnce();
        }else{
            \Kuoqi::$logs->log('no private function','warn');
        }
        $file->setBasename('config.php');
        if($file->exist()){
            $config = $file->includeMore();
            if(is_array($config)){
                foreach($config as $k=>$v){
                    Config::set($k,$v);
                }
            }
        }else{
            \Kuoqi::$logs->log('no private config','warn');
        }
    }
    //init input
    public function __initInput()
    {
        Error::process('init input');
        $this->url_driver = new Url();
        //判断第一个是否为入口文件，如果有入口文件则要剔除入口文件信息
        //注意这里
        $this->url_driver->setLink(\Kuoqi\Params::$REQUEST_URL);
        //rawurldecode
        $this->url_driver->analysis();
        $this->route =  $this->url_driver->route;
        Config::setInputget( $this->url_driver->route->extra);
        Config::setInputpost($_POST);
        $stream = file_get_contents('PHP://input');
        $json = json_decode($stream,true);
        if($json){
            Config::setInputjson($json);
        }
    }

    public function __initBehavior()
    {
        //尝试加载控制器
        Error::process('app init behavior');
    }

    /**
     * 获取控制器路由信息
     * @return string
     */
    public function getControllerRoute(){
        return sprintf('\App\%s\Controller\%sController'
            ,Route::code2hump($this->route->m)
            ,Route::code2hump($this->route->c)
        );
    }
    //获取控制器的文件路径，不使用反射的方式获取控制器了，能预见的未来就不需要自动进行，需要手动把握
    //使用加载控制器的方式进行
    public function loadController(){
        $file = sprintf('{@APP}\%s\%s\Controller\%sController%s'
            ,\Kuoqi\Config::get('APP_PATH')
            ,Route::hump2code($this->route->m)
            ,Route::code2hump($this->route->c)
            ,\Kuoqi\Config::get('CTL_EXT')
        );
        $fl = \Kuoqi\Route::str2rootpath($file);
        if(is_file($fl)&&is_readable($fl)){
            include_once ($fl);
        }else{
            throw new Error("控制器文件".$file."加载异常");
        }
    }

    /**
     * 判断当前已经出错了，这个情况后续的输如
     * isError 2024-08-05
     */
    public function isError($stream){
        $this->setFormat("error");
        $this->is_custom_error = true;
        $this->bindAction($stream);
        return $stream;
    }

    //绑定应用行为习惯
    public function  bindBehavior($behavior){
        Error::process('绑定应用行为习惯');
//        if(is_array($behavior)){
//            //配置format格式化的行为方式
//            if(isset($behavior['format'])){
//                $this->format = $behavior['format'];
//            }
//        }
        //todo--其他绑定行为到本app中
    }
    public function bindAction($stream){
        //todo--绑定动作响应的数据信息
        $this->action_return = $stream;
    }

    /**
     * @return string 获取模块的名称
     */
    public function getModeler(){ return \Kuoqi\Route::code2hump($this->route->m);}
    //获取控制器的名称
    public function getController(){ return \Kuoqi\Route::code2hump($this->route->c);}
    //获取操作方法名称
    public function getActioner(){ return \Kuoqi\Route::code2hump($this->route->a);}
    //获取内核信息
    //public function getCore(){ return $this->core;}
    //获取格式化信息 format
    public function getFormat(){ return $this->format;}
    //设置格式化信息
    public function setFormat($format){ return $this->format = $format;}

    /**
     * @desc 获取完整的view的路径信息
     * @param $tpid
     * @return mixed
     */
    public function completeViewpath($tpid){
        return \Kuoqi\App\Route::stdView($tpid);
    }
    //完成完整的路由信息
    public function completeTemplatepath($tpid){
        return new \Kuoqi\Route($tpid);
    }
    //获取参数信息
    public function getAssigns($extraArgs = array()){
        return array_merge($this->_assigns,$extraArgs);
    }
    public function getAssign($args){
        return $this->_assigns[$args];
    }
    //设置参数信息
    public function setAssign($args,$extArgs = null){
        if(is_array($args)){
            foreach($args as $k=>$v){
                $this->setAssign($k,$v);
            }
        }else{
            $this->_assigns[$args] = $extArgs;
        }
    }

    //设置响应头部信息
    public function setHeader($n,$v = ''){
        return $this->response->setHeader($n,$v);
    }

    /**
     * 兼容上一个版本,获取当前系统用户的完整控制器名称
     * @return string
     */
    public function getClassname(){
        return $this->getControllerRoute();
    }

    public function execController()
    {
        $this->loadController();
        $route = $this->getControllerRoute();
        if(class_exists($route,false)){
            $this->controller = new $route();
            $this->bindBehavior($this->controller->beforeBehavior());
            if(method_exists($this->controller,'behavior')){
                $this->bindBehavior($this->controller->behavior());
            }
        }else{
            throw new Error('控制器['. $route .'] 不存在');
        }
    }
    //检测并绑定动作信息
    public function execActioner(){
        if(method_exists($this->controller,$this->getActioner())){
            $action = $this->getActioner();
            $actcontext = $this->controller->$action();
            if(!$this->is_custom_error){
                $this->bindAction($actcontext);
            }
        }else{
            $this->controller->__call($this->getActioner(),[]);
            Kuoqi\Error::process('动作不存在，进入到call流程');
        }
    }



}