<?php
namespace Kuoqi;
/**
 * 缓存基础模块
 * Class Cache
 * @package Kuoqi
 */
class Cache{
    /**
     * 设置模型的文件
     * @param $stream
     * @return string
     */
    static public function setTemplate($stream){
        $tpid = md5($stream);
        $fn = sprintf('{@APP}/%s/%s/%s%s'
            ,Config::get('RUNTIME_PATH')
            ,Config::get('TEMPLATE_PATH')
            ,$tpid
            ,\Kuoqi\Params::$TEMPLATE_EXT
        );
        $fdriver = new FileDriver($fn);
        $fdriver->setContent($stream);
        if(!$fdriver->save()){
            Error::process('配置模板'.$fn.'失败');
        }
        return $tpid;
    }
    /**
     * 获取模型的文件
     * @param $tpid
     * @param $args
     * @return bool|string
     */
    static public function getTemplate($tpid,$args){
        $fn = sprintf('{@APP}/%s/%s/%s%s'
            ,Config::get('RUNTIME_PATH')
            ,Config::get('TEMPLATE_PATH')
            ,$tpid
            ,\Kuoqi\Params::$TEMPLATE_EXT
        );
        $fdriver = new FileDriver($fn);
        return $fdriver->readWithphp($args);
    }
    /**
     * 设置缓存
     * @param $name 缓存名称
     * @param $code 缓存的内容
     * @param int $exp_time 缓存存活时间
     * @param string $cpath 二级文件夹
     * @param bool $isCommon 是否公共缓存
     * @return mixed|null
     */
    static public function setCache($name,$code,$exp_time = 3600,$cpath = '',$isCommon = false){
        $fn = md5($name);
        $filename = sprintf('{@APP}/%s/%s%s%s/%s.cache'
            ,Config::get("RUNTIME_PATH")
            ,Config::get("CACHE_PATH")
            ,$cpath
            ,($isCommon||(!\Kuoqi::$app->route))?"":('/'.\Kuoqi::$app->route->m)
            ,$fn
        );
        $file = new FileDriver($filename);
        $file->setContent(serialize($code).'::::::'.(time()+intval($exp_time)));
        if(!$file->save()){
            Error::process('缓存文件'.$file->getFile().'配置失败');
        }
        return $fn;
    }

    /**
     * 获取缓存内容
     * @param $name 缓存名称
     * @param string $cpath 缓存路径
     * @param bool $isCommon 是否公共文件夹
     * @return mixed|null
     */
    static public function getCache($name,$cpath = '',$isCommon = false){
        $fn = md5($name);
        $filename = sprintf('{@APP}/%s/%s%s%s/%s.cache'
            ,Config::get("RUNTIME_PATH")
            ,Config::get("CACHE_PATH")
            ,$cpath
            ,($isCommon||(!\Kuoqi::$app->route))?"":('/'.\Kuoqi::$app->route->m)
            ,$fn
        );

        $file = new FileDriver($filename);
        $code = $file->getContent();
        $tmp = explode('::::::',$code);
        if(count($tmp)==2){
            $time = $tmp[1];
            if($time>time()){
                return  unserialize($code);
            }else{
                //已过期，触发删除
                self::delCache($name,$cpath,$isCommon );
                return null;
            }
        }else{
            return null;
        }
    }
    /**
     * 删除缓存文件
     * @param $name 缓存文件名
     * @param string $cpath 额外路径信息
     * @param bool $isCommon 是否公共文件
     * @return bool
     */
    static public function delCache($name,$cpath = '',$isCommon = false){
        $fn = md5($name);
        $filename = sprintf('{@APP}/%s/%s%s%s/%s.cache'
            ,Config::get("RUNTIME_PATH")
            ,Config::get("CACHE_PATH")
            ,$cpath
            ,($isCommon||(!\Kuoqi::$app->route))?"":('/'.\Kuoqi::$app->route->m)
            ,$fn
        );
        $file = new FileDriver($filename);
        return $file->exist()&&$file->delete();
    }
}
