<?php
namespace Kuoqi;

use Kuoqi;
use Kuoqi\Error;
use Kuoqi\Params;
use Kuoqi\Route;
use Kuoqi\FileDriver;
use Kuoqi\Cache;

/**
 * @name 核心组键
 * @desc - 用户控制器基础的实现内容,本模块所有信息都是操作当前注册在kuoqi下面的app的信息来达到目的
 * Class Controller
 * @package Kuoqi
*/
class Controller{
    public function __construct()
    {
    }

    /**
     * 行为执行之前的初始化，作为控制器，这个是控制器的标准，如果没有这个则不配为控制器
     */
    public function beforeBehavior()
    {

    }

    //当前控制器使用的app
    static protected $_app = null;
    static public function useApp($app)
    {
        self::$_app = $app;
    }

    /**
     * 控制器的权限maps
     * @var array
     * protected $action_maps = array(
        'title'=>'管理员管理',
        'list'=>array(
            'index'=>'列表',
            'edit'=>'增加/编辑',
            'del'=>'删除',
         ),
       );
     */
    protected $action_maps = array();

    /**
     * @param $layoutId layout使用公共模板的修复
     */
    protected function layout($layoutId){
        if($layoutId){
            $this->app()->layout_id = $this->app()->completeViewpath($layoutId);
        }else{
            $this->app()->layout_id = false;
        }
    }

    /**
     * 配置参数
     * @param $args
     * @param null $extArgs
     * @return mixed
     */
    protected function assign($args,$extArgs = null){
        if(($extArgs==null)&&(is_string($args))){
            return $this->app()->getAssign($args);
        }else{
            return $this->app()->setAssign($args,$extArgs);
        }

    }
    /**
     * 配置输出完整版数据内容
     * vbase
     * @param string $templateId
     * @param array $templateParams
     * @return string
     * @throws \Kuoqi\Error
     */
    protected function render($templateId = '',$templateParams = array()){
        $fn = \Kuoqi\App\Route::stdView($templateId);
        $this->app()->use_view = true;//开启view模块
        $this->app()->path_viewer = $fn;
        $fileDriver = new FileDriver('{@APP}'.$fn);
        if(!$fileDriver->exist()){
            throw new \Kuoqi\Error("View ".$fn."模板文件不存在，请配置后在使用本功能");
        }
        $filecontent = $fileDriver->getContent();//获取文件内容
        $stream = Interpreter::execCode($filecontent,$this->app()->getAssigns($templateParams));
        $tpid = Cache::setTemplate($stream);
        $stream = Cache::getTemplate($tpid,$this->app()->getAssigns($templateParams));
        \Kuoqi::$logs->log("rander执行","process");
        return trim($stream);
    }

    /**
     * 配置app 最终只输出当前的片段内容
     * vbase
     * @param string $templateId
     * @param array $templateParams
     * @return string
     * @throws \Kuoqi\Error
     */
    protected function renderPartial($templateId = '',$templateParams = array()){
        //首先第一步，补全app路由信息
        $fn = \Kuoqi\App\Route::stdView($templateId);
        $this->app()->layout_id = false;//关闭layout
        $this->app()->use_view = true;//开启view模块
        //尝试返回这个模块的数据信息
        $this->app()->path_viewer = $fn;
        //使用模块进行解析内容片段
        $fileDriver = new FileDriver('{@APP}'.$fn);
        if(!$fileDriver->exist()){
            throw new \Kuoqi\Error("View ".$fn."模板文件不存在，请配置后在使用本功能");
        }
        $filecontent = $fileDriver->getContent();//获取文件内容
        $stream = Interpreter::execCode($filecontent,$this->app()->getAssigns($templateParams));
        $tpid = Cache::setTemplate($stream);
        $stream = Cache::getTemplate($tpid,$this->app()->getAssigns($templateParams));
        return trim($stream);
    }

    //获取当前的JSON的内容转码
    protected function renderJson($templateId = '',$templateParams = array()){
        //首先第一步，补全app路由信息
        $fn = \Kuoqi\App\Route::stdView($templateId,".json");
        $this->app()->layout_id = false;//关闭layout
        $this->app()->use_view = true;//开启view模块
        //尝试返回这个模块的数据信息
        $this->app()->path_viewer = $fn;
        //使用模块进行解析内容片段
        $fileDriver = new FileDriver('{@APP}'.$fn);
        if(!$fileDriver->exist()){
            throw new \Kuoqi\Error("View ".$fn."模板文件不存在，请配置后在使用本功能");
        }
        $filecontent = $fileDriver->getContent();//获取文件内容
        $stream = Interpreter::execCode($filecontent,$this->app()->getAssigns($templateParams));
        $tpid = Cache::setTemplate($stream);
        $stream = Cache::getTemplate($tpid,$this->app()->getAssigns($templateParams));
        //$stream = preg_replace("/\s+/","",$stream);
        $stream = \Kuoqi\Helper::jsonencode(json_decode($stream));
        return trim($stream);
    }

    //配置APP只输出一段代码
    protected function renderText($stream){
        //首先第一步，补全app路由信息
        $this->app()->layout_id = false;//关闭layout
        $this->app()->use_view = true;//开启view模块
        return trim($stream);
    }



    /**
     * 配置输出完整版数据内容
     * vbase
     * @param string $templateId
     * @param array $templateParams
     * @return string
     * @throws \Kuoqi\Error
     */
    protected function display($templateId = '',$templateParams = array()){
        return $this->render($templateId = '',$templateParams = array());
    }

    //直接展示内容信息
    protected function show($msg){ echo sprintf('<pre>%s</pre>',$msg);}
    protected function showSuccess($msg){ echo sprintf('<pre>%s</pre>',$msg);}
    protected function showError($msg){ echo sprintf('<pre style="color:orangered;">%s</pre>',$msg);}

    /**
     * 执行跳转页面的中间页
     * @param string $msg
     * @param int $timer
     * @param null $url
     * @return string
     * @throws \Kuoqi\Error
     */
    protected function success($msg = 'ok',$timer = 0,$url = null){
        if(!Config::get('ROUTE_TEMPLATE_SUCCESS')){
            throw new \Kuoqi\Error("JumpSuccess模板文件不存在，请配置后在使用本功能");
        }
        $route = new Route(Config::get('ROUTE_TEMPLATE_SUCCESS'),false);//首先第一步，补全app路由信息
        //$this->app()->use_layout = false;//关闭layout
        $this->app()->use_view = true;//开启view模块
        $this->app()->path_viewer = $route->path;//尝试返回这个模块的数据信息
        if($route->isSystem()){
            $fileDriver  = new FileDriver('{@ROOT}/'.$route->path.\Kuoqi\Params::$TEMPLATE_EXT);
        }else{
            $fileDriver  = new FileDriver('{@APP}/'.$route->path.\Kuoqi\Params::$TEMPLATE_EXT);
        }
        if(!$fileDriver->isValied()){
            \Kuoqi\Error::process('use '.$route->path);
            throw new \Kuoqi\Error("JumpSuccess模板文件不存在，请配置后在使用本功能");
        }
        $filecontent = $fileDriver->getContent();//获取文件内容
        $stream = Interpreter::execCode($filecontent,array('msg'=>$msg,'timer'=>$timer,'url'=>$url));
        $tpid = Cache::setTemplate($stream);
        $stream = Cache::getTemplate($tpid,array('msg'=>$msg,'timer'=>$timer,'url'=>$url));
        return trim($stream);
    }

    /**
     * 执行失败中间页或者中间json
     * @param string $msg
     * @param int $timer
     * @param null $url
     * @return string
     * @throws \Kuoqi\Error
     */
    protected function error($msg = 'error',$timer = 0,$url = null){
        if(!Config::get('ROUTE_TEMPLATE_ERROR')){
            throw new \Kuoqi\Error("JumpError模板文件不存在，请配置后在使用本功能");
        }
        $route = new Route(Config::get('ROUTE_TEMPLATE_ERROR'),false);//首先第一步，补全app路由信息
        //$this->app()->use_layout = false;//关闭layout

        $this->app()->use_view = true;//开启view模块
        $this->app()->path_viewer = $route->path;//尝试返回这个模块的数据信息
        if($route->isSystem()){
            $fileDriver  = new FileDriver('{@ROOT}/'.$route->path.\Kuoqi\Params::$TEMPLATE_EXT);
        }else{
            $fileDriver  = new FileDriver('{@APP}/'.$route->path.\Kuoqi\Params::$TEMPLATE_EXT);
        }
        if(!$fileDriver->isValied()){
            \Kuoqi\Error::process('use '.$route->path);
            throw new \Kuoqi\Error("JumpSuccess模板文件不存在，请配置后在使用本功能");
        }
        $filecontent = $fileDriver->getContent();//获取文件内容
        $stream = Interpreter::execCode($filecontent,array('msg'=>$msg,'timer'=>$timer,'url'=>$url));
        $tpid = Cache::setTemplate($stream);
        $stream = Cache::getTemplate($tpid,array('msg'=>$msg,'timer'=>$timer,'url'=>$url));
        return $this->app()->isError(trim($stream));
    }

    /**
     * 重定向到某个页面
     * @param $url
     * @param bool $isComplex
     * @throws \Kuoqi\Error
     */
    protected function redirect($url,$isComplex = false){
        $isComplex||$url = \Kuoqi\Helper::url($url);
        if($url==""){
            throw new \Kuoqi\Error("不支持重定向到当前页面，容易造成死循环");
        }
        die(header('location:'.$url));
        //todo--完整方案请使用下面的内容
        //直接输出302重定向
        $this->app()->response->setHeader('Location',$url);
        $this->app()->response->setCode(302);
        //不能直接重定向输出内容，会导致不兼容自己的系统，所有，需要最终结果统一处理
        $this->app()->response->flush();//冲刷内容
        return ;//$this->app()->response->flush302($url);
    }

    /**
     * @param $config
     * @param null $message
     * @param array $data
     * @return string
     * User kuoqi
     * Datetime 2022/3/1 17:41
     * Remark 初始化json返回
     */
    public function jsonReturn($config,$message = NULL,$data = array())
    {
        //配置layout为空
        $this->app()->layout_id = false;
        $this->app()->setFormat('json_string');
        if(is_int($config)||is_string($config)){
            $config = $this->arrReturn($config,$message,$data);
        }
        if(PHP_VERSION>='5.5'){
            $config = json_encode($config,JSON_UNESCAPED_UNICODE);
        }else{
            $config = json_encode($config);
        }
        return $config;
    }

    /**
     * 初始化jsonp
     * createby kuoqi at 2022/8/24 11:06
     * @param $config
     * @param null $message
     * @param array $data
     * @return string
     * @throws \Kuoqi\Error
     */
    public function jsonpReturn($config,$message = NULL,$data = array())
    {
        if(is_int($config)){
            $config = $this->arrReturn($config,$message,$data);
        }
        $data = json_encode($config);
        $callback = I('request.callback','handleResponse','string');
        $string = "{$callback}({$data})";
        return $string;
    }

    /**
     * @param int $status
     * @param string $message
     * @param array $data
     * @return array
     * User kuoqi
     * Datetime 2022/3/1 17:42
     * Remark arr Return
     */
    public function arrReturn($status = 1,$message = '',$data = array())
    {
        $output = array('status'=>$status,'message'=>$message);
        if($status){
            if($status==9){
                $output['url'] = $data;
            }else{
                $output['data'] = $data;
            }
        }
        return $output;
    }

    /**
     * @param $layoutId
     * User kuoqi
     * Datetime 2022/3/1 16:01
     * Remark 初始化layout相关信息
     */

    /**
     * @return bool
     * User kuoqi
     * Datetime 2022/2/28 14:49
     * Remark 判定请求方式是否为get
     */
    public function isGet(){
        return (\Kuoqi\Params::$METHOD=='GET')?true:false;
    }

    /**
     * @return bool
     * User kuoqi
     * Datetime 2022/2/28 14:49
     * Remark 判定请求方式是否为post
     */
    public function isPost(){
        return (\Kuoqi\Params::$METHOD=='POST')?true:false;
    }

    protected function app(){
        return self::$_app;
    }

    /**
     * 执行脚本，脚本位于script目录下
     * @param $script
     * @param array $opts
     */
    protected function script($script,$opts = array()){
        $file = sprintf('{@APP}\%s\%s\script\%s.php'
            ,\Kuoqi\Route::hump2code(Config::get("APP_PATH"))
            ,\Kuoqi\Route::hump2code($this->app()->getModeler())
            ,\Kuoqi\Route::hump2code($script)
        );
        $this->processSet($file,$opts);
    }

    /**
     * 将信息加入到守护进程中
     * @param $file
     * @param array $opts
     */
    protected function processSet($file,$opts = array()){
        $process = new \Kuoqi\Process\GuardProcess();
        $process->pidWithadd($file,$opts);
    }

    /**
     * 退出
     * @throws \Kuoqi\Error
     */
    public function quit($msg){
        $error = new \Kuoqi\Error($msg);
        $error->quit();
    }


    public function __call($name, $arguments)
    {
        throw new \Kuoqi\Error('找不到执行方法['.$name.']');
    }
}