<?php
namespace Kuoqi;
use Kuoqi\Curl\InitMethod;

/**
 * 标准化的一个curl请求
 * Class Curlv2
 * @package Kuoqi
 * url:网络地址。
content_type:内容编码。
http_code:HTTP状态码。
header_size:header的大小。
request_size:请求的大小。
filetime:文件创建的时间。
ssl_verify_result:SSL验证结果。
redirect_count:跳转计数。
total_time:总耗时。
namelookup_time:DNS查询耗时。
connect_time:等待连接耗时。
pretransfer_time:传输前准备耗时。
size_uplpad:上传数据的大小。
size_download:下载数据的大小。
speed_download:下载速度。
speed_upload:上传速度。
download_content_length:下载内容的长度。
upload_content_length:上传内容的长度。
starttransfer_time:开始传输的时间表。
redirect_time:重定向耗时。
 */
class Curl extends \Kuoqi\KuoqiException {

    protected $uri = '';//请求的地址
    protected $method = 'GET';//请求类型
    protected $params = array();//常规的参数信息
    protected $response_type = 'string';//响应的类型
    protected $time_out = 0;//超时时间，设置为0则判定为永不过期
    protected $resource = null;//资源
    protected $_header = array('Content-Type'=>'multipart/form-data');//'Content-Type','multipart/form-data'
    protected $response_buffer = '';//响应的原始数据内容
    protected $isfollwrlocation = true;//是否支持重定向；CURLOPT_FOLLOWLOCATION
    protected $timeout = 0;//超时的值s
    protected $timeout_type = 'MS';//超时的单位
    //是否验证https
    public $verify_host = false;
    public $verify_peer = false;
    //使用useragent
    public $useragent = true;//是否使用用户的基础信息
    public $cookie = true;//是否使用cookie

    public $showRequestHeader = false;//展示请求头信息
    public $showResponseHeader = false;//展示相应头相关信息
    public $request_headinfo = '';//返回的头部相关的信息
    public $response_headinfo = '';//相应的头部信息
    public $resonse_info = array();//请求响应的头部信息


    //尝试打开一个资源流
    public function open(){
        $this->resource = curl_init();
    }

    /**
     * 执行一个请求
     * exec 2024-11-18
     */
    public function exec(){
        if(!$this->resource){
            $this->setError('执行异常，请使用$this->open打开流资源');
            return;
        }
        //设置请求的类型
        $this->execMethod();//初始化method部分
        curl_setopt($this->resource,CURLOPT_CUSTOMREQUEST,$this->getMethod());

        //设置链接的相关信息
        curl_setopt($this->resource,CURLOPT_URL,$this->uri);

        //获取相关的头部的相关数据信息
        $tmp = array();
        foreach ($this->_header as $k=>$v){
            $tmp[] = sprintf('%s: %s',$k,$v);
        }
        curl_setopt($this->resource,CURLOPT_HTTPHEADER,$tmp);

        //是否重定向
        curl_setopt($this->resource,CURLOPT_FOLLOWLOCATION,$this->isfollwrlocation);

        curl_setopt($this->resource,CURLOPT_RETURNTRANSFER,true);//启用是否返回的方式

        //验证https的相关数据信息
        curl_setopt($this->resource,CURLOPT_SSL_VERIFYPEER,$this->verify_host);
        curl_setopt($this->resource,CURLOPT_SSL_VERIFYHOST,$this->verify_peer);

        //使用UserAgent HTTP_USER_AGENT
        if($this->useragent){
            if($this->useragent===true){
                $this->useragent = isset($_SERVER['HTTP_USER_AGENT'])?$_SERVER['HTTP_USER_AGENT']:false;
            }
            if($this->useragent){
                curl_setopt($this->resource, CURLOPT_USERAGENT,$this->useragent);
            }
        }

        //使用cookie
        if($this->cookie){
            if($this->cookie===true){
                //todo--这里再继续研究，这个应该可以使用
            }else{
                curl_setopt($this->resource,CURLOPT_COOKIESESSION,true);
                curl_setopt($this->resource, CURLOPT_COOKIE, $this->cookie);
            }
        }

        //检测是否要携带请求头信息
        if($this->showRequestHeader){
            curl_setopt($this->resource,CURLINFO_HEADER_OUT,true);
            //在curl_exec函数执行后，通过 curl_getinfo($ch,CURLINFO_HEADER_OUT) 来获取curl执行请求的请求数据。
        }
        if($this->showResponseHeader){
            curl_setopt($this->resource, CURLOPT_HEADER, true);
            curl_setopt($this->resource, CURLOPT_NOBODY,false);
            //或者获取设置 curl_setopt($ch, CURLOPT_NOBODY,false);然后对curl_exec获取的值通过\r\n\r\n（响应头后有个空行）进行分割截取第一部分即为响应头信息。
        }
        $this->response_buffer = curl_exec($this->resource);
        if($this->showResponseHeader){
            //这种是尝试请求响应头部的信息
            $list = explode("\r\n\r\n",$this->response_buffer);
            //然后进行最后一个出栈
            $this->response_buffer = array_pop($list);
            $this->response_headinfo = implode("\r\n\r\n",$list);
        }

        if($errcode = curl_errno($this->resource)){
            $this->errcode =$errcode;
            $this->errmsg = "请求异常:".curl_error($this->resource);
        }else{
            $this->resonse_info = curl_getinfo($this->resource);
        }
    }

    //获取请求的相关信息
    public function getRequestHeadinfo(){
        return isset($this->resonse_info['request_header'])?$this->resonse_info['request_header']:'';
    }
    //获取响应的头部信息
    public function getResonseHeadinfo(){
        return $this->response_headinfo;
    }
    public function setResponseType($type){
        return strtoupper($this->response_type = $type);
    }
    //获取响应的类型
    public function getResponseType(){
        return strtoupper($this->response_type);
    }
    //获取响应的内容
    public function getResponse($type = ''){
        if($type){ $this->setResponseType($type);}
        switch ($this->getResponseType()){
            case 'OBJECT':return $this;
            case 'JSON':return $this->responseJson();break;
            case 'STRING':return $this->response_buffer;break;
            default:throw new Error("响应类型[".$this->getResponseType()."]不存在");
        }
    }
    public function responseJson(){
        if(PHP_VERSION>='5.5'){
            return json_decode($this->response_buffer,JSON_UNESCAPED_UNICODE);
        }else{
            return json_decode($this->response_buffer);
        }
    }

    //关闭一个流
    public function close(){
        $this->resource&&curl_close($this->resource);
        $this->resource = null;//清空资源的内容模块
    }

    public function getMethod(){
        $method = strtoupper($this->method);
        if(!in_array($method,array('GET','POST','FORM','JSON','XML'))){ $method = 'GET';}
        return $method;
    }
    //设置请求method
    public function setMethod($method){
        $this->method = $method;
    }

    //设置一个uri,在发起之前设置有效
    public function setUri($uri){
        $this->uri = $uri;
    }
    //设置头部参数信息
    public function setHeader($n,$v){
        $this->_header[$n] = $v;
    }
    public function getParams(){
        return $this->params;
    }
    public function setParams($params){
        return $this->params = $params;
    }
    public function setCookie($cookie){
        $this->cookie = $cookie;
    }
    //获取资源
    public function getResource(){
        return $this->resource;
    }
    public function getUri(){
        return $this->uri;
    }

    function setTimeout($code,$isMS = true){
        $this->timeout = intval($code);
        if($isMS){
            $this->timeout_type = 'MS';
        }else{
            $this->timeout_type = 'S';
        }
    }
    //初始化method带来的相关信息
    public function execMethod(){
        switch($this->getMethod()){
            case 'JSON':InitMethod::methodWithJson($this);break;
            case 'FORM':InitMethod::methodWithForm($this);break;
            case 'POST':InitMethod::methodWithPost($this);break;
            case 'XML':InitMethod::methodWithXml($this);break;
            case 'GET':InitMethod::methodWithGet($this);break;
            default:throw new Error("method[".$this->getMethod().']解析方法不存在');
        }
    }

    static public function SendTo($url = '', $method = '', $params = array(), $rsptype = '',$timeout = 0){
        $ths = new self();
        $url&&$ths->uri = $url;
        $method&&$ths->method = $method;
        $params&&$ths->params = $params;
        $rsptype&&$ths->response_type = strtoupper($rsptype);
        $timeout&&$ths->time_out = floatval($timeout);
        $ths->open();//打开一个基本的传送通道
        $ths->exec();//发送请求
        $ths->close();
        return $ths->getResponse();
    }


}
