<?php
namespace Kuoqi;
use Kuoqi\DbDriver\PdoDriver;

class DbDriver extends \Kuoqi\KuoqiException {
    static public $TARGET_MYSQL = 'Mysql';//目标类型
    static public $TARGET_LIST = array('Mysql');//已经支持的链接方式
    static public $SERVER_LIST = array('Pdo');//已经支持的链接数据库工具
    static public $SERVER_PDO = 'Pdo';//使用pdo的对象进行链接数据库

    public $target = '';//当前的目标服务器类型
    public $server = '';//使用的链接数据库的工具
    public $dbhost = '';//使用IP/TCP,地址
    public $dbname = '';//数据库名
    public $dbport = '';//使用端口号
    public $user = '';//使用数据库用户名
    public $pass = '';//使用数据库链接密码
    public $prefix = '';//数据库表前缀
    public $charset = '';//utf8mb4
    public $runtime = 0;//默认缓存秒数
    public $attributes = array();//绑定额外的属性信息

    public function __construct()
    {
        $this->target = Config::get('DB_TARGET');//设置使用目标类型
        $this->server = Config::get('DB_SERVER');//使用的链接数据库的工具
        $this->dbhost = Config::get("DB_HOST");
        $this->dbport = Config::get("DB_PORT");
        $this->dbname = Config::get("DB_NAME");
        $this->user = Config::get("DB_USER");
        $this->pass = \Kuoqi\Openssl::encode(Config::get("DB_PASS"));//使用安全系数较高的方式，保护密码不会因为意外泄露
        $this->prefix = Config::get("DB_PREFIX");
        $this->charset = Config::get("DB_CHARSET");
        $this->runtime = Config::get("DB_C_TIME");
        $this->attributes = array(
            \PDO::ATTR_ERRMODE,\PDO::ERRMODE_SILENT
        );
    }

    //设置目标服务器的目标数据库类型
    public function target($target = ''){
        if(!in_array($target,self::$TARGET_LIST)){
            $this->target = Config::get('DB_TARGET');
        }
    }
    //设置使用工具
    public function server($server = ''){
        if(!in_array($server,self::$SERVER_LIST)){
            $this->server = Config::get('DB_SERVER');
        }
    }
    //使用链接函数进行一个链接
    public function connect(){
        try{
            $pdo = new DbDriver\PdoDriver();
            $pdo->connect();
            return $pdo;
        }catch(\Kuoqi\Error $err){
            $err->quit();
        }
    }
}
