<?php
namespace Kuoqi;
/**
 * 使用文件模块，这个文件模块更像是使用主机里面文件的样子
 * Class DirDriver
 * @package Kuoqi
 */
class DirDriver extends KuoqiException {
    static $TYPE_DIR = 'DIR';//判定为文件夹的类型
    static $TYPE_FILE = 'FILE';//判定为文件的类型
    //当前操作的目录
    public $dir = '';//文件路径信息
    public $root = '';//当前路劲的根目录
    //初始化文件的时候
    public function __construct($dir = '',$try_create = false)
    {
        $this->root = $this->dir = realpath(\Kuoqi\Route::str2rootpath($dir));
        if($this->isPath()){
            $this->errcode = 0;
        }else{
            //尝试创建目录
            if(@mkdir(\Kuoqi\Route::str2rootpath( $this->root),0777,true)){
                $this->errcode = 0;
            }else{
                $this->errcode = -1;
                $this->errmsg = '文件目录不存在';
            }
        }
    }

    //判定是否为目录，如果是目录则返回当前目录路径，不存在则返回false
    public function isPath($path = ''){
        if(is_dir($this->dir . $path)){
            return $this->dir . $path;
        }else{
            return false;
        }
    }
    //判定为文件
    public function isFile($path = ''){
        if(is_file($this->dir . $path)){
            return $this->dir . $path;
        }else{
            return false;
        }
    }

    //浏览当前目录,简单的获取当前目录的相关信息
    public function scan(){
        $list = scandir($this->dir);
        $arr = array();
        foreach ($list as $v){
            if(in_array($v,array('.','..'))){ continue;}
            $arr[] = $v;
        }
        return $arr;
    }
    //完整版的获取文件目录，可标记当前为文件还是
    //以数组的形式标记当前目录对象列表信息
    public function scanAsArray(){
        $list = $this->scan();
        $lis_path = array();
        $lis_file = array();
        foreach ($list as $v){
            if(in_array($v,array('.','..'))){ continue;}
            if($path = $this->isPath('/'.$v)){
                //目录存在，则返回为目录
                $tmp = array(
                    'path'=>$v,
                    'type'=>'DIR',
                    'type_msg'=>'文件夹',
                    'size'=>'',//大小
                    'filemtime'=>filemtime($path),
                    'is_back'=>($v=='..')?1:0,//判定为返回上级的目录
                    'is_now'=>($v=='.')?1:0,//判定为当前目录
                );
                $lis_path[] = $tmp;
            }else if($file = $this->isFile('/'.$v)){
                //直接判定为文件了，这种情况
                $tmp = array(
                    'path'=>$v,
                    'type'=>'FILE',
                    'type_msg'=>substr(strrchr($v,'.'),1).'文件',
                    'size'=>filesize($file),
                    'filemtime'=>filemtime($file),
                );
                $lis_file[] = $tmp;
            }
        }
        return array_merge($lis_path,$lis_file);
    }

    /**
     * 查找文件
     * @param $name
     * @return bool|DirDriver
     */
    public function find($name){
        $list = $this->scan();
        foreach ($list as $v){
            if($v==$name){
                return new self($this->dir);
            }
        }
        return false;
    }

    /**
     * 已经到顶的不能继续返回啦
     * @return bool
     */
    public function goback(){
        $dir = dirname($this->dir);
        if($dir==$this->dir){
            return false;
        }else{
            $this->dir = $dir;
            return true;
        }
    }

    public function cd($path){
        //判定是否以首斜杠开头
        if(stripos($path,"..")===0){
            //这个判定为当前目录的上级目录
            throw new Error("上级目录的文件夹");
        }else if(stripos($path,".")===0){
            $this->dir = realpath($this->dir.$path);
        }else if(stripos($path,"/")===0){
            $this->dir = realpath(\Kuoqi\Route::str2rootpath('{@APP}'.$path));
        }else if(stripos($path,"\\")===0){
            $this->dir = realpath(\Kuoqi\Route::str2rootpath('{@APP}'.$path));
        }
        else{
            throw new Error("判定为其它目录:".$path);
        }
    }

    /**
     * getDir 2024-05-30 getReal($path = '')的[别名]方法，将在下个版本移除
     * @param string $path
     * @return bool|string
     * @throws Error
     */
    public function getDir($path = ''){
        return $this->getReal($path);
    }

    /**
     * getReal 2024-05-30 获取基于当前目录的一个目录或者文件
     * @param string $path
     * @return bool|string
     * @throws Error
     */
    public function getReal($path = '',$withDir = true){
        //如果判定头部信息跟当前的dir信息一样的话 ，那就制动返回当前头部的信息
        if(stripos($path,"..")===0){
            //这个判定为当前目录的上级目录
            throw new Error("上级目录的文件夹");
        }else if(stripos($path,".")===0){
            $path = $this->dir.$path;
        }else if(stripos($path,"/")===0){
            $path = \Kuoqi\Route::str2rootpath('{@APP}'.$path);
        }else if(stripos($path,"\\")===0){
            $path = \Kuoqi\Route::str2rootpath('{@APP}'.$path);
        }else if(stripos($path,$this->dir)===0){

        }
        else{
            $path = $this->dir .'/'.$path;
        }

        $path =  realpath($path);
        if($withDir){
            return $path;
        }else{
            return str_replace($this->root,"",$path);
        }
    }

    /**
     * getFiles 2024-05-30 获取所有的文件信息
     * @return array
     * @throws Error
     */
    public function getFiles(){
        //获取当前的所有文件，和获取当前目录的所有文件夹
        $list = scandir($this->getDir());
        $list2 = array();
        foreach ($list as $v){
            if(in_array($v,array(".",".."))){ continue;}
            $tmp = $this->getDir("/./".$v);
            if(is_dir($tmp)){
                $list3 = (new PathDriver($tmp))->getFiles();
                $list2 = array_merge($list2,$list3);
                continue;
            }
            $list2[] = $tmp;
        }
        return $list2;
    }

    //循环
    public function eachFiles($_callback = ""){
        $list = scandir($this->getDir());
        foreach ($list as $v){
            if(in_array($v,array(".",".."))){ continue;}
            $tmp = $this->getReal($v);
            if(is_dir($tmp)){
                (new PathDriver($tmp))->eachFiles($_callback);
                continue;
            }else if(is_file($tmp)){
                $_callback&&$_callback(realpath($tmp));
            }else{
                \Kuoqi::error("\Kuoqi\Driver::eachFiles异常 ".$tmp);
            }
        }
    }
}