<?php
namespace Kuoqi;
use Kuoqi\FileDriver\Mime;
use Kuoqi\FileDriver\Reader;

/**
 * Class FileDriver 使用同意的文件信息
 * @package Kuoqi
 */
class FileDriver extends KuoqiException {
    protected $file = '';//完整的文件信息，这个参数只在输入的时候有用过
    protected $isAnalysis = false;//是否已经经过解析
    protected $file_name = '';//文件名的部分
    protected $file_ext = '';//文件名后缀部分
    protected $file_dirname = '';//文件的路径部分信息
    protected $content = '';//内容区域的内容信息
    protected $mime = '';//mime
    public $safe_code = '<?php if(!defined("SAFE_CODE")){header("HTTP/1.0 404 Not Found");die();}?>';
    public function __construct($file = '')
    {
        $this->file = $file;
        $this->fileAnalysis();
    }
    /**
     * 加载一次
     * @return bool
     */
    public function includeOnce(){
        return \Kuoqi::__autoloadfile($this->getFile(false),true);
    }

    /**
     * 加载文件
     * @return bool|mixed
     */
    public function includeMore(){
        return \Kuoqi::__autoloadfile($this->getFile(false),false);
    }
    /**
     * 获取文件的路径信息
     * @param bool $isTrue 是否获取真实路径，默认获取
     * @return mixed|string 返回路径字符串信息
     */
    public function getFile($isTrue = true){
        $file = sprintf('%s/%s%s'
            ,$this->file_dirname
            ,$this->file_name
            ,$this->file_ext
        );
        $file = preg_replace("/\/+/",'/',$file);
        return $isTrue?\Kuoqi\Route::str2rootpath($file):$file;
    }
    /**
     * 获取文件的路径信息
     * @param bool $istrue 是否获取真实路径，默认获取
     * @return string 返回路径字符串信息
     */
    public function getFullfile($isTrue = true){ return $this->getFile($isTrue);}
    /**
     * 判定是否存在，这个判定会判定是否存在的
     * createby kuoqi at 2022/11/15 17:36
     * @return bool
     */
    public function exist(){
        $exist = is_file($this->isAnalysis?$this->getFile():\Kuoqi\Route::str2rootpath($this->file));
        return $exist;
    }

    /**
     * 清空文件的内容
     * clear 2024-11-22
     * @return bool
     */
    public function clear(){
        return @file_put_contents($this->getFullfile(true),"");
    }

    /**
     * 创建一个空文件
     * create 2024-11-22
     * @return bool
     */
    public function create(){
        return $this->save();
    }
    /**
     * 删除文件
     * @return bool
     */
    public function delete(){
        return @unlink($this->getFile());
    }

    /**
     * 静态删除一个文件
     * stdDelete 2024-11-25
     * @param $filename
     * @return bool
     */
    public static function stdDelete($filename){
        return @unlink(\Kuoqi\Route::str2rootpath($filename));
    }

    //basename，包括文件基础与后缀信息
    public function setBasename($basename = ''){
        $code = strrchr($basename,'.');
        if($code){
            $this->file_name = substr($basename,0,-strlen($code));
            $this->file_ext = $code;
        }
    }

    /**
     * 获取文件的内容信息
     * @param bool $isCache
     * @return mixed|null
     */
    public function getContent($isCache = true){
        if($isCache&&$this->content){
            return $this->content;
        }
        $fname = $this->getFile();
        if(is_file($fname)&&is_readable($fname)){
            $this->errcode = 0;
            return $this->content = str_replace($this->safe_code,'',file_get_contents($fname));
        }else{
            $this->errmsg = 'file '.$this->getFile().' can not exise or readabled';
            $this->errcode = -1;
            return null;
        }
    }
    public function getFullpath($isTrue = false){
       return $this->getPath($isTrue);
    }

    /**
     * 获取文件路径
     * @return string
     */
    public function getPath($isTrue = false){
        return $this->getDir($isTrue);
    }
    //获取文件的路径
    public function getDir($isTrue = false){
        $file = $this->file_dirname;
        return $isTrue?\Kuoqi\Route::str2rootpath($file):$file;
    }

    /**
     * 获取当前的文件名
     * getName 2024-10-07
     * @param bool $withExt
     * @return string
     */
    public function getName($withExt = true){
        return $withExt?($this->file_name.$this->file_ext):$this->file_name;
    }

    /**
     * 获取文件后缀名
     * @return string
     */
    public function getExt(){
        return $this->file_ext;
    }
    //检测并创建文件路径信息
    public function checkPath(){
        if(!(is_dir($this->getPath(true))||@mkdir($this->getPath(true),0777,true))){
            $this->errcode = -1;
            $this->errmsg = 'dir '.$this->getPath().'] not writable';
            return false;
        }
        return true;
    }
    //设置content
    public function setContent($strem){
        $this->content = $strem;
    }

    /**
     * 保存content内容进入到文件
     * save 2024-11-22
     * @return bool
     */
    public function save(){
        if(!$this->checkPath()){ return false;}
        if(@file_put_contents($this->getFullfile(),$this->safe_code.$this->content)===false){
            //写入数据失败，请检查文件权限
            $this->errcode = -1;
            $this->errmsg = 'file '.$this->getFile().'] not writable';
            return false;
        }
        return true;
    }
    //直接不使用读取的方式进行文件追加
    public function append($content){
        $handle = fopen($this->getFile(true),"a");
        fwrite($handle,$content);
        fclose($handle);
    }

    /**
     * 以php的方式解析
     * @param $args
     * @param bool $isOnce
     * @return bool|string
     */
    public function readAsphp($args,$isOnce = false){
        $file = $this->getFile(true);
        ob_start();
        foreach($args as $k=>$v){$$k=$v;}
        if(is_file($file)){
            $isOnce?include_once($file):include($file);
            $this->is_valied = true;
        }else{
            $this->errcode = -1;
            $this->errmsg = $file.' is non-existes';
            return false;
        }
        return ob_get_clean();
    }
    //以json的方式读取文件
    public function readAsJson(){
        $stream = $this->getContent(true);
        //将信息压缩转换
        $stream = str_replace("\n","",$stream);
        $stream = preg_replace("/\s+/","",$stream);
        $stream = str_replace(",]","]",$stream);
        $stream = str_replace(",}","}",$stream);
        $config = json_decode($stream,true);
        return $config;
    }
    //以php的方式解析
    public function readWithphp($args,$isOnce = false){
        return $this->readAsphp($args,$isOnce);
    }
    //已js的方式阅读
    public function readAsjs(){
        $reader = new Reader();
        $reader->setContext($this->getContent());
        $reader->readAsjs();
    }
    //以css的方式阅读
    public function readAscss(){
        $reader = new Reader();
        $reader->setContext($this->getContent());
        $reader->readAscss();
    }
    public function readAsmp4(){
        $reader = new Reader();
        $reader->setContext($this->getContent());
        $reader->readAsmp4();
    }
    /**
     * 转换到阅读模式
     * @return Reader
     */
    public function toReader(){
        $reader = new Reader();
        $reader->setContext($this->getContent());
        return $reader;
    }
    //文件的基础解析
    final function fileAnalysis(){
        $this->isAnalysis = true;
        $this->file_dirname = dirname($this->file);
        $bf = basename($this->file);
        $code = strrchr($bf,'.');
        if($code){
            $this->file_name = substr($bf,0,-strlen($code));
            $this->file_ext = $code;
        }else{
            $this->file_name = $bf;
            $this->file_ext = "";
        }
    }

    //下载文件
    public function download(){
        ob_end_clean();
        header('content-type:application/octet-stream');
        header(sprintf('Content-Disposition: attachment; filename="%s%s";',$this->file_name,$this->file_ext));
        header("Content-Transfer-Encoding: binary");
        echo $this->getContent(false);
        die();
    }

    //文件移动
    public function moveTo($nfpath,$nfname = ''){
        if(!$nfname){ $nfname = $this->getFilename();}
        $file = $this->getFile(true);
        //获取到当前的文件内容
        $this->getContent(false);
        $this->file_dirname = $nfpath;
        $this->safe_code="";
        $this->setBasename($nfname);
        if($this->save()){
            unlink($file);
            return true;
        }else{
            return false;
        }
    }
    //
    public function getFilename(){
        return $this->file_name.$this->file_ext;
    }

    public function getSize(){
        return filesize($this->getFile(true));
    }

    /**
     * 获取文件的MD5
     * getMd5 2024-11-25
     * @param bool $use_cache
     * @return string
     */
    public function getMd5($use_cache = true){
        if($this->content){
            return md5($this->content);
        }else{
            return md5_file($this->getFullfile(true));
        }
    }

    public function flushTofile(){
        $stream = $this->getContent();
        header('content-type:'.$this->getMime());
        die($stream);
    }
    public function getMime(){
        if($this->mime){
            return $this->mime;
        }
        $ext = strrchr($this->getFilename(),'.');
        return $this->mime = Mime::ext2mime($ext);
    }

    /**
     * 静态获取文件的后缀名
     * @param $filename
     * @return string
     */
    public static function stdExt($filename ,$defext=".tmp"){
        $code = strrchr($filename,'.');
        if($code===false){
            return ".tmp";
        }
        return $code;
    }
}