<?php
namespace Kuoqi;
/**
 * 配置ftp的客户端
 * Class Ftpserver
 * @package Kuoqi
 * 内部转换过程中，需要文件夹以/作为开头和结尾
 */
class Ftpserver extends KuoqiException
{
    static $MODE_PASV = 1;//被动模式
    static $MODE_PORT = 2;//主动模式

    public $buffer = null;//服务地址
    protected $loc_root = '';//本地文件夹根目录
    protected $loc_path = '';//本地当前使用路基
    protected $ftp_path = '';//服务器当前使用路基
    public function __construct(){}

    //执行链接
    public function connect($ftp_server,$port = '21'){
        if($this->buffer = ftp_connect($ftp_server,$port)){
            return true;
        }else{
            $this->errcode = -1;
            $this->errmsg = '连接ftp服务失败：'."Couldn't connect to $ftp_server";
            return false;
        }
    }
    //登录一个ftp服务器
    public function login($ftp_user,$ftp_psd){
        if(ftp_login($this->buffer,$ftp_user,$ftp_psd)){
            return true;
        }else{
            $this->errcode = -1;
            $this->errmsg = '登录ftp服务失败：'."Couldn't connect as {$ftp_user}";
            return false;
        }
    }

    //关闭一个服务
    public function close(){
        ftp_close($this->buffer);
        $this->buffer = null;
    }
    //设置本地文件夹的根目录
    public function setRoot($loc_root){
        $this->loc_root = \Kuoqi\Route::str2rootpath($loc_root);
    }
    //设置模式
    public function setMode($mode){
        ftp_pasv($this->buffer,($mode==self::$MODE_PASV)?true:false);
    }

    /**
     * 获取本地的路径信息
     * @param string $ext_path 需要使用的额外路径名，不包含最前面的/
     * @param bool $is_root 是否返回真实路径
     * @return string 返回路径字符串信息
     */
    public function getLocpath($ext_path = '',$is_real = false){
        $str = $this->loc_path.($ext_path?($ext_path):"");
        if($is_real){
            $str =  $this->loc_root.$str;
        }
        return preg_replace('/\/+/','/',$str);
    }

    /**
     * 跳转到某个路径下，同步跳转服务端与本地目录
     * @param $path 跳转的路径信息，以{@loc_root}之下的绝对路径
     * @return bool 是否跳转成功
     */
    public function cdDir($path){
        //跳转这个之前，会返回服务器跟路径进行使用
        ftp_chdir($this->buffer,'/');
        if(@ftp_chdir($this->buffer,$path)){
            $this->loc_path = $this->ftp_path = $path;//同步路径信息
            return true;
        }else{
            $this->errcode = -1;
            $this->errmsg = "切换FTP目录失败";
            return false;
        }
    }
    //在当前目录之下，跳转到下一级目录
    public function cd($path,$try_create = false){
        if(@ftp_chdir($this->buffer,$path)){
            $this->loc_path .= $path.'/';
            $this->ftp_path .= $path.'/';
            return true;
        }else{
            if($try_create){
                if($this->mkdir($path)){
                    return $this->cd($path);
                }else{
                    return false;
                }
            }else{
                $this->errcode = -1;
                $this->errmsg = "切换FTP目录失败";
                return false;
            }
        }
    }
    //返回上一级目录
    public function goBack(){
        $this->loc_path = dirname($this->loc_path).'/';
        $this->ftp_path = dirname($this->ftp_path).'/';
        ftp_cdup($this->buffer);
    }
    //创建远程目录信息
    public function mkdir($path){
        if( @ftp_mkdir($this->buffer,$path)){
            return true;
        }else{
            $this->errcode = -1;
            $this->errmsg = "创建目录失败";
            return false;
        }
    }

    /**
     * 以相对完整的数组信息返回当前的文件列表
     * @return array
     * @throws Error
     */
    public function scanAsArray(){
        return FtpServer\Tools::explainDirlist($this->buffer,'.');
    }
    //查看文件的最后更新时间
    public function mdtime($filename){
        return ftp_mdtm($this->buffer,$this->ftp_path.$filename);
    }
    //上传一个文件或者目录
    public function put($path){
        //判定文件是否合法s
        if(preg_match('/\\\\\/\:\*\?\"\<\>\|/',$path)){
            $this->errcode = -1;
            $this->errmsg = '文件或文件夹不能包含以下字符：\\/:*?"<>|';
            return false;
        }
        //获取本地的文件信息
        //var_dump($path,$this->loc_path);exit();
        $realpath = $this->getLocpath('/'.$path,true);
        if(is_file($realpath)){
            //上传文件
            //$ftp = $this->ftp_path.$path;
            if( @ftp_put($this->buffer,$path,$realpath,FTP_BINARY)){
                return true;
            }else{
                $this->errcode = -1;
                $this->errmsg = "上传文件失败";
                return false;
            }
            return true;
        }else if(is_dir($realpath)){
            //上传目录
            $this->cd($path,true);//跳转到下一级目录
            if(!$this->isValied()){
                $this->errcode = -1;
                $this->errmsg = '上传'.$this->loc_path.'/'.$path.'失败';
                return false;
            }
            //列出这个目录下的所有文件信息，然后进行一个递归
            \Kuoqi::$logs->log($this->getLocpath('',true));
            $dirDriver = new \Kuoqi\DirDriver($this->getLocpath('',true));

            //循环当前的文件夹信息
            foreach ($dirDriver->scan() as $item){
                $this->put($item);//冲刷一个数据
                //执行一次之后就要重置path路径信息
            }
            $this->goBack();//返回本级目录
            return true;
        }else{
            throw new Error("FTP异常：".$this->getLocpath($path));
        }
    }

    /**
     * 传送模式，只能为 FTP_ASCII（文本模式）或 FTP_BINARY（二进制模式）。
     * @param $path 路径信息
     * @return bool 返回成功或者失败
     */
    public function get($path){
        //传送模式，只能为 FTP_ASCII（文本模式）或 FTP_BINARY（二进制模式）。
        //查看本地是否有这个路径，没有则创建路径信息
        $locpath = $this->getLocpath('/'.$path,true);
        if(@ftp_get($this->buffer,$locpath,$path,FTP_BINARY)){
            return true;
        }else{
            $this->errmsg = '下载失败';
            $this->errcode = -1;
            return false;
        }
    }
    //推送信息到服务器中
    public function push($path = ''){
        $locfile = $this->getLocpath($path,true);
        //获取到本地的文件列表
        $filetime = filemtime($locfile);
        if(is_file($locfile)){
            //如果判定为文件，然后测试文件的状态
            $time = $this->mdtime($path);
            if($time==-1){
                //文件不存在，然后这个则直接上传
                //var_dump($path);exit();
                \Kuoqi::warn("上传文件:".$locfile);
                ftp_put($this->buffer,$path,$locfile,FTP_BINARY);
            }else if($time<$filetime){
                //文件的相关信息
                //文件有更新，然后这个进行更新
                \Kuoqi::warn(PHP_EOL."文件更新:".$locfile);
                ftp_put($this->buffer,$path,$locfile,FTP_BINARY);
            }
        }else if(is_dir($locfile)){
            //判断远程是否有这个文件夹，如果没有则尝试创建
            //切换到新的FTP目录
            $this->cd($path,true);
            \Kuoqi::warn(".",false);
            $list = scandir($locfile);
            foreach ($list as $it){
                if(in_array($it,array('.','..'))){ continue;}
                if(preg_match('/^\./',$it)){
                    continue;
                }
                $this->push($it);
            }
            $this->goBack();
        }else{
            throw new Error("FTP异常：".$this->getLocpath($path));
        }
    }
    //拉取服务器信息到本地
    public function pull(){}

    /**
     * * 初始化一个FTP链接及登录
     * @param $ftp_server 服务地址
     * @param $ftp_port 服务端口
     * @param $ftp_user 服务用户名
     * @param $ftp_psd 服务密码
     * @return Ftpserver
     */
    static function init($ftp_server,$ftp_port,$ftp_user,$ftp_psd){
        $server = new self();
        $server->connect($ftp_server,$ftp_port);
        $server->login($ftp_user,$ftp_psd);
        //$server->connect($ftp_server,$ftp_port)&&$server->login($ftp_user,$ftp_psd);
        return $server;
    }

}	