<?php
namespace Kuoqi;

use Kuoqi;
class Helper
{
    static protected $_app = null;
    static public function useApp($app)
    {
        self::$_app = $app;
    }
    static function cache($cname,$cval = '',$ctime = '0',$cpath = '',$isCommon = false){
        if($cval===''){
            return Cache::getCache($cname,$cpath,$isCommon);
        }else{
            return Cache::setCache($cname,$cval,$ctime,$cpath,$isCommon);
        }
    }
    /**
     * 数字转字符串
     * code2size 2024-08-22
     * @param $code
     * @return string
     */
    static function code2size($code){
        $code = floatval($code);
        $unit = 'B';
        if($code> 1024){
            $code = $code / 1024;
            $unit = 'KB';
            if($code>1024){
                $code = $code / 1024;
                $unit = 'MB';
                if($code > 1024){
                    $code = $code / 1024;
                    $unit = 'GB';
                }
            }
        }
        return sprintf('%.2f%s',$code,$unit);
    }
    //判断对错的函数
    static function determine($code,$keySure,$keyErr){
        if($code){
            return $keySure;
        }else{
            return $keyErr;
        }
    }
    static function getCache($cname,$cpath = '',$isCommon = false){
        return Cache::getCache($cname,$cpath,$isCommon);
    }
    //获取数据输入的助手函数
    static function getParam($param = '',$default = NULL,$delFunc = NULL,$isOpt = false){
        $client = Input::initParam($param,$default,$delFunc,$isOpt);
        return Input::outputParam($client);
    }

    static function setCache($cname,$cval = '',$ctime = '0',$cpath = '',$isCommon = false){
        return Cache::setCache($cname,$cval,$ctime,$cpath,$isCommon);
    }

    static function config($n,$v=null){
        if($v==null){
            return \Kuoqi\Config::get($n);
        }else{
            return \Kuoqi\Config::set($n,$v);
        }
    }

    /**
     * 设置一个参数
     * @param $n
     * @param null $v
     * @return mixed
     */
    static function setConfig($n,$v=null){
        return \Kuoqi\Config::set($n,$v);
    }

    /**
     * 获取一个参数
     * @param $n
     * @return null
     */
    static function getConfig($n){
        return \Kuoqi\Config::get($n);
    }

    /**
     * 链接数据库的操作
     * @param string $isPrefix 数据表名
     * @param bool $dateTable 数据表的别名
     * @return DbDriver\PdoDriver
     * @throws Error
     */
    static function Mysql($isPrefix = '',$asName = false){
        $driver = new \Kuoqi\DbDriver\PdoDriver();
        $driver->connect();
        if($asName){
            $driver->table($isPrefix,$asName);
        }elseif($isPrefix){
            $driver->table($isPrefix);
        }
        return $driver;
    }

    /**
     * 获取手机号码
     * phone 2024-08-13
     * @param $phone
     * @return null|string|string[]
     */
    static function phone($phone){
        return preg_replace('/^(\d{3})(\d+)(\d{4})$/','$1****$3',$phone);
    }

    /**
     * url跳转
     * @param null $route
     * @param array $query
     * @param bool $isComplex
     * @return string
     * @throws Error
     */
    static function url($route = NULL,$query = array(),$isComplex = false){
        $url = new Url($route,$query);
        $link = $url->tolink($isComplex);
        return $link;
    }

    static function password_encode($str){
        return \Kuoqi\Format::getPassword($str);
    }

    //删除一个session数据
    static public function delSession($sn){ return \Kuoqi\Session::del($sn);}
    /**
     * 使用模板
     * @param $layoutId
     * @param array $tplArgs
     * @return string
     * @throws Error
     */
    static public function useTemplate($layoutId,$tplArgs = array()){
        $fn = self::$_app->completeViewpath($layoutId);
        $fileDriver = new FileDriver('{@APP}'.$fn);
        if(!$fileDriver->exist()){
            throw new \Kuoqi\Error("View ".$fn."模板文件不存在，请配置后在使用本功能");
        }
        $filecontent = $fileDriver->getContent();//获取文件内容
        $stream = Interpreter::execCode($filecontent,self::$_app->getAssigns($tplArgs));
        $tpid = Cache::setTemplate($stream);
        $stream = Cache::getTemplate($tpid,self::$_app->getAssigns($tplArgs));
        return trim($stream);
    }
    //设置curl请求
    static public function curl($url = '', $method = '', $params = array(), $rsptype = '',$timeout = 0){
        return \Kuoqi\Curl::SendTo($url, $method, $params, $rsptype,$timeout);
    }
    static public function getNumber($priex = ''){
        return $priex .''.date('YmdHis').rand(100,999);
    }
    /**
     * jsonencode
     * @param $stream
     * @return string
     */
    static public function jsonencode($stream){
        if(PHP_VERSION>='5.5'){
            $stream = json_encode($stream,JSON_UNESCAPED_UNICODE);
        }else{
            $stream= json_encode($stream);
        }
        return $stream;
    }
    static public function num2letter($num){
        $num = intval($num);
        $code = 26;
        $str = "";
        while ($num>0){
            $num--;
            $str = chr($num % $code + 65).$str;
            $num = intval($num / $code);
        }
        return $str;
    }
    /**
     * 获取到用户的IP地址
     * IP 2024-07-23
     * @return array|false|string
     */
    static public function IP(){
        $IPaddress='';
        if(isset($_SERVER)){
            if(isset($_SERVER["HTTP_X_FORWARDED_FOR"])){
                $IPaddress = $_SERVER["HTTP_X_FORWARDED_FOR"];
            }else if(isset($_SERVER["HTTP_CLIENT_IP"])){
                $IPaddress = $_SERVER["HTTP_CLIENT_IP"];
            }else{
                $IPaddress = $_SERVER["REMOTE_ADDR"];
            }
        }else{
            if(getenv("HTTP_X_FORWARDED_FOR")){
                $IPaddress = getenv("HTTP_X_FORWARDED_FOR");
            }else if(getenv("HTTP_CLIENT_IP")) {
                $IPaddress = getenv("HTTP_CLIENT_IP");
            }else{
                $IPaddress = getenv("REMOTE_ADDR");
            }
        }
        return $IPaddress;
    }
    /**
     * 判断是否为移动端
     * isMobile 2024-08-22
     * @return bool
     */
    public static function isMobile(){
        if(stristr($_SERVER['HTTP_VIA'],"wap")){
            return true;
        }elseif(strpos(strtoupper(@$_SERVER['HTTP_ACCEPT']),"VND.WAP.WML") > 0){
            return true;
        }elseif(preg_match('/(blackberry|configuration\/cldc|hp |hp-|htc |htc_|htc-|iemobile|kindle|midp|mmp|motorola|mobile|nokia|opera mini|opera |Googlebot-Mobile|YahooSeeker\/M1A1-R2D2|android|iphone|ipod|mobi|palm|palmos|pocket|portalmmm|ppc;|smartphone|sonyericsson|sqh|spv|symbian|treo|up.browser|up.link|vodafone|windows ce|xda |xda_)/i', $_SERVER['HTTP_USER_AGENT'])){
            return true;
        }else
        {
            return false;
        }
    }
    /**
     * 判断是否为PC电脑
     * isPC 2024-08-22
     * @return bool
     */
    public static function isPC(){
        if(stristr($_SERVER['HTTP_VIA'],"wap")){
            return false;
        }elseif(strpos(strtoupper(@$_SERVER['HTTP_ACCEPT']),"VND.WAP.WML") > 0){
            return false;
        }elseif(preg_match('/(blackberry|configuration\/cldc|hp |hp-|htc |htc_|htc-|iemobile|kindle|midp|mmp|motorola|mobile|nokia|opera mini|opera |Googlebot-Mobile|YahooSeeker\/M1A1-R2D2|android|iphone|ipod|mobi|palm|palmos|pocket|portalmmm|ppc;|smartphone|sonyericsson|sqh|spv|symbian|treo|up.browser|up.link|vodafone|windows ce|xda |xda_)/i', $_SERVER['HTTP_USER_AGENT'])){
            return false;
        }else
        {
            return true;
        }
    }
    /**
     * 是否间隔提交
     * isSubmit 2024-08-22
     * @param $timer
     * @return bool
     */
    public static function isSubmit($timer){
        if($hittime = self::getSession("_issubmit")){
            if($hittime+$timer/1000>time()){
                return false;
            }
        }
        self::setSession("_issubmit",time());
        return true;
    }
    public static function isWX(){
        if(preg_match('/(MicroMessenger)/',$_SERVER['HTTP_USER_AGENT'])){
            return true;
        }
        else
        {
            return false;
        }
    }

    public static function session($sn,$sv){
        if($sv){
            //设置session
            return \Kuoqi\Session::set($sn,$sv);
        }else{
            //获取session
            return \Kuoqi\Session::get($sn);
        }
    }

    /**
     * 设置session
     * setSession 2024-10-21
     * @param $sn
     * @param $sv
     * @return mixed|void
     */
    public static function setSession($sn,$sv){ return \Kuoqi\Session::set($sn,$sv);}
    /**
     * 获取session
     * createby kuoqi at 2022/8/24 11:03
     * @param $sn
     * @return null
     */
    public static function getSession($sn){ return \Kuoqi\Session::get($sn);}
    //执行token的创建
    static public function tokenCreate($token){
        return \Kuoqi\Token::create($token);
    }
    //读取token
    static public function tokenRead($token){
        return \Kuoqi\Token::read($token);
    }

    /**
     * 价格的encode
     * price 2024-08-29
     * @param $code
     * @return string
     */
    public static function price_encode($code){
        return sprintf('%.2f',intval($code)/100);
    }

    public static function base64encode($stream){
        return \Kuoqi\Secret::base64_encode($stream);
    }
    public static function base64decode($stream){
        return \Kuoqi\Secret::base64_decode($stream);
    }

    public static function byteDecode($code){
        $unit = "Byte";
        $code = $code;
        if($code/1024>1){
            $unit = "KB";
            $code = sprintf("%.2f",$code / 1024);
            if($code/1024>1){
                $unit = "MB";
                $code = sprintf("%.2f",$code / 1024);
                if($code/1024>1){
                    $unit = "GB";
                    $code = sprintf("%.2f",$code / 1024);
                }
            }
        }
        return $code.$unit;
    }
    public static function byteEncode($code){
        return $code;
    }

    /**
     * 缩略图的相关处理信息
     * imgthumb 2024-10-06
     * @param $link
     * @param $w
     * @param $h
     * @param int $code
     * @return mixed
     */
    public static function imgthumb($link,$w,$h,$code = 0){
        $path = "/runtime/imgthumb/_".$w."_".$h."_".$code."/".md5($link)."_".$w."_".$h.'_'.$code.FileDriver::stdExt($link);
        $fileDriver = new FileDriver("{@HOME}".$path);
        if($fileDriver->exist()){
            return \Kuoqi\Route::str2rootpath("{@WEB}".$path);
        }else{
            $thumb = new Kuoqi\FileDriver\ImageThumb();
            $thumb->inputWithPath("{@HOME}".$link);
            $thumb->setSize($w,$h);
            $thumb->mode($code);
            $thumb->toFile("{@HOME}".$path);
            if($thumb->isValied()){
                return \Kuoqi\Route::str2rootpath("{@WEB}".$path);
            }else{
                return $link;
            }
        }
    }
}