<?php
namespace Kuoqi;
class Image extends \Kuoqi\KuoqiException {
    protected $_w = 1;
    protected $_h = 1;
    protected $image = null;//当前的图片信息
    protected $tmp_path = '{@APP}/runtime/temp/image';

    public function __construct($w,$h)
    {
        $this->_w = $w;
        $this->_h = $h;
        $this->image = imagecreatetruecolor($w,$h);

    }


    public function getWidth(){
        return $this->_w;
    }
    public function getHeight(){
        return $this->_h;
    }

    /**
     *  使用复制功能呢，配置需要复制的大小，然后自动创建一个新的图片区域进行一个回馈给主程序
     * @param int $cx 复制起点的x轴坐标
     * @param int $cy 复制起点的y轴坐标
     * @param int $dw 复制区域的宽度
     * @param int $dh 复制区域的高度
     * @return Image\Gd2
     * @throws Error
     */
    public function copy($cx,$cy,$dw,$dh){
        $img = new Image\Gd2($dw,$dh);
        if(imagecopyresampled($img->getImage(),$this->image,0,0,$cx,$cy,$dw,$dh,$dw,$dh)){
            return $img;
        }else{
            throw new Error("复制失败，请检查");
        }
    }

    /**
     * 获取当前的图片的信息
     * @return null|resource
     */
    public function getImage(){
        return $this->image;
    }

    /**
     * 根据一个图片然后设置图片的基础信息
     * @param $resource
     */
    public function setImage($resource){
        $this->image = $resource;
        $this->_w = imagesx($resource);
        $this->_h = imagesy($resource);
    }

    /**
     * 输出为png图片或者返回png图片
     * @param string $fn
     * @throws Error
     */
    public function toPng($fn = ""){
        if(!$this->image){
            throw new Error("图片获取失败，请检查");
            return ;
        }
        imagealphablending($this->image, false);
        imagesavealpha($this->image, true);
        if($fn){
            $file = new \Kuoqi\FileDriver($fn);
            $file->checkPath();
            if($file->isValied()){
                imagepng($this->image,$file->getFile());
                imagedestroy($this->image);
            }
        }else{
            ob_end_clean();
            header('content-type:image/png');
            imagepng($this->image);
            imagedestroy($this->image);
            die();
        }
    }

    /**
     * 输出jpg或者返回jpg图片
     * @param string $fn
     * @throws Error
     */
    public function toJpg($fn = ""){
        if(!$this->image){
            throw new Error("图片获取失败，请检查");
            return ;
        }
        imagealphablending($this->image, false);
        imagesavealpha($this->image, true);
        if($fn){
            $file = new \Kuoqi\FileDriver($fn);
            $file->checkPath();
            if($file->isValied()){
                imagejpeg($this->image,$file->getFile());
                imagedestroy($this->image);
            }
        }else{
            ob_end_clean();
            header('content-type:image/jpeg');
            imagejpeg($this->image);
            imagedestroy($this->image);
            die();
        }
    }

    /**
     * 输出为一个链接地址
     * @return string
     * @throws Error
     */
    public function toStream(){
        if(!$this->image){
            throw new Error("图片获取失败，请检查");
        }
        ob_clean();
        ob_start();
        imagealphablending($this->image, false);
        imagesavealpha($this->image, true);
        imagepng($this->image);
        imagedestroy($this->image);
        $img = ob_get_clean();//获取缓冲区内容
        $imgInfo = 'data:png;base64,'.chunk_split(base64_encode($img));//转base64
        return $imgInfo;
    }

    /**
     * 输出一个base64字符串
     * @return string|void
     * User kuoqi
     * Datetime 2022/5/12 14:22
     * Remark
     */
    public function toBase64(){
        if(!$this->image){
            throw new Error("图片获取失败，请检查");
        }
        ob_clean();
        ob_start();
        imagealphablending($this->image, false);
        imagesavealpha($this->image, true);
        imagepng($this->image);
        imagedestroy($this->image);
        $img = ob_get_clean();//获取缓冲区内容
        $imgInfo = 'data:png;base64,'.base64_encode($img);//转base64
        return $imgInfo;
    }

    /**
     * 转换为path 输出
     * @param string $path
     * @param string $basename
     * @return string
     * @throws Error
     */
    public function toPath($path = '',$basename = ''){
        if(!$path){
            $path = $this->tmp_path;
        }
        if(!$this->image){
            throw new Error("图片获取失败，请检查");
        }
        $path = Route::str2rootpath($path);
        is_dir($path)||@mkdir($path,777,true);
        $file = $path .'/_'.($basename?$basename:'temp.'.time().rand(1000,9999)).'.image';
        imagealphablending($this->image, false);
        imagesavealpha($this->image, true);
        imagepng($this->image,$file);
        imagedestroy($this->image);
        return $file;
    }

    /**
     * 解析为文件
     * @param $fn
     * @throws Error
     */
    public function toFile($fn){
        if(!$this->image){
            throw new Error("图片获取失败，请检查");
        }
        $file = new \Kuoqi\FileDriver($fn);
        $file->checkPath();
        if($file->isValied()){
            //关闭 alpha 渲染并设置 alpha 标志
            imagealphablending($this->image, false);
            imagesavealpha($this->image, true);
            imagepng($this->image,$file->getFile());
            imagedestroy($this->image);
        }
    }

    //转换为图像
    public function toImage(){
        if(!$this->image){
            throw new Error("图片获取失败，请检查");
        }
        ob_end_clean();
        ob_start();
        imagealphablending($this->image, false);
        imagesavealpha($this->image, true);
        header("content-type:image/png");
        imagepng($this->image);
        imagedestroy($this->image);
        die();
    }

    /**
     * 转换为可以用的url
     * @param $server
     * @param $path
     * @param string $basename
     * @return mixed|void
     * User kuoqi
     * Datetime 2022/4/20 15:03
     * Remark
     */
    public function toUrl($server,$path,$basename = ''){
        $path = Route::str2rootpath($path);
        $file = $path .Params::$DS. '_' .($basename?$basename:basename($this->file));
        if(is_file($file)){
            $web = Config::get('HOME');
            $url =  str_replace($web,$server,$file);
            $url = str_replace(Params::$DS,Params::$LL,$url);
            return $url;
        }
        is_dir($path)||@mkdir($path,777,true);
        if(!$this->image){
            throw new Error("图片获取失败，请检查");
        }

        imagealphablending($this->image, false);
        imagesavealpha($this->image, true);
        imagepng($this->image,$file);
        imagedestroy($this->image);
        $web = Config::get('HOME');
        $url =  str_replace($web,$server,$file);
        $url = str_replace(Params::$DS,Params::$LL,$url);
        return $url;
    }

}