<?php
namespace Kuoqi;
/**
 * 打包模块
 * Class Package
 * @package Kuoqi
 */
class Package extends \Kuoqi\KuoqiException {
    protected $tree = array();
    protected $tree_focus = array();
    protected $probe = "";//当前的路径信息
    protected $package = null;//当前准备的包
    protected $down_filename = "";//设置下载文件名
    public function __construct()
    {
        $this->probe = new Package\Probe();
    }

    /**
     * 跳转到某个文件夹
     * @param $path 跳转的目录
     */
    public function cd($path){
        $this->probe->focus($path);
    }

    /**
     * 增加数据流到某个文件
     * @param $filename
     * @param $filestream
     */
    public function addStream($filename,$filestream){
        $this->expandTree();//拓展当前的树
        $file = "{@APP}/runtime/tmpfile/".MD5($filestream).'.tmp';
        $driver = new \Kuoqi\FileDriver($file);
        $driver->setContent($filestream);
        $driver->save();
        $this->tree_focus[$filename] = $file;
    }

    /**
     * 增加一个空文件夹
     * @param $pathname
     */
    public function addPath($pathname){
        $this->expandTree();
        $this->tree_focus[$pathname] = array();
    }

    /**
     * 增加文件
     * @param $filename
     * @param $file
     */
    public function addFile($filename,$file){
        $this->expandTree();//拓展当前的树
        $this->tree_focus[$filename] = $file;
    }

    /**
     * addGlob 增加所有的文件及文件夹信息
     * @param $filepath
     */
    public function addGlob($filepath){
        $path = \Kuoqi\Route::str2rootpath($filepath);
        $list = scandir($path);
        $focus = $this->probe->getPath();

        //var_dump($path,$focus,$list,'<hr />');
        foreach ($list as $item){
            if(in_array($item,array('.','..'))){ continue;}
            $filename = $path .'/'. $item;
            if(is_file($filename)){
                $this->addFile($item,$filename);
            }else if(is_dir($filename)){
                $this->addPath($item);
                $this->cd($item);
                $this->addGlob($filename);
            }
            $this->cd($focus);
        }
    }

    /**
     * 拓展树
     */
    protected function expandTree(){
        $this->tree_focus = &$this->tree;
        foreach ($this->probe->getList() as $item){
            if(!isset($this->tree_focus[$item])){
                $this->tree_focus[$item] = array();
            }
            $this->tree_focus = &$this->tree_focus[$item];
        }
        return $this->tree_focus;
    }
    //设置文件名
    public function setFilename($downfilename){
        $this->down_filename = basename($downfilename);
    }

    /**
     * 打包为ZIP文件
     * @param string $file
     * @throws Error
     */
    public function toZip($file = ""){
        //展示为一个Zip包
        $this->package = new \Kuoqi\Pack\Zip();
        $this->cd("/");
        $this->toZipContent();
        if($file){
            //todo--下载为具体的文件
            throw new Error("保存为ZIP文件的方式等待开发");
        }else{
            $gfile = $this->package->getFile();
            if($this->package->exise()){
                $this->package->close();
            }else{
                throw new Error("压缩文件为空，请检查");
            }
            $reader = new \Kuoqi\FileDriver\Reader();
            $reader->setContext(file_get_contents(\Kuoqi\Route::str2rootpath($gfile)));
            $reader->downloadAsZip($this->down_filename?$this->down_filename:basename($gfile));
        }
    }
    public function toZipContent(){
        foreach ($this->expandTree() as $name=>$item){
            if(is_string($item)){
                //这个可以判定为一个文件
                $file = \Kuoqi\Route::str2rootpath($item);
                $this->package->putFile($this->probe->getPath($name),$file);
            }else if(is_array($item)){
                //这个是目录文件，然后增加目录
                $this->package->putPath($this->probe->getPath($name));
                if($item){
                    //这里就是一个文件夹
                    //跳转到这个文件夹下面去执行
                    $this->cd($name);
                    $this->toZipContent();
                    $this->cd("..");
                }
            }
        }
    }

}