<?php
namespace Kuoqi;
class PathDriver extends \Kuoqi\KuoqiException {
    protected $_truepath = "";
    protected $root = "";//当前对象的根目录

    /**
     * PathDriver constructor. __construct
     * @param string $path
     */
    public function __construct($path = '.')
    {
        $this->root = $this->_truepath = \Kuoqi\Route::str2rootpath($path);
        if($path = realpath($this->root)){
            $this->root = str_replace("\\","/",$path);
        }
    }

    /**
     * 跳转到一个目录下面,限制到当前的目录下，这个是作为安全的限制
     * cd 2024-11-29
     * @param $path
     * @throws Error
     */
    public function cd($path){
        //判定是否以首斜杠开头
        if(in_array($path,array("..","../"))){
            $this->_truepath = dirname($this->_truepath);
        }else if(in_array($path,array(".","./",""))){
        }else if(in_array($path,array("/","\\"))){
            $this->_truepath = $this->root;
        }else if(stripos($path,"/")===0){
            $path = str_replace("\\",",",$path);
            $path = preg_replace("/\/+/","/",$path);
            $this->_truepath = $this->root.$path;
        }else{
            $path = str_replace("\\","/",$path);
            //判断是否跟root重复，如果重复，则去除root的部分
            if(stripos($path,$this->root)===0){
                $this->_truepath = $path;
            }else if(stripos($this->root,$path)===0){
                //$this->_truepath = $this->_truepath;
            }else{
                $path = str_replace($this->root,"",$path);
                $arr = explode("/",$path);
                foreach ($arr as $item){
                    if(in_array($item,array(".",""))){ continue;}
                    if($item==".."){
                        $this->_truepath = dirname($this->_truepath);
                        continue;
                    }
                    $this->_truepath .= '/'.$item;
                }
            }
        }
        if(!((stripos($this->_truepath,\Kuoqi\Route::str2rootpath('{@ROOT}'))===0)||(stripos($this->_truepath,\Kuoqi\Route::str2rootpath('{@HOME}'))===0))){
            throw new Error("目录无权限:[".$this->_truepath."] root is ".\Kuoqi\Route::str2rootpath('{@ROOT}'));
        }
    }

    /**
     * getDir 2024-05-30 获取当前目录
     * @param string $fname
     * @return string
     */
    public function getDir($fname = ''){
        return  str_replace("\\","/",$this->_truepath.$fname);
    }

    /**
     * 目录是否存在
     * exise 2024-12-08
     * @return bool
     */
    public function exise(){
        if(is_dir($this->getDir())){
            return true;
        }else{
            return false;
        }
    }

    /**
     * 创建目录
     * create 2024-12-08
     * @return bool
     */
    public function create(){
        return @mkdir($this->getDir(),0777,true);
    }

    /**
     * 删除目录
     * delete 2024-11-25
     * @param bool $isloop 是否进行循环删除
     * @return bool
     */
    public function delete($isloop = false){
        if($isloop){
            $files = $this->getFiles();
            foreach ($files as $file){
                fileDriver::stdDelete($file);
            }
            foreach ($this->getPaths() as $path){
                $path = new PathDriver($path);
                $path->delete(true);
            }
            return @rmdir( $this->_truepath);
        }else{
            return @rmdir( $this->_truepath);
        }
    }

    //获取所有的目录文件
    public function getPaths($ignore_paths = array()){
        $list = scandir($this->getDir());
        $list2 = array();
        foreach ($list as $item){
            if(in_array($item,array('.','..'))){ continue;}
            if(!is_dir($this->getDir('/'.$item))){
                continue;
            }
            if(in_array($item,$ignore_paths)){
                continue;
            }
            $list2[]= $item;
        }
        return $list2;
    }

    /**
     * getFiles 2024-05-30 获取所有的文件信息
     * @return array
     */
    public function getFiles(){
        //获取当前的所有文件，和获取当前目录的所有文件夹
        $list = scandir($this->getDir());
        $list2 = array();
        foreach ($list as $v){
            if(in_array($v,array(".",".."))){ continue;}
            $tmp = $this->getDir("/".$v);
            if(is_dir($tmp)){
                $list3 = (new PathDriver($tmp))->getFiles();
                $list2 = array_merge($list2,$list3);
                continue;
            }
            $list2[] = $tmp;
        }
        return $list2;
    }

    //循环
    public function eachFiles($_callback = ""){
        $list = scandir($this->getDir());
        foreach ($list as $v){
            if(in_array($v,array(".",".."))){ continue;}
            $tmp = $this->getDir("/".$v);
            if(is_dir($tmp)){
                (new PathDriver($tmp))->eachFiles($_callback);
                continue;
            }
            $_callback&&$_callback($tmp);
        }
    }

    /**
     * 获取真实的目录
     * getReal 2024-11-29
     * @param string $path
     * @param bool $withDir
     * @return mixed|string
     */
    public function getReal($path = '',$withDir = true){
        $true = realpath ($this->_truepath);
        if($true){
            $this->_truepath = str_replace("\\","/",$true);
        }else{
            return false;
        }
        $opath = $this->getDir();
        $this->cd($path);
        $npath = $this->getDir();
        $this->cd($opath);
        if($withDir){
            return $npath;
        }else{
            return str_replace($this->root,"",$npath);
        }
    }

}