<?php
namespace Kuoqi;

/**
 * Class WebSocket
 * @package Kuoqi\Worker
 */
class Socket extends  \Kuoqi\KuoqiException {
    protected $server_address = '0.0.0.0';
    protected $server_port = '';
    protected $debug = true;
    protected $backlog = SOMAXCONN;//在win32上，如果通过SOMAXCONN，负责套接字的底层服务提供商将把backlog设置为最大合理值。
    protected $sockets = array();
    protected $clients = array();//这个是客户端的列表信息
    protected $master = null;
    protected $tv_sec = null;//是否为阻塞模式，null为无限阻塞。0:为非阻塞模式。用于linux功能更强
    //当前socket绑定的唯一锁文件
    //如果锁文件没有锁信息了，即可判定当前未运行，则可尝试重新触发
    protected $lock_pid = '';

    /**
     * 初始化一个socket的相关操作
     * Socket constructor.
     * @param $address
     * @param $port
     * @param array $option
     */
    public function __construct($address,$port,$option = array()){
        set_time_limit(0);
        ignore_user_abort(0);
        if(isset($option['tv_sec'])){ $this->tv_sec = $option['tv_sec'];}
        $this->server_address = $address;
        $this->server_port = $port;
        if(!($this->master=socket_create(AF_INET, SOCK_STREAM, SOL_TCP))){
            $this->errmsg = "socket_create() failed";
            $this->is_valied = false;
        }
        if(!socket_set_option($this->master, SOL_SOCKET, SO_REUSEADDR, 1)){
            $this->errmsg = "socket_option() failed";
            $this->is_valied = false;
        }
        //如果存在数据，则报告套接字是否停留在socket_close（）上。默认情况下，当套接字关闭时，它会尝试发送所有未发送的数据。在面向连接的套接字的情况下，socket_close（）将等待其对等方确认数据。
        //如果l_onoff为非零且l_linger为零，则在面向连接的套接字的情况下，将丢弃所有未发送的数据，并将RST（重置）发送给对等机。
        //另一方面，如果l_onoff不为零且l_linger不为零，则socket_close（）将阻塞，直到发送完所有数据或l_lingerecurces中指定的时间为止。如果套接字是非阻塞的，那么socket_close（）将失败并返回错误。
        socket_set_option($this->master, SOL_SOCKET, SO_LINGER, array('l_linger' => 0, 'l_onoff' => 1));
        if(!@socket_bind($this->master,$this->server_address,$this->server_port)){
            $this->errmsg = sprintf('KuoqiWorker warning:socket_bind failed on %s:%s with %s'."\n",
                $this->server_address,
                $this->server_port,
                socket_strerror(socket_last_error())
            );
            $this->is_valied = false;
        }
        //最多将有个积压传入连接排队等待处理。如果连接请求到达时队列已满，则客户端可能会收到一个错误，指示ECONREFUNCE，或者，如果基础协议支持传输，则可能会忽略该请求，以便重试成功。
        //传递给backlogparameter的最大数量在很大程度上取决于底层平台。在Linux上，它很遗憾地被截断为SOMAXCONN。在win32上，如果通过SOMAXCONN，负责套接字的底层服务提供商将把backlog设置为最大合理值。在这个平台上，没有标准的规定来找出实际的积压价值。
        if(!socket_listen($this->master,$this->backlog)){
            $this->errmsg = "socket_listen() failed";
            $this->is_valied = false;
        }
        $this->sockets[] = $this->master;

        //$this->lister->serverConstruct();
    }
    /**
     * 开始执行
     */
    public function start(){
        $this->log("welcome to kuoqi socket model");
        $this->run();
    }
    /**
     * 执行运行
     */
    public function run(){
        while(true){
            $this->listen();
        }
    }
    /**
     * 运行监听
     */
    public function listen(){
        if($this->event['listen']){
            $closer = $this->event['listen'];
            $closer($this);
        }
        $socketArr = $this->sockets;
        $write = NULL;
        $except = NULL;
        //自动选择来消息的socket 如果是握手 自动选择主机
        //int socket_select ( array &$read , array &$write , array &$except , int $tv_sec [, int $tv_usec = 0 ] )
        //无线阻塞
        //tv_sec : 0 为非阻塞模式
        //tv_sec : null 为完全非阻塞模式
        if(false ===socket_select($socketArr, $write, $except, $this->tv_sec)){
            $this->error("socket_select() failed, reason: " . socket_strerror(socket_last_error()));
        }
        foreach ($socketArr as $k=>$socket){
            if ($socket == $this->master){
                //如果是关于主机的链接
                $client = socket_accept($this->master);
                //准备链接的时候已经断了，这个不管
                if ($client < 0){
                    $this->log("socket_accept() failed");
                    continue;
                }
                //执行链接一个客户端的操作
                $this->doConnect($client);
            }else{
                //如果是关于客户端的一些链接发送信息类
                $bytes = @socket_recv($socket,$buffer,2048,0);
                if ($bytes == 0){
                    $this->doDisconnect($socket);//这个客户端突然发送一些为空的数据，标识自动断开了，GG
                }else{
                    //登录,$key为对应序号
                    //$key = array_search($socket, $this->sockets);
                    //$this->doMessage($socket, $this->decode($buffer));
                    $this->doMessage($socket, $buffer);
                }
            }
        }
    }
    //执行客户端链接的相关操作
    function doConnect($socket){
        $bytes = @socket_recv($socket,$buffer,2048,0);
        if($bytes==0){
            $this->log("this user no send any authdata");
        }else{
            array_push($this->sockets, $socket);
            //socket_write($socket, $upgrade, strlen($upgrade));
            //$this->postMessage($socket,"欢迎接入幻冰WebSocket");
            if($this->event['connect']){
                $closer = $this->event['connect'];
                $closer($socket,$this);
            }
            $this->log("\n" . $socket . " CONNECTED! on ".date("Y-n-d H:i:s"));
        }
    }

    //发送给客户端的相关信息
    public function postMessage($client,$message,$isText = true){
        if(is_array($message)){
            $message = json_encode($message);
        }
        //$msg = $this->frame($msg);
        if($isText){
            //如果是文本的情况进行压缩
            $message = $this->encode($message,129);
        }else{
            $message = $this->encode($message,130);
        }

        return @socket_write($client, $message, strlen($message));
    }

    //获取sockets列表
    public function getSockets(){
        return $this->sockets;
    }
    //获取所有的客户端的列表
    /**
     * 获取sockets的列表，只能排除某一个客户端
     * @param $exclude_client
     * @return array
     */
    public function getClients($exclude_client = null){
        $array = array();
        foreach ($this->sockets as $item){
            //排除当前进程
            if($item == $exclude_client){ continue;}
            //排除主进程信息
            if($item==$this->master){ continue;}
            $array[] = $item;
        }
        return $array;
    }

    //解码texts
    function decode($buffer) {
        return \Kuoqi\Secret::datapass_decode($buffer);
//        $len = $masks = $data = $decoded = null;
//        if(!$buffer){
//            //如果发现这个不符合基础规范的话，如何处理
//            return ;
//        }
//        $len = ord($buffer[1]) & 127;
//
//        if ($len === 126) {
//            $masks = substr($buffer, 4, 4);
//            $data = substr($buffer, 8);
//        }
//        else if ($len === 127) {
//            $masks = substr($buffer, 10, 4);
//            $data = substr($buffer, 14);
//        }
//        else {
//            $masks = substr($buffer, 2, 4);
//            $data = substr($buffer, 6);
//        }
//        for ($index = 0; $index < strlen($data); $index++) {
//            $decoded .= $data[$index] ^ $masks[$index % 4];
//        }
//        return $decoded;
    }

    function doDisconnect($socket){
        $this->log('client leave');
        $index = array_search($socket, $this->sockets);
        socket_close($socket);
        $this->log($socket . " DISCONNECTED!");
        if ($index >= 0){
            unset($this->sockets[$index]);
            if($this->event['close']){
                $closer = $this->event['close'];
                $closer($socket,$this);
            }
        }
    }

    /**
     * 执行接收到消息
     * @param $client
     * @param $buffer
     * User kuoqi
     * Datetime 2022/5/8 20:28
     * Remark
     */
    function doMessage($client,$buffer){
        if($this->event['message']){
            $closer = $this->event['message'];
            $closer($client,$buffer,$this);
        }else{
            $this->log('get message,please use $this->bind("message",function(){}) to deal this');
        }

        //$this->decode($buffer);
    }

    /**
     * @param $stream
     * @param int $code
     * @return string
     * // 这边仅实现传输文本帧！第一个字节，文本帧 1000 0001 => 129
    // 这边仅实现传输文本帧！第一个字节，二进制流帧 1000 0002 => 130
     */
    public function encode($stream,$code = 129){
        return \Kuoqi\Secret::datapass_encode($stream,$code);
    }
//    public function encode($stream,$code = 129){
//        if(!is_scalar($stream)){
//            \Kuoqi::log("只允许发送标量数据");
//        }
//        $first_byte = chr($code);
//        $len = strlen($stream);
//        if($len<=125)
//        {
//            $second_byte = chr($len);
//            //return $first_byte.chr($len).$stream;
//        }else if($len<=65535){
//            $second_byte = chr(126) . pack('n' , $len);
//            //return $first_byte.chr(126).pack("n", $len).$stream;
//        }else{
//            $second_byte = chr(127) . pack('N' , $len);
//            //$second_byte = chr(127) . pack('xxxN' , $len);
//            //return $first_byte.chr(127).pack("J", $len).$stream;
//        }
//        $encoded_data = $first_byte . $second_byte . $stream;
//        return $encoded_data;
//    }

    protected $event = array(
        'connect'=>'',//客户端链接的时候执行本方法内容
        'listen'=>'',//关联listen函数做一次数据处理的信息返回
        'message'=>'',//接收到消息的时候执行
        'close'=>'',//关闭时候执行
    );

    public function bind($event,$function){
        $this->event[$event] = $function;
    }

    public function error($msg){
        throw new \Kuoqi\Error("event error");
    }

    public function log($msg){
        //echo $msg;
        //throw new \Kuoqi\Error("event log");
        echo $msg.PHP_EOL;
    }

    public function say($msg){
        //todo--say
        //throw new \Kuoqi\Error("event say");
    }



}