<?php
namespace Kuoqi\App;

use Kuoqi;

/**
 * ignore_user_abort();
 * ob_start();
 *
 * // do stuff, generate output
 *
 * // get size of the content
 * $length = ob_get_length();
 *
 * // tell client to close the connection after $length bytes received
 * header('Connection: close');//链接关闭
 * header("Content-Length: $length");
 *
 * // flush all output
 * ob_end_flush();
 * ob_flush();
 * flush();
 */

/**
 * 对于最后的数据的相应的相关处理
 * Class Response
 * Create By Kuoqi on 2024-08-05 12:22
 * @package Kuoqi\App
 */
class Response extends \Kuoqi\KuoqiException {
    static $_CODE_STATUS = array(
        '200'=>'OK',
        '302'=>'REDIRECT',
        '404'=>'NOT Found',
    );
    //返回head的状态代码
    public $code = 200;
    public $status = 'OK';//返回的状态描述
    protected $_headers = array();//响应的头部属性相关信息
    /**
     * @var string
     * User kuoqi
     * Datetime 2022/2/28 11:30
     * Remark 页面输出的类型信息
     */
    protected $content_type = '';
    /**
     * @var string
     * User kuoqi
     * Datetime 2022/2/28 11:30
     * Remark stream输出的内容信息
     */
    public $stream = '';
    protected $app = null;
    public function __construct($app)
    {
        $this->app = $app;
        $this->content_type = 'text/html;charset='.Kuoqi\Config::get('CHARSET');
    }

    /**
     * 设置标头信息
     * @param $n
     * @param null $v
     * @return null
     */
    public function setHeader($n,$v = null){
        if(is_array($n)){
            foreach ($n as $k=>$v){
                $this->setHeader($k,$v);
            }
        }else{
            return $this->_headers[$n] = $v;
        }
    }
    /**
     * 获取标头信息
     * @param $n
     * @return mixed|null
     */
    public function getHeader($n){
        return isset($this->_headers[$n])?$this->_headers[$n]:null;
    }

    /**
     * User kuoqi
     * Datetime 2022/2/28 11:31
     * Remark 解析核心参数到初始化
     * 解析，core,format的字段信息
     */
    public function init(){

        //todo--解析核心字段到输出模式

    }

    /**
     * @param $stream
     * User kuoqi
     * Datetime 2022/2/28 14:07
     * Remark 响应json输出
     */
    public function toJson(){
        //配置请求头信息为json字段
        $this->content_type = 'application/json;charset=UTF-8';
        if(PHP_VERSION>='5.5'){
            $this->stream = json_encode($this->app->action_return,JSON_UNESCAPED_UNICODE);
        }else{
            $this->stream = json_encode($this->app->action_return);
        }
    }

    /**
     * @param $app
     * User kuoqi
     * Datetime 2022/3/1 18:27
     * Remark 响应json string 输出
     */
    public function toJsonString(){
        //配置请求头信息为json字段
        $this->content_type = 'application/json;charset=UTF-8';
        $this->stream = $this->app->action_return;
    }

    //异常的输入
    public function toCustonError(){
        //配置请求头信息为json字段
        //判定是否有layout
        if($this->app->layout_id){
            $fileDriver = new Kuoqi\FileDriver('{@APP}'.$this->app->layout_id);
            $filecontent = $fileDriver->getContent();//获取文件内容
            if(!$fileDriver->isValied()){
                throw new Kuoqi\Error(sprintf("%s文件不存在，请检查后再试"
                    ,$this->app->layout_id
                ));
            }
            $assign = array();
            $assign['_CONTENT_'] = $this->app->action_return;
            $stream = Kuoqi\Interpreter::execCode($filecontent,Kuoqi::$app->getAssigns($assign));
            $tpid = Kuoqi\Cache::setTemplate($stream);
            $this->stream = trim(Kuoqi\Cache::getTemplate($tpid,Kuoqi::$app->getAssigns($assign)));
        }else{
            $this->stream = $this->app->action_return;
        }
    }

    /**
     * 浏览器web输出
     * @param $app
     * @return Error
     * @throws Error
     */
    public function toWeb(){
        //配置请求头信息为json字段
        //判定是否有layout
        if($this->app->layout_id){
            $fileDriver = new Kuoqi\FileDriver('{@APP}'.$this->app->layout_id);
            $filecontent = $fileDriver->getContent();//获取文件内容
            if(!$fileDriver->isValied()){
                throw new Kuoqi\Error(sprintf("%s文件不存在，请检查后再试"
                    ,$this->app->layout_id
                ));
            }
            $assign = array();
            $assign['_CONTENT_'] = $this->app->action_return;
            $stream = Kuoqi\Interpreter::execCode($filecontent,Kuoqi::$app->getAssigns($assign));
            $tpid = Kuoqi\Cache::setTemplate($stream);
            $this->stream = trim(Kuoqi\Cache::getTemplate($tpid,Kuoqi::$app->getAssigns($assign)));
        }else{
            $this->stream = $this->app->action_return;
        }
    }

    /**
     * User kuoqi
     * Datetime 2022/2/28 11:30
     * Remark 响应输出
     */
    public function flush($isReturn = false){
        if($isReturn){
            //将请求信息返回
            $header = "HTTP/1.1 {$this->code} {$this->status}"."\r\n";
            $header .= "Date:".gmdate('D,dMYH:i:sT')."\r\n";
            $header .= "Content-Type:{$this->content_type}"."\r\n";
            $header .= "Content-Length:0"."\r\n";
            foreach ($this->_headers as $k=>$v){
                $header .= sprintf('%s: %s',$k,$v)."\r\n";
            }
            return $header;
        }else{
            $this->app->is_flush = true;
            $flush = new \Kuoqi\flush();
            foreach ($this->_headers as $k=>$v){
                $flush->setHead($k,$v);
            }
            $flush->setContext($this->stream);
            $flush->toFlush();
        }
    }

    /**
     * 设置code
     * @param $code
     */
    public function setCode($code){
        if(isset(self::$_CODE_STATUS[$code])){
            $this->code = $code;
            $this->status = self::$_CODE_STATUS[$code];
        }else{
            $this->code = $code;
            $this->status = "(unset)";
        }
    }
}