<?php
namespace Kuoqi\App;
use Kuoqi\Config;
use Kuoqi\Error;

/**
 * Class Route
 * Create By Kuoqi on 2024-07-15 14:49
 * @package Kuoqi\App
 */
class Route{
    public $dnsdeep = 0;//使用层级0
    public $m = ''; // model的相关信息
    public $c = ''; // controller的相关信息
    public $a = ''; // action的相关信息
    public $extra = array();//其他参数的相关数据信息
    public function __construct()
    {
        $this->m = Config::get('DEFAULT_MOD');
        $this->c = Config::get('DEFAULT_CTL');
        $this->a = Config::get('DEFAULT_ACT');
    }

    /**
     * 将内容转换为VIEW的模版
     * stdView 2024-07-15
     * @param $route
     * @param string $ext
     * @return string
     */
    static public function stdView($route,$ext = ''){
        $cls = new self();
        $cls->setRoute(str_replace("/","\\",$route));
        $actioner = $cls->a;
        foreach ($cls->extra as $k=>$v){
            $actioner .= '/'.$k.($v?('/'.$v):'');
        }
        $viewfile = sprintf('/%s/%s/view/%s/%s%s'
            ,\Kuoqi\Route::hump2code(Config::get("APP_PATH"))
            ,\Kuoqi\Route::hump2code($cls->m)
            ,\Kuoqi\Route::hump2code($cls->c)
            ,\Kuoqi\Route::hump2code($actioner)
            ,$ext?$ext:\Kuoqi\Params::$VIEW_EXT
        );
        return $viewfile;
    }

    //设置路由器的基础信息部分
    public function setRoute($route){
        if($route){
            $route_arr = explode('\\',$route);
            if($route_arr[0]==''){
                //绝对路径的信息
                $this->syncDnsAbsolute($route_arr);
            }else if($route_arr[0]=='@'){
                if(\Kuoqi::$app->route){
                    $this->m = \Kuoqi::$app->route->m;
                    $this->c = \Kuoqi::$app->route->c;
                    $this->a = \Kuoqi::$app->route->a;
                }
                foreach ($route_arr as $k=>$v){
                    if($k==0){ continue;}
                    else if($k==1){$this->a = $v;}
                    else{
                        if(!($k%2)){
                            $this->extra[$v] = isset($route_arr[$k+1])?$route_arr[$k+1]:null;
                        }
                    }
                }
            }else{
                //相对路径信息
                $this->syncDnsRelative($route_arr);
            }
        }else{
            //这个是空的情况，是典型的相对路径的标识，直接判定为相对路径
            $this->syncDnsRelative(array());
        }
    }
    //增加路由信息
    public function addRoute($n,$v){
        if(!$n){
            return;
        }
        if(in_array($n,array('m','c','a'))){
            $this->$n = $v;
        }else{
            $this->extra[$n] = rawurldecode($v);
        }
    }
    //增加路由器部分的数据信息
    public function addRoutes($routes){
        foreach ($routes as $k=>$v){
            $this->addRoute($k,$v);
        }
    }

    final function syncDnsAbsolute($route_arr){

        foreach ($route_arr as $k=>$v){
            if($k==0){
                //这个什么都没有，什么都不做
            }else if($k==1 - $this->dnsdeep){
                $v&&($this->m = $v);
            }else if($k==2 - $this->dnsdeep){
                $v&&($this->c = $v);
            }else if($k==3 - $this->dnsdeep){
                $v&&($this->a = $v);
            }else{
                if( !(($k - $this->dnsdeep) % 2)){
                    $this->extra[$v] = isset($route_arr[$k+1])?$route_arr[$k+1]:null;
                }
            }
        }
    }
    final function syncDnsRelative($route_arr){
        //获取需要处理的数组信息
        if(\Kuoqi::$app->route){
            $this->m = \Kuoqi::$app->route->m;
            $this->c = \Kuoqi::$app->route->c;
            $this->a = \Kuoqi::$app->route->a;
        }
        $count = count($route_arr);
        //设置需要查看的数量信息，count,
        if($count > 3 - $this->dnsdeep){$count = 3 - $this->dnsdeep;}
        foreach ($route_arr as $k=>$v){
            if($k==$count - 3){
                $v&&($this->m = $v);
            }else if($k==$count - 2){
                $v&&($this->c = $v);
            }else if($k==$count - 1){
                $v&&($this->a = $v);
            }else{
                if( ($k  - $this->dnsdeep) % 2){
                    $this->extra[$v] = isset($route_arr[$k+1])?$route_arr[$k+1]:null;
                }
            }
        }
    }


    //初始化dns的相关信息
    final function syncDnsinfo(){
        $dnslist = Config::get('DNS_LIST');
        if($dnslist&&is_array($dnslist)&&isset($dnslist[\Kuoqi\Params::$HOST])){
            $this->_dns_params = $dnslist[\Kuoqi\Params::$HOST];
            if(isset($this->_dns_params['m'])){
                $this->dnsdeep = 1;
                $this->m = $this->_dns_params['m'];
                if(isset($this->_dns_params['c'])){
                    $this->dnsdeep = 2;
                    $this->c = $this->_dns_params['c'];
                    if(isset($this->_dns_params['a'])){
                        $this->dnsdeep = 3;
                        $this->a = $this->_dns_params['a'];
                    }
                }
            }
        }
    }
}