<?php
namespace Kuoqi\Cli;

/**
 * Class Compress
 * @package Kuoqi\Cli
 * php index.php compress cfg -c C:\www\kuoqi\kuoqi-ui\css.json
 */
class Compress extends Behavior {

    protected $document = array(
        "[compress cfg -c config.json]压缩的配置json",
        "-c config 为使用配置文件的形式",
        "-current 当前压缩使用的原始目录",
        "注意两个反斜杠的书写要使用转义，否则压缩过程中会错误匹配为注释信息导致文本压缩异常"
        );
    /**
     * createby kuoqi at 2022/11/11 23:21 初始化start的方法
     * //php kuoqi.php compress js -i kuoqi-php.v1.00.03/application/test.js -o kuoqi-php.v1.00.03/application/test.min.js
     */
    public function start(){
        if($this->command == "cfg"){
            //配置文件的相关信息
            if (!isset($this->params['-c'])) {
                $this->error("请输入-c配置文件信息");
            }
            $fileDriver = new \Kuoqi\FileDriver($this->params['-c']);
            if (!$fileDriver->exist()) {
                $this->error("输入文件不存在，请检查");
            }
            $config = $fileDriver->readAsJson();
            //然后解析css文件对象
            return $this->doAnalysisConfig($config);
        }else{
            //如果命令为直接的一个json文件的这种情况，则判定为当前路径下获取
            if(preg_match('/\.json$/',$this->command)){
                //直接使用配置文件的用例方式
                if(!isset($this->params['current_dir'])){
                    return $this->error("请填写参数[-current_dir]");
                }
                $fileDriver = new \Kuoqi\FileDriver($this->params['current_dir'].'/'.$this->command);
                if (!$fileDriver->exist()) {
                    $this->error("输入文件不存在，请检查");
                }
                $config = $fileDriver->readAsJson();
                if(!$config){
                    $this->error("文件非JSON配置文件模式，请检查");
                }
                return $this->doAnalysisConfig($config);
            }else{
                return $this->help();//展示帮助方法
            }
        }
    }

    /**
     * 执行解析config
     * @param $config
     */
    public function doAnalysisConfig($config){
        if(!isset($config['type'])){
            $this->error("请配置压缩的文本类型[js,css,php.....]");
        }
        if($config['type']=='css'){
            $compress = new Compress\Compresscss();
            $compress->useConfig($config);
            $compress->output();
            $this->error("success");
        }else if($config['type']=='js'){
            $compress = new Compress\Compressjs();
            $compress->useConfig($config);
            if($compress->output()){
                $this->show('output:'.$compress->path($compress->config['output']));
                $this->error("success");
            }else{
                $this->error($compress->getError());
            }
        }else if($config['type']=='web'){
            $compress = new Compress\CompressWeb();
            $compress->useConfig($config);
            if($compress->output()){
                $this->error("success");
            }else{
                $this->error($compress->getError());
            }
        }else{
            $this->error("暂不支持压缩类型[".$config['type']."][js,css,php.....]");
        }
    }
}