<?php
namespace Kuoqi\Cli;
use Kuoqi\Cli\Manage\ClientManage;
use Kuoqi\Cli\Manage\ServerManage;
use Kuoqi\DirDriver;
use Kuoqi\Error;

/**
 * Class Manage
 * @package Kuoqi\Cli
 */
class Manage extends Behavior {
    protected $_webroot = "";
    protected $_path = "";//当前的目录信息
    protected $master = null;//当前客户端的链接程序
    //忽略的目录名单
    protected $_ignores = array();
    protected $_replaces = array();
    protected $document = array(
        "[Manage]",
        "服务器管理的地方的相关信息,pull,push,init"
        );
    /**
     * createby kuoqi at 2022/11/11 23:21 初始化start的方法
     * //php kuoqi.php compress js -i kuoqi-php.v1.00.03/application/test.js -o kuoqi-php.v1.00.03/application/test.min.js
     */
    public function start(){
        if($this->command == "pull"){
            $this->doPreinit();//预初始化
            $this->doPull();
        }else if($this->command == "push"){
            $this->doPreinit();//预初始化
            $this->doPush();
        }else if($this->command == "init"){
            $this->doInit();
        }else if($this->command=="server"){
            $this->doServer();
        }else{
            return $this->help();//展示帮助方法
        }
    }

    //执行预处理
    public function doPreinit(){
        //判断当前路径下是否拥有基础的配置
        $this->checkWeb();
    }

    protected function checkWeb(){
        $dirDriver = new \Kuoqi\DirDriver($this->current_dir);
        $file = $dirDriver->find('.kuoqi');
        if($file){
            //如果存在，则直接使用
        }else{
            while ($dirDriver->goback()){
                $file = $dirDriver->find('.kuoqi');
                if($file){
                    break;
                }
            }
        }
        if($file==false){
            throw new Error("请先初始化一个站点(PS:初始化的站点需要远程服务的支持)");
            return false;
        }
        $this->_webroot = $file->dir;
        //然后获取到忽略文件
        $ignores = $this->_webroot.'/.kuoqi/.ignore';
        $replaces = $this->_webroot.'/.kuoqi/.replace';
        $ignores = file($ignores,FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        $replaces = file($replaces,FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES);
        $this->_ignores = array();
        $this->_replaces = array();
        foreach ($ignores as $item){
            $comm = strstr($item,'#');
            if($comm===false){
                $this->_ignores[] = $item;
            }else if($comm>0){
                $this->_ignores[] = substr($item,0,$comm);
            }
        }
        foreach ($replaces as $item){
            $item = str_replace("\n","",trim($item));
            $comm = strstr($item,'#');
            $code = "";
            if($comm===false){
                $code = $item;
            }else if($comm>0){
                $code = substr($item,0,$comm);
            }
            $code = explode(" ",$code);
            if(count($code)==2){
                $this->_replaces[$code[0]] = $code[1];
            }
        }
    }

    public function doServer(){
        new ServerManage($this->current_dir);
    }

    //执行初始化
    public function doInit(){
        //当前目录创建一个文件，然后文件
        $file = new \Kuoqi\FileDriver($this->current_dir.'/.kuoqi/.config');
        $file->safe_code = "";
        $config = array();
        $config['current_dir'] = $this->current_dir;
        $config['host'] = "";
        $config['port'] = "";
        $config['token'] = "";
        \Kuoqi::warn("请初始化配置相关信息");
        \Kuoqi::warn("请输入host地址：",false);
        $config['host'] = \Kuoqi\Input::waitCliline();
        \Kuoqi::warn("请输入端口地址：",false);
        $config['port'] = \Kuoqi\Input::waitCliline();
        $file->setContent(\Kuoqi\Helper::jsonencode($config));
        //测试链接是否正常
        \Kuoqi::warn("链接服务器中...");
        $client = stream_socket_client(sprintf('tcp://%s:%s',$config['host'],$config['port']));
        if($client){
            $file->save();
            $file->setBasename(".ignore");
            if(!$file->exist()){
                $file->setContent(ClientManage::ruleIgnore());
                $file->save();
            }
            $file->setBasename('.replace');
            if(!$file->exist()){
                $file->setContent(ClientManage::ruleReplace());
                $file->save();
            }
            \Kuoqi::warn("配置站点成功");
        }else{
            \Kuoqi::warn("客户端链接失败，是否继续使用[Y/N]:",false);
            $code = strtoupper(\Kuoqi\Input::waitCliline());
            while (!in_array(($code),array('Y','N'))){
                \Kuoqi::warn("客户端链接失败，是否继续使用[Y/N]:",false);
                $code = strtoupper(\Kuoqi\Input::waitCliline());
            }
            if($code=='Y'){
                $file->save();
                $file->setBasename(".ignore");
                $file->setContent(ClientManage::ruleIgnore());
                $file->save();
                $file->setBasename('.replace');
                $file->setContent(ClientManage::ruleReplace());
                $file->save();
                \Kuoqi::warn("配置站点成功");
            }else{
                $file->delete();
                \Kuoqi::warn("取消该操作");
            }
        }
    }
    //执行pull
    protected function doPull(){  \Kuoqi::warn("pull");}
    protected function doPush(){
        \Kuoqi::warn("执行命令 push.");
        $config = new \Kuoqi\FileDriver($this->_webroot.'/.kuoqi/.config');
        $config = $config->readAsJson();
        $this->master = new \Kuoqi\Socket\Client($config['host'],$config['port']);
        \Kuoqi::warn("链接服务器",false);
        $this->master->start();
        if(!$this->master->isValied()){
            return \Kuoqi::warn("失败，".$this->master->getError());
        }
        \Kuoqi::warn("成功");
        //获取当前的目录信息，进行一个传递
        $dir = str_replace(\Kuoqi\Route::str2rootpath($this->_webroot),"",\Kuoqi\Route::str2rootpath($this->current_dir));
        $this->doPushPath($this->master,$dir);
        \Kuoqi::warn("推送数据完成 .");
    }
    //执行推送到文件夹
    public function doPushPath($client,$dir){
        //判断目录是否在忽略目录名单中
//        foreach ($this->_ignores as $k){
//            if(fnmatch( $k, $dir )){
//                return;
//            }
//        }
        //判断当前目录是否存在，如果不存在则尝试创建之
        if(!ClientManage::isDir($client,$dir)){
            ClientManage::createDir($client,$dir);
        }
        //浏览目录的数据信息
        $wlist = ClientManage::scandir($this->master,$dir);
        //获取本地的相关文件目录信息
        $dirDriver = new \Kuoqi\DirDriver($this->_webroot.$dir);
        $llist = $dirDriver->scanAsArray();
        foreach ($llist as $item){
            $check = strpos($item['path'],".");
            if($check===0){
                //非需要处理的内容，不用处理
                continue;
            }
            //这个是文件
            $is_continue = 0;
            foreach ($this->_ignores as $k){
                if(fnmatch( $k, $dir.'/'.$item['path'] )){
                    $is_continue = 1;continue;
                }
            }
            if($is_continue){
                continue;
            }
            if($item['type']=='DIR'){
                //这个是目录
                //if($item['filemtime']>$wlist[$item['path']]){}
                $this->doPushPath($client,$dir.'/'.$item['path']);
            }else{
                //推送一个文件上到系统
                if(isset($wlist[$item['path']])&&($item['filemtime']<=$wlist[$item['path']])){
                    continue;
                }

                \Kuoqi::warn("上传".$dir.'/'.$item['path'].' size '.filesize($this->_webroot.$dir.'/'.$item['path']),false);
                foreach ($this->_replaces as $k=>$v){
                    if($k==$dir.'/'.$item['path']){
                        ClientManage::pushFile($this->master,$v,$this->_webroot.$dir.'/'.$item['path']);
                    }
                }
                ClientManage::pushFile($this->master,$dir.'/'.$item['path'],$this->_webroot.$dir.'/'.$item['path']);
                \Kuoqi::warn('成功');
            }
        }
    }
}