<?php
namespace Kuoqi\DbDriver;
use Kuoqi\Cache;
use Kuoqi\Config;
use Kuoqi\Error;
use Kuoqi\Input;
use PDO;

/**
 * 还是使用这个作为数据库的完全基本操作，根据配置来的那种
 * Class PdoDriver
 * @package Kuoqi\DbDriver
 */
class PdoDriver implements DbInterface{
    public $PdoDriver = null;//这个是pdo服务的部分，这里只处理PDO自己内部的事情，保持纯净无污染的驱动
    protected $_connectinfo = '';//数据库链接信息，保存基础的链接字符串的相关信息
    protected $_target_type = '';//目标数据库的类型
    public $prefix = '';//数据表前缀
    public $_back_sql = false;//是否返回sql语句，用于语句自查，子语句等相关场景
    public $cachetime = 0;//是否缓存，如果缓存的话设置缓存时长
    public $dbdriver = null;//数据库原始的内容部分（待核定）
    public $sqldriver = null;//sql部分（待核定）
    public $objfield = null;//字段的驱动对象（待核定）
    public $objtable = null;//table的驱动处理
    public $objwhere = null;//where条件的驱动（待核定）
    public $objgroup = null;//分组查询的数组（待核定）
    public $objorder = null;//排序的数组（待核定）
    public $objlimit = null;//limit的相关处理（待核定）
    public $objdata = null;//（待核定）
    public $objjoin = null;//（待核定）
    public $_options = array();
    protected $_charset = '';
    protected $_dbname = '';
    protected $_tbname = '';

    //根据oracle的数据库内容创建oracle数据库
    static public function create($config,$options = array()){
        $pdoDriver = new self();
        $config = \Kuoqi::opts($config,array(
            'DB_TARGET'=>\Kuoqi\Config::get("DB_TARGET"),//目标数据库类型
            'DB_HOST'=>\Kuoqi\Config::get("DB_HOST"),//使用IP/TCP
            'DB_NAME'=>\Kuoqi\Config::get("DB_NAME"),//使用数据库
            'DB_PORT'=>\Kuoqi\Config::get("DB_PORT"),//端口
            'DB_USER'=>\Kuoqi\Config::get("DB_USER"),//使用数据库用户名[kuoqicmj]
            'DB_PSWD'=>\Kuoqi\Config::get("DB_PSWD"),//使用数据库链接密码[]
            'DB_PREFIX'=>\Kuoqi\Config::get("DB_PREFIX"),//数据库表前缀
            'DB_CHARSET'=>\Kuoqi\Config::get("DB_CHARSET"),//数据库字符集【UTF8】[utf8mb4]
            'DB_C_TIME'=>\Kuoqi\Config::get("DB_C_TIME"),//默认缓存秒数
        ));
        $pdoDriver->setChar($config['DB_CHARSET']);
        $pdoDriver->setTarget($config['DB_TARGET']);
        $pdoDriver->prefix = $config['DB_PREFIX'];
        $pdoDriver->setOptions($options);
        $pdoDriver->setDbname($config['DB_NAME']);
        $pdoDriver->setConnectinfo($config);
        return $pdoDriver;
    }

    /**
     * 设置当前使用的编码格式
     * @param $char
     */
    public function setChar($char)
    {
        $this->_charset = $char;
    }

    /**
     * 设置额外参数数据信息
     * @param $options
     */
    public function setOptions($options){
        $this->_options = $options;
    }

    /**
     * 设置外部显示数据库名
     * @param $dbname
     */
    public function setDbname($dbname){
        $this->_dbname = $dbname;
    }

    /**
     * 设置链接信息
     * @param $connectinfo
     */
    public function setConnectinfo($connectinfo){
        $this->_connectinfo = \Kuoqi\Openssl::encode(serialize($connectinfo));
    }
    public function setTarget($type){
        $this->_target_type = strtoupper($type);
    }
    //获取目标服务器类型
    public function getTarget(){
        return strtoupper($this->_target_type);
    }

    //获取版本号信息
    public function getVersion(){
        $sql = 'select VERSION() as version';
        $query = $this->query($sql);
        $data = $query->fetch(\PDO::FETCH_ASSOC);
        return $data['version'];
    }

    /**
     * PdoDriver constructor.
     * @param string $host
     * @param string $port
     * @param string $dbname
     * @param string $username
     * @param string $passwd
     * @param array $options
     * @param string $charset
     * @throws Error
     */
    public function __construct($host = '',$port = '',$dbname = '', $username = '', $passwd = '', array $options = array(),$charset = '')
    {
        $config = \Kuoqi::opts(array(
            'DB_TARGET'=>\Kuoqi\Config::get("DB_TARGET"),
            'DB_HOST'=>$host,//使用IP/TCP
            'DB_NAME'=>$dbname,//使用数据库
            'DB_PORT'=>$port,//端口
            'DB_USER'=>$username,//使用数据库用户名[kuoqicmj]
            'DB_PSWD'=>$passwd,//使用数据库链接密码[]
            'DB_PREFIX'=>\Kuoqi\Config::get("DB_PREFIX"),//数据库表前缀
            'DB_CHARSET'=>\Kuoqi\Config::get("DB_CHARSET"),//数据库字符集【UTF8】[utf8mb4]
            'DB_C_TIME'=>\Kuoqi\Config::get("DB_C_TIME"),//默认缓存秒数
        ),array(
            'DB_TARGET'=>\Kuoqi\Config::get("DB_TARGET"),//目标数据库类型
            'DB_HOST'=>\Kuoqi\Config::get("DB_HOST"),//使用IP/TCP
            'DB_NAME'=>\Kuoqi\Config::get("DB_NAME"),//使用数据库
            'DB_PORT'=>\Kuoqi\Config::get("DB_PORT"),//端口
            'DB_USER'=>\Kuoqi\Config::get("DB_USER"),//使用数据库用户名[kuoqicmj]
            'DB_PSWD'=>\Kuoqi\Config::get("DB_PSWD"),//使用数据库链接密码[]
            'DB_PREFIX'=>\Kuoqi\Config::get("DB_PREFIX"),//数据库表前缀
            'DB_CHARSET'=>\Kuoqi\Config::get("DB_CHARSET"),//数据库字符集【UTF8】[utf8mb4]
            'DB_C_TIME'=>\Kuoqi\Config::get("DB_C_TIME"),//默认缓存秒数
        ));
        $this->_charset = $config['DB_CHARSET'];
        $this->setTarget($config['DB_TARGET']);
        $this->prefix = $config['DB_PREFIX'];
        $this->_options = $options;
        $this->_dbname = $config['DB_NAME'];
        $this->setConnectinfo($config);
        //content需要手动链接才行
    }
    /**
     * 缓存
     * @param $time 缓存时长，填0则强制不缓存
     * @return $this 返回当前对象
     */
    public function cache($time)
    {
        $this->cachetime = intval($time);return $this;
    }

    //尝试创建一个数据库
    public function tryCreateDb($dbname,$dbcharset,$dborder){
        $sql = sprintf("SHOW DATABASES like '%s'",$dbname);
        $query = $this->query($sql);
        $data = $query->fetch(PDO::FETCH_ASSOC);
        if($data){
            //数据库存在
        }else{
            //数据库不存在,尝试创建数据库信息
            $sql = sprintf("CREATE DATABASE `%s` CHARACTER SET %s"
                ,urlencode($dbname),$dbcharset
            );
            $count = $this->query($sql);
            return true;
        }
    }

    /**
     * 链接数据库的说法
     * @throws Error
     */
    public function connect(){
        if($this->getTarget()=="MYSQL"){
            $this->sqldriver = new Sql\Sql($this);
            $this->connectMysql(true);
        }else if($this->getTarget()=="OCI"){
            $this->sqldriver = new Oracle\SqlOracle($this);
            $this->connectOracle();
        }else{
            throw new Error("链接数据库失败：数据库驱动[".$this->_target_type."]不存在，请检查");
        }
    }

    /**
     * 链接mysql数据库
     * @throws Error
     */
    public function connectMysql($withDbname = true){
        $clsid = md5($this->_connectinfo);
        $this->PdoDriver = \Kuoqi::getAskcache($clsid);
        if($this->PdoDriver){ return; }
        $info =  unserialize(\Kuoqi\Openssl::decode($this->_connectinfo));
        if($info){
            \Kuoqi\Logs::listenStart('connect_mysql');
            $dsn = sprintf('mysql:host=%s;port=%s'
                ,$info['DB_HOST']
                ,$info['DB_PORT']
            );
            $this->PdoDriver = new \PDO($dsn, $info['DB_USER'], $info['DB_PSWD'], $this->_options);
            if($this->_charset){
                $this->query("set names ".$this->_charset);
            }
            if($withDbname){
                $this->useDbname($info['DB_NAME']);
            }
            \Kuoqi::setAskcache($clsid,$this->PdoDriver);
            \Kuoqi\Logs::initLog('connect mysql success','sql-connect',\Kuoqi\Logs::listenEnd('connect_mysql'));
        }else{
            \Kuoqi::$logs->log('链接失败：链接信息为['.print_r($info,true).']','error');
            throw new \Kuoqi\Error("数据库链接失败：[pdoDriver->_connectinfo]验证失败");
        }
    }

    /**
     * 链接oracle数据库
     * @throws Error
     */
    public function connectOracle(){
        $clsid = md5($this->_connectinfo);
        $this->PdoDriver = \Kuoqi::getAskcache($clsid);
        if($this->PdoDriver){ return; }
        $info =  unserialize(\Kuoqi\Openssl::decode($this->_connectinfo));
        if($info){
            \Kuoqi\Logs::listenStart('connect_oci');
            $dsn = sprintf("oci:dbname=//%s:%s/orcl;charset=%s;database=%s",
                $info['DB_HOST'],
                $info['DB_PORT'],
                $this->_charset,
                $info['DB_NAME']
            );
            $this->PdoDriver = new \PDO($dsn, $info['DB_USER'], $info['DB_PSWD'], $this->_options);
            \Kuoqi::setAskcache($clsid,$this->PdoDriver);
            \Kuoqi\Logs::initLog('connect oci success','sql-connect',\Kuoqi\Logs::listenEnd('connect_oci'));
        }else{
            \Kuoqi::$logs->log('链接失败：链接信息为['.print_r($info,true).']','error');
            throw new \Kuoqi\Error("数据库链接失败：[pdoDriver->_connectinfo]验证失败");
        }
    }


    /**
     * 目标服务器
     * @param $target_type
     */
    public function target($target_type)
    {
        return $this->setTarget($target_type);
    }


    /**
     * 使用数据库作为目标数据库
     * useDbname 2024-07-09
     * @param $dbname
     * @return bool
     */
    public function useDbname($dbname){
        $res = $this->PdoDriver->exec("USE `$dbname`");
        if($res===false){
            return false;
        }else{
            return true;
        }
    }

    /**
     * 获取数据库名
     * @return string
     */
    public function getDbname(){
        return $this->_dbname;
    }
    /**
     * 获取场景的sql内容
     * @param $scene
     * @return string
     * @throws Error
     */
    public function getSqlscene($scene){
        return $this->sqldriver->scale($scene);
    }
    //获取数据表名
    public function getTbname(){
        throw new Error("获取数据表名");
    }


    /**
     * 数据表名
     * @param $tbname 数据表原始名称
     * @param string $tbalias 表的别名
     * @return $this
     */
    public function table($tbname,$asName = '')
    {
        $this->objtable = new Sql\Table($this,$tbname,$asName);
        return $this;
    }

    /**
     * @param $code 待替换的前缀
     * @return $this
     */
    public function prefix($code){
        $this->prefix = $code;
        return $this;
    }
    /**
     * 简单的配置表别称
     * @param $tbaslas 别名，在table之后可用
     * @return $this
     */
    public function alias($tbaslas){
        $this->objtable->alias = $tbaslas;
        return $this;
    }
    /**
     * 是否返回sql语句
     * @param bool $isSql
     * @return $this
     */
    public function fetchSql($isSql = false){
        $this->_back_sql = $isSql;
        return $this;
    }

    /**
     * 判定是否mysql数据库
     * @return bool
     */
    public function isMysql(){
        if($this->getTarget()=="MYSQL"){
            return true;
        }else{
            return false;
        }
    }

    /***
     * 判断当前是否为oralce数据库
     * @return bool
     */
    public function isOracle(){
        if($this->getTarget()=="OCI"){
            return true;
        }else{
            return false;
        }
    }

    /**
     * 增加数据或者修改数据的数据写入
     * @param $data
     * @return $this
     * @throws Error
     */
    public function data($data)
    {
        if($this->isMysql())
        {
            $this->objdata = new Sql\Data($this);
        }
        else if($this->isOracle())
        {
            $this->objdata = new Oracle\DataOracle($this);
        }
        else
        {
            throw new Error("数据库类型不存在:".$this->getTarget());
        }
        $this->objdata->input($data);
        return $this;
    }

    /**
     * 增加或者修改数据的时候去已经有的数据中追加数据的操作，没有数据则重建
     * @param $data
     * @return $this
     * @throws Error
     */
    public function andData($data)
    {
        if(!$this->objdata)
        {
            if($this->isMysql())
            {
                $this->objdata = new Sql\Data($this);
            }
            else if($this->isOracle())
            {
                $this->objdata = new Oracle\DataOracle($this);
            }
            else
            {
                throw new Error("数据库类型不存在:".$this->getTarget());
            }
        }
        $this->objdata->input($data);
        return $this;
    }

    /**
     * where条件对象
     * @param $condition
     * @return $this
     * @throws Error
     */
    public function where($condition){
        if($this->isMysql())
        {
            $this->objwhere = new Sql\Where();
        }
        else if($this->isOracle())
        {
            $this->objwhere = new Oracle\WhereOracle();
        }
        else
        {
            throw new Error("数据库类型不存在:".$this->getTarget());
        }
        $this->objwhere->input($condition);//处理内容
        return $this;
    }
    public function andWhere($condition){
        if(!$this->objwhere)
        {
            if($this->isMysql())
            {
                $this->objwhere = new Sql\Where();
            }
            else if($this->isOracle())
            {
                $this->objwhere = new Oracle\WhereOracle();
            }
            else
            {
                throw new Error("数据库类型不存在:".$this->getTarget());
            }
        }
        $this->objwhere->input($condition);//处理内容
    }
    /**
     * 传入字段的方法定义
     * @param $params
     * @return $this
     */
    public function field($params){
        $this->objfield = new Sql\Field($params);
        return $this;
    }

    /**
     * 加入字段的内容
     * andField 2024-09-07
     * @param $params
     * @return $this
     * @throws Error
     */
    public function andField($params){
        if(!$this->objfield)
        {
            if($this->isMysql())
            {
                $this->objfield = new Sql\Field($params);
            }
            else
            {
                throw new Error("数据库类型不存在:".$this->getTarget());
            }
        }
        $this->objfield->input($params);
        return $this;
    }

    /**
     * 数据库排序
     * @param $code
     * @return $this
     */
    public function order($code){
        $this->objorder = 'ORDER BY '.str_replace('"','\"',$code);
        return $this;
    }
    /**
     * 分组
     * @param $code
     * @return $this
     */
    public function group($code,$having = ""){
        $this->objgroup = 'GROUP BY '.str_replace('"','\"',$code);
        if($having){
            $this->objgroup .= sprintf(" HAVING %s",$having);
        }
        return $this;
    }
    /**
     * 数据库限制
     * @param $offset
     * @param int $count
     * @return $this
     * @throws \Kuoqi\Error
     */
    public function limit($offset,$count = 0){
        if(is_string($offset)){
            $offset = intval($offset);
        }else if(is_array($offset)){
            if(count($offset)!=2){
                throw new \Kuoqi\Error('limit数组只支持两个值');
            }
            $count = $offset[1];
            $offset = $offset[0];

        }

        $this->objlimit = sprintf('LIMIT %s%s',
            $offset,//offset
            ($count>0)?','.$count:''//count
        );
        return $this;
    }
    /**
     * join
     * @param $rex
     * @param string $condition
     * @return $this
     */
    public function join($rex, $condition = ''){
        if($this->objjoin){
            $this->objjoin->input($rex,$condition);
        }else{
            $this->objjoin = new Sql\Join($this);
            $this->objjoin->input($rex,$condition);
        }
        return $this;
    }
    //执行清空内容信息，主要真针对，limit,where,data,join,table等相关的清空
    public function clear(){
        //$this->objtable = null;//清空table数据
        $this->objjoin = null;//清空join数据
        $this->objdata = null;//清空data数据
        $this->objfield = null;//清空field数据
        $this->objgroup = null;//清空group数据
        $this->objlimit = null;//清空limit数据
        $this->objorder = null;//清空order数据
        $this->objwhere = null;//清空where数据
    }

    /**
     * query的执行
     * @param $statement
     * @return mixed
     * @throws Error
     */
    public function query($statement)
    {
        \Kuoqi\Logs::listenStart();
        $query = $this->PdoDriver->query($statement);
        \Kuoqi\Logs::initLog($statement,'query',\Kuoqi\Logs::listenEnd());
        if($query===false){
            $info = $this->PdoDriver->errorInfo();
            if($info[1]=='2006'){
                //尝试重新连接服务器
                $this->PdoDriver = null;
                $this->connect();
                return $this->query($statement);
            }else{
                $msg = "[PdoDriver][{$info[2]}] on <span style='color: #333;text-decoration: line-through;'>{$statement}</span>";
                throw new Error($msg);
            }
        }else{
            return $query;
        }
    }

    /**
     * getField数据单字段查询
     * @param $kname
     * @return mixed
     */
    public function getField($kname){
        $fields = $this->field($kname)->find();
        if($this->_back_sql){
            return $fields;
        }else{
            return $fields[$kname];
        }
    }
    //数据单字段设置
    public function setField($kname,$kvalue){
        return $this->data(array($kname=>$kvalue))->update();
    }
    //数据库结果类-获取单个字段
    public function getOne($kname){
        return $this->getField($kname);
    }
    //数据库结果类-设置单个字段
    public function setOne($kname,$kvalue){
        return $this->setField($kname,$kvalue);
    }
    //数据库结果类-简单查询
    public function find($andWhere = array())
    {
        $sql = $this->getSqlscene('find');
        if($this->_back_sql){
            return $sql;
        }
        //启用缓存的步骤
        if($this->cachetime&&$data = Cache::getCache($sql,'/database',false))
        {
            return $data;
        }
        //使用查询语句
        $query = $this->query($sql);
        $data = $query->fetch(\PDO::FETCH_ASSOC);
        if($this->cachetime)
        {
            Cache::setCache($sql,$data,$this->cachetime,'/database',false);
            $this->cachetime = 0;//变更缓存的时间信息
        }
        return $data;
    }

    /**
     * 数据库结果类-简单多条数据查询
     * @param array $andWhere
     * @return mixed|null|string
     * @throws Error
     */
    public function select($andWhere = array()){
        $sql = $this->getSqlscene('select');
        if($this->_back_sql){
            return $sql;
        }
        //启用缓存的步骤
        if($this->cachetime&&$data = Cache::getCache($sql,'/database',false)){
            return $data;
        }
        //使用查询语句
        $query = $this->query($sql);
        $data = $query->fetchAll(\PDO::FETCH_ASSOC);
        if($this->cachetime){
            Cache::setCache($sql,$data,$this->cachetime,'/database',false);
            $this->cachetime = 0;//变更缓存的时间信息
        }
        return $data;
    }
    public function exec($sql){
        return $this->PdoDriver->exec($sql);
    }
    //数据库结果类-简单增加
    public function insert($data = array())
    {
        $this->andData($data);
        $sql = $this->getSqlscene('insert');
        if($this->_back_sql){
            return $sql;
        }
        $this->query($sql);
        $data = $this->PdoDriver->lastInsertId();
        return $data;
    }
    //数据库结果类-insert 的别称
    public function add($data = array()){
        return $this->insert($data);
    }
    //数据库结果类-简单更新
    public function update($data = array()){
        $this->andData($data);
        $sql = $this->getSqlscene('update');
        if($this->_back_sql){
            return $sql;
        }
        $data = $this->exec($sql);
        return $data;
    }
    //数据库结果类-update的别称
    public function save($data = array()){
        return $this->update($data);
    }
    //数据库结果类-简单查询
    public function delete($andWhere = array()){
        $this->andWhere($andWhere);
        $sql = $this->getSqlscene('delete');
        if($this->_back_sql){
            return $sql;
        }
        $data = $this->exec($sql);
        return $data;
    }

    /**
     * 数据库结果类-删除 同delete
     * @param array $andWhere
     * @return mixed
     */
    public function del($andWhere = array()){
        return $this->delete($andWhere);
    }
    //数据库结果类-简单的条数查询
    public function count(){
        $sql = $this->getSqlscene('count');
        if($this->_back_sql){
            return $sql;
        }
        $query = $this->query($sql);
        $status = $query->fetchColumn();
        return $status;
    }
    //获取所有列名,默认获取完整版
    public function getColumns(){
        $sql = $this->getSqlscene('columns');
        if($this->_back_sql){
            return $sql;
        }
        //启用缓存的步骤
        if($this->cachetime&&$data = Cache::getCache($sql,'/database',false)){
            return $data;
        }
        $query = $this->query($sql);
        $data = $query->fetchAll(\PDO::FETCH_ASSOC);
        if($this->cachetime){
            Cache::setCache($sql,$data,$this->cachetime,'/database',false);
            $this->cachetime = 0;//变更缓存的时间信息
        }
        return $data;
    }
    /**
     * 事务开始
     * @return bool
     */
    public function transStart(){
        return $this->PdoDriver->beginTransaction();
    }
    /**
     * 事务开始
     * @return bool
     */
    public function startTrans()
    {
        return $this->transStart();
    }
    /**
     * 事务回滚
     * @return bool
     */
    public function transBack(){
        if($this->PdoDriver->inTransaction()){
            $this->PdoDriver->rollBack();
        }
        return true;
    }
    /***
     * 【as transBack】事务回滚
     * @return bool
     */
    public function rollBack(){
        return $this->transBack();
    }
    /**
     * 事务提交
     * @return bool
     */
    public function transCommit(){
        if($this->PdoDriver->inTransaction()){
            $this->PdoDriver->commit();
        }
        return true;
    }
    public function commit()
    {
        return $this->transCommit();
    }

    /**
     * 数据库分页信息
     * @param string $pager
     * @param null $count
     * @return array
     * @throws Error
     */
    public function paginate($pager = '',$count = NULL){
        if(!$count){
            $count =I(Config::get('DB_PAGE_COUNTVAR'),Config::get('DB_PAGE_COUNT'),'intval');
        }
        if(!$pager){
            $pager = I(Config::get('DB_PAGE_VAR'),1,'intval');
        }
        if($pager<1){ $pager = 1;}
        $output = array();
        $output['totalCount'] = $this->count();
        $this->limit(($pager-1)*$count,$count);
        $output['data'] = $this->select();
        $output['pageNow'] = $pager;
        $output['pageCount'] = $count;
        $page = new Sql\Page();
        $page->Pager = $pager;
        $page->PageCount  = $count;
        $page->TotalCount = $output['totalCount'];
        if($pager==1){
            $output['hasPrev'] = false;
        }else{
            $output['hasPrev'] = true;
        }
        if($pager*$count<$output['totalCount']){
            $output['hasNext'] = true;
        }else{
            $output['hasNext'] = false;
        }
        $output['pager'] = $page->create();
        return $output;
    }
}