<?php
namespace Kuoqi\FileDriver;
use Kuoqi\Error;
use Kuoqi\FileDriver;

/**
 * 文件上传的模块的相关信息
 * Class Upload
 * @package Kuoqi\FtpServer
 */
class ConfigFileDriver extends \Kuoqi\KuoqiException {

    protected $config = array();

    protected $ofl = "";

    /**
     * 初始化
     * ConfigFileDriver constructor.
     * @param $fname
     */
    public function __construct($fname)
    {
        $this->ofl = $fname;
        $this->sync();//进行内容重载
    }

    /**
     * 重载数据
     * sync 2024-07-23
     */
    public function sync(){
        $fname = \Kuoqi\Route::str2rootpath($this->ofl);
        if(is_file($fname)&&is_readable($fname)){
            $arr = @include ($fname);
            if(is_array($arr)){
                $this->config = $arr;
            }
        }else{
            $this->setError(sprintf("引用配置文件[%s]失败，配置文件不存在或不可读",$fname));
        }
    }

    /**
     * 获取当前有多少条数据信息
     * getCount 2024-07-23
     */
    public function getCount(){
        return count($this->config);
    }

    //获取
    public function get($n){
        return isset($this->config[$n])?$this->config[$n]:"";
    }

    //设置
    public function set($n,$v){
        $this->config[$n] = $v;
    }

    //保存
    public function save(){
        $array = array();
        ksort($this->config);
        foreach ($this->config as $k=>$v){
            $array[] =  sprintf("\n    '%s'=>%s",$k,$this->syncLine($v));
        }
        $stream = sprintf("<?php\nreturn array(%s\n);",implode(",",$array));
        $fileDriver = new FileDriver($this->ofl);
        $fileDriver->safe_code = "";
        $fileDriver->setContent($stream);
        $fileDriver->save();
    }

    //输出一个数据
    public function syncLine($config){
        if(is_string($config)){
            return "'".$config."'";
        }else if(is_int($config)){
            return $config;
        }else if(is_bool($config)){
            if($config==true){
                return "true";
            }else{
                return "false";
            }
        }else if(is_array($config)){
            if($config){
                if (array_values($config) === $config) {
                    //这个是索引数组
                    $array = array();
                    foreach ($config as $item){
                        $array[] =  $this->syncLine($item);
                    }
                    return sprintf("array(%s)",implode(",",$array)) ;
                }else{
                    //这个是关联数组
                    $array = array();
                    foreach ($config as $k=>$v){
                        $array[] =  sprintf("'%s'=>%s",$k,$this->syncLine($v));
                    }
                    return sprintf("array(%s)",implode(",",$array)) ;
                }
            }else{
               return 'array()';
            }
        }else{
            throw new Error("暂无相关场景支撑:".json_encode($config));
        }
    }

    //全部输出
    public function toArray(){
        return $this->config;
    }
}