<?php
namespace Kuoqi\FileDriver;
/**
 * 文件上传的模块的相关信息
 * Class Upload
 * @package Kuoqi\FtpServer
 */
class Upload extends \Kuoqi\KuoqiException {
    protected $_file = null;//当前下面的文件
    public $max_size = '2M';//文件限制上传的大小
    public $mimelist = array();//可上传的mime列表
    public $file_driver = null;//上次上传的文件驱动
    protected $file_link = "";//文件的链接
    protected $info = array();
    public function __construct($upfile='',$upsize = '')
    {
        //选择的第一个，然后进行一个的配置
        $this->_file = $upfile?$upfile:array_pop($_FILES);
        $upsize&&$this->max_size = $upsize;
        $this->mimelist = Mime::mimeFilelist();
        //$this->upload_path = \Kuoqi\Helper::getConfig('PATH_UPLOAD').'/'.date('Ymd').'/';
    }

    /**
     * 检测是否获取到上传文件的相关信息
     * @return bool
     */
    protected function checkFileExist(){
        if((is_array($this->_file)&&(count($this->_file)==5))){
            return true;
        }else{
            $last = error_get_last();
            if($last&&$last['message']){
                $this->errmsg = $last['message'];
                $this->errcode = -1;
                return false;
            }else{
                $this->errmsg = 'file upload error with no file';
                $this->errcode = -1;
                return false;
            }

        }
    }
    protected function checkFileComplete(){
        if($this->_file['error']==0){
            return true;
        }else{
            $this->errmsg = 'upload_file is not complete';
            $this->errcode = -1;
            return false;
        }
    }
    protected function checkFileReadable(){
        if(is_readable($this->_file['tmp_name'])){
            return true;
        }else{
            $this->errmsg = 'upload_file tmp can not readabled';
            $this->errcode = -1;
            return false;
        }
    }
    protected function checkFileMaxWithSystem(){
        $size = ini_get('upload_max_filesize');
        $size = \Kuoqi\Format::str2byte($size);
        if($this->_file['size']>$size){
            $this->errmsg = 'file_size not greater than php.ini upload_max_filesize';
            $this->errcode = -1;
            return false;
        }else{
            return true;
        }
    }
    protected function checkFileMax(){
        $size = \Kuoqi\Format::str2byte($this->max_size);
        if($this->_file['size']>$size){
            $this->errmsg = 'file_size not greater than setting max_size';
            $this->errcode = -1;
            return false;
        }else{
            return true;
        }
    }
    protected function checkFileMime(){
        if(in_array($this->_file['type'],$this->mimelist)){
            return true;
        }else{
            $this->errmsg = 'file_mime not in list mime with '.$this->_file['type'].' and library is '.json_encode($this->mimelist);
            $this->errcode = -1;
            return false;
        }
    }

    /**
     * 设置上传文件的地址
     * setFilelink 2024-11-21
     * @param $filelink
     */
    public function setFilelink($filelink){
        $this->file_link = $filelink;
    }

    //检测文件的完整性
    public function checkFile(){
        return $this->checkFileExist()
        &&$this->checkFileComplete()//检验文件完整性
        &&$this->checkFileReadable()//检验文件是否可读
        &&$this->checkFileMaxWithSystem()
        &&$this->checkFileMax()//检验文件大小是否操作当前设置
        &&$this->checkFileMime();//检验文件类型是否可以上传
    }

    public function getExt(){
        //尝试获取当前文件是否有ext
        $ext = strrchr($this->_file['name'],'.');
        if(!$ext){
            $ext = Mime::mime2ext($this->_file['type']);
        }
        return $ext;
    }
    public function info()
    {
        return $this->info;
    }

    //获取临时文件文件名
    public function getTmpname(){
        return $this->_file['name'];
    }
    //获取临时完整文件
    public function getTmpfile(){
        return $this->_file['tmp_name'];
    }

    //执行一个upload上传
    public function Upload($upfile = ''){
        if($upfile){ $this->_file = $upfile;}
        if(!$this->checkFile()){ return false;}
        $ext = $this->getExt();
        if($this->file_link){
            $this->file_driver = new \Kuoqi\FileDriver($this->file_link);
            $this->file_driver->checkPath();
        }else{
            $this->file_link = '/upload/'.date('Ymd').'/'.md5($this->_file['name'].time().rand(1000,9999)).$ext;
            $this->file_driver = new \Kuoqi\FileDriver('{@HOME}'.$this->file_link);
            $this->file_driver->checkPath();
        }

        if(move_uploaded_file($this->_file['tmp_name'],$this->file_driver->getFile(true))){
            $this->info['mime'] = $this->_file['type'];
            $this->info['ext'] = $ext;
            $this->info['name'] = $this->_file['name'];
            $this->info['filename'] = $this->file_driver->getFilename();
            $this->info['url'] = $this->getUrl();
            $this->info['host'] = \Kuoqi\Helper::getConfig("HOST");
            $this->info['path'] = $this->file_link;
            $this->info['size'] = $this->_file['size'];
            $this->info['md5'] = $this->md5();
            $this->info['fullpath'] = $this->file_driver->getFile(true);
            return true;
        }else{
            $this->errcode = 1;
            $this->errmsg = "文件移动失败，请检查";
            return false;
        }
    }

    //获取已经完成的文件地址
    public function getUrl(){
        return \Kuoqi\Route::str2rootpath('{@WEB}'.$this->file_link);
    }
    public function md5(){
        $md5 = $this->file_driver->getMd5();
        return $md5;
    }

    /**
     * 判定文件为excel
     * createby kuoqi at 2022/9/29 11:14
     * @return bool
     */
    public function isExcel(){
        if(!in_array($this->_file['type'],[
            'application/octet-stream',
            'application/vnd.ms-excel',
            'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet'
        ])){
            $this->errmsg = 'file_mime not in list mime with '.$this->_file['type'].' and library is '.json_encode($this->mimelist);
            $this->is_valied = false;
            return false;
        }
        return true;
    }

    public function isImg(){
        $list = [
            'image/jpeg',
            'image/jpg',
            'image/png'
        ];
        if(!in_array(strtolower($this->_file['type']),$list)){
            $this->errmsg = 'file_mime not in list mime with '.$this->_file['type'].' and library is '.json_encode($list);
            $this->is_valied = false;
            return false;
        }
        return true;
    }

    /**
     * 判断为通用文件,通用文件为非运行的相关文件的信息
     * isFile 2024-12-05
     * @return bool
     */
    public function isFile(){
        $list = [
            '.jpeg','.jpg','.png','.mp4','.mp3','.xls','.xlsx','.pdf', '.ppt','.pptx','.doc','.docx','.zip','.rar',
        ];
        if(!in_array(strtolower($this->getExt()),$list)){
            $this->setError('file_ext not in list mime with '.$this->getExt().' and library is '.json_encode($list));
            return false;
        }
        return true;
    }

}