<?php
namespace Kuoqi\Kuoqiui;
use Kuoqi\Error;
use Kuoqi\Config;

/**
 * 分页控件
 * Class Form
 * @package Kuoqi\Kuoqiui
 */
class Paginate{
    protected $format = '';//使用模板
    public $apiurl = '';//当前的接口请求地址
    public $total_count = 0;//数据总条数信息
    public $data_provider = array();//条例的数据信息
    public $page_count = 20;//每页的数据信息
    public $page_now = 1;//当前页面信息
    public $pager = '';//分页的最基本的信息
    public $columns = '';//字段的相关信息
    public $checkbox = false;//不展示checkbox
    public $ID = '';//自动填充的唯一识别ID
    public $index = 'id';//checkbox的索引字段，也是很多数据的索引，如果多选什么的这个字段是必填的，不填默认为ID
    public $classname = 'table table-kuoqiui';//基础的class信息
    protected $stream = '';//字符串信息
    protected $stream_head = '';//头部信息的构建
    protected $stream_body = '';//body信息的构建

    //清空字符串输出
    public function clear(){
        $this->stream = $this->stream_head = $this->stream_body = '';
    }

    //头部信息构建
    public function structureHead(){
        foreach($this->columns as $col){
            $classnamehead = '';
            if(isset($col['classname_head'])){
                $classnamehead = $col['classname_head'];
            }
            $this->stream_head .= sprintf('<th class="%s">%s</th>'
                ,$classnamehead
                ,$col['title']
            );
        }
    }
    //构建带UI的头部信息
    public function structureCompleteHead(){

    }

    //body信息的构建
    public function structureBody(){
        foreach($this->data_provider as $item){
            $this->stream_body .= '<tr>';
            foreach($this->columns as $col){
                $this->stream_body .= sprintf('<td %s %s>%s</td>',
                    (isset($col['classname'])&&$col['classname'])?('class="'.$col['classname'].'"'):'',
                    isset($col['width'])?('width=\''.$col['width'].'\''):'',
                    $col['value']($item)
                );
            }
            $this->stream_body .= '</tr>';
        }
    }

    //
    public function structurePager(){
        return sprintf('<tr><td colspan="%s"><div class="pdl-15 pdr-15 row-flex text-ch"><div class="col-other">%s%s%s</div><ul class="pager">%s</ul></div></td></tr>'
            ,count($this->columns)
            ,'<span class="label pdl-none">共'.ceil($this->total_count / $this->page_count).'页</span>'
            ,'<span class="label pdl-none">当前'.($this->page_now).'页</span>'
            ,'<span class="label pdl-none">共'.($this->total_count).'条数据</span>'
            ,$this->pager
        );
    }

    //输出表格
    public function toTable(){
        $this->clear();
        $table = sprintf('<div class="card card-table"><div class="card-body"><table %s class="table tabel-kuoqiui %s" data-totalcount="%s" data-pagecount="%s" data-pagenow="%s" data-index="%s" data-apiurl="%s"><thead>%s</thead><tbody>%s</tbody></table></div><div class="card-foot"></div></div>',
            $this->ID?'id="'.$this->ID.'"':"",
            $this->classname,
            $this->total_count,
            $this->page_count,
            $this->page_now,
            $this->index,
            $this->apiurl,
            (new Paginate\TableHead($this))->toString(),
            (new Paginate\TableBody($this))->toString($this->data_provider)
        );
        return $table;
    }
    //输出为数据body部分
    public function toBody(){
        return (new Paginate\TableBody($this))->toString($this->data_provider);
    }

    /**
     * 使用简单的由后端给的数据进行创建分页信息
     * @param array $opts
     * @return string
     * @throws Error
     */
    static public function simple($opts = array()){
        $ths = new self();
        if(!isset($opts['dataer'])){
            throw new Error("请传入dataer相关信息，包含,totalCount,data,pageNow,pageCount");
        }
        if(!isset($opts['columns'])){
            throw new Error("请传入columns字段信息");
        }
        if(isset($opts['classname'])){
            $ths->classname = $opts['classname'];
        }else{
            $ths->classname = 'table table-toggle';
        }
        $ths->total_count = $opts['dataer']['totalCount'];
        $ths->data = $opts['dataer']['data'];
        $ths->data_provider = $opts['dataer']['data'];
        $ths->page_now = $opts['dataer']['pageNow'];
        $ths->page_count = $opts['dataer']['pageCount'];
        $ths->pager = $opts['dataer']['pager'];
        $ths->columns = $opts['columns'];
        $ths->clear();
        $ths->structureHead();//头部的相关信息构建
        $ths->structureBody();//body的相关信息的构建
        $table = sprintf('<table class="%s"><thead><tr>%s</tr></thead><tbody>%s%s</tbody></table>',
            $ths->classname,
            $ths->stream_head,
            $ths->stream_body,
            $ths->structurePager()
        );
        return $table;
    }

    /**
     * KuoqiUI分页模块入口
     * Kuoqiui 2024-07-20
     * @param array $opts
     * @return Paginate
     * @throws Error
     */
    static public function Kuoqiui($opts = array()){
        $ths = new self();
        $ths->format = 'kuoqiui';
        if(!isset($opts['dataer'])){
            throw new Error("请传入dataer相关信息，包含,totalCount,data,pageNow,pageCount");
        }
        if(!isset($opts['columns'])){
            throw new Error("请传入columns字段信息");
        }
        if(isset($opts['classname'])){
            $ths->classname = $opts['classname'];
        }
        $params = I("get");
        $params['search'] = I("get.search","","string");
        $ths->ID = isset($opts['id'])?$opts['id']:"";
        $ths->index = isset($opts['index'])?$opts['index']:"id";
        $ths->checkbox = isset($opts['checkbox'])?boolval($opts['checkbox']):false;
        $ths->total_count = $opts['dataer']['totalCount'];
        $ths->data = $opts['dataer']['data'];
        $ths->data_provider = $opts['dataer']['data'];
        $ths->page_now = $opts['dataer']['pageNow'];
        $ths->page_count = $opts['dataer']['pageCount'];
        $ths->pager = $opts['dataer']['pager'];
        $ths->columns = $opts['columns'];
        return $ths;
    }

}