<?php
namespace Kuoqi\Kuoqiui\Form;
use Kuoqi\Error;
use Kuoqi\Kuoqiui\Form\Item\DateItemForm;
use Kuoqi\Kuoqiui\Form\Item\IntItemForm;
use Kuoqi\Kuoqiui\Form\Item\ListItemForm;
use Kuoqi\Kuoqiui\Form\Item\RadioItemForm;
use Kuoqi\Kuoqiui\Form\Item\SelectItemForm;
use Kuoqi\Kuoqiui\Form\Item\TextareaItemForm;
use Kuoqi\Kuoqiui\Form\Item\TextItemForm;

/**
 * 一个form表单的字段的标识，基于ui.1.13及版本以上的显示
 * Class ItemForm
 * @package Kuoqi\Kuoqiui\Form
 */
class ItemForm extends ConfigForm {
    public $is_require = 0;
    public $type = "text";
    public $format = "flex";
    public $title = "";
    public $name = "";
    public $value = "";
    public $desc = "";
    public $tips = "";
    public $data = "";
    public $id = "";
    public $sort = "";
    public $status = "";
    public $defval = "";
    protected $config = array();
    /**
     * 一个字段的参数的初始化
     * ItemForm constructor.
     * @param array $opts
     */
    public function __construct($opts = array())
    {
        $this->config = \Kuoqi::opts($opts,array(
            'is_require'=>0,//是否必填，必填为1，非必填为0
            'type'=>'text',//使用的基础格式内容，需要符合基础格式的信息
            'format'=>'flex',//使用flex的布局方式
            'title'=>"",//当前字段默认使用as外部显示的名称，如果没有显示当前的name
            'name'=>"",//当前字段的保存名称
            'value'=>'',//当前字段的值
            'desc'=>'',//当前字段的描述信息，这个是用于单独的描述，与自动填补在表单的默认数据不同
            'tips'=>'',//这个是默认提示在内容选项区域的提示
            'data'=>'',//额外填充的数据的相关信息
            'id'=>'',//绑定在当前上面的唯一ID
            'sort'=>9,//
            'status'=>1,//
        ));
        $this->is_require = $this->config['is_require'];
        $this->type  = $this->config['type'];
        $this->format  = $this->config['format'];
        $this->title  = $this->config['title'];
        $this->name  = $this->config['name'];
        $this->value  = $this->config['value'];
        $this->desc  = $this->config['desc'];
        $this->tips  = $this->config['tips'];
        $this->data  = $this->config['data'];
        $this->id  = $this->config['id'];
        $this->sort  = $this->config['sort'];
        $this->status  = $this->config['status'];
    }

    //显示为视图的文件内容
    public function toView(){
        $items = array();
        $id  = 'select_'.rand(1000,9999);
        $items[] = new TextItemForm(array(
            'is_require'=>1,
            'title'=>'字段title',
            'name'=>"title",
            'value'=>$this->title,
            'format'=>'flex',
        ));
        $items[] = new TextItemForm(array(
            'is_require'=>1,
            'title'=>'字段name',
            'name'=>"name",
            'value'=>$this->name,
            'format'=>'flex',
        ));
        $items[] = new SelectItemForm(array(
            'is_require'=>0,
            'title'=>'字段布局',
            'name'=>"format",
            'value'=>$this->format,
            'format'=>'flex',
            'data'=>ConfigForm::$FORMAT_LIST_TIPS,
            'desc'=>'请选择待提交的字段类型',
        ));
        $items[] = new SelectItemForm(array(
            'is_require'=>0,
            'title'=>'字段类型',
            'name'=>"type",
            'value'=>$this->type,
            'format'=>'flex',
            'data'=>ConfigForm::$NAME_LIST_TIPS,
            'desc'=>'请选择待提交的字段类型',
            'id'=>$id,
        ));

        $stream = "";
        foreach ($items as $item){
            $stream .= $item->toView();
        }
        $stream .= '<div id="'.$id.'-extbox"></div>';
        $items = array();
        $items[] = new TextItemForm(array(
            'is_require'=>0,
            'title'=>'默认值',
            'name'=>"value",
            'value'=>$this->value,
            'tips'=>"表单项的默认值",
            'format'=>'flex',
        ));
        $items[] = new TextItemForm(array(
            'is_require'=>0,
            'title'=>'字段提示',
            'name'=>"tips",
            'value'=>$this->tips,
            'tips'=>"请设置字段的提示",
            'format'=>'flex',
        ));
        $items[] = new RadioItemForm(array(
            'title'=>'是否必填',
            'name'=>"is_require",
            'value'=>$this->is_require,
            'format'=>'flex',
            'desc'=>'请填写当前字段',
            'data'=>array(
                array("name"=>"是","value"=>"1"),
                array("name"=>"否","value"=>"0"),
            ),
        ));
        $items[] = new RadioItemForm(array(
            'title'=>'是否启用',
            'name'=>"status",
            'value'=>$this->status,
            'format'=>'flex',
            'desc'=>'请判断当前字段是否启用',
            'data'=>array(
                array("name"=>"是","value"=>"1"),
                array("name"=>"否","value"=>"0"),
            ),
        ));
        $items[] = new TextItemForm(array(
            'is_require'=>0,
            'title'=>'字段排序',
            'name'=>"sort",
            'value'=>$this->sort,
            'tips'=>"请填写字段的排序",
            'format'=>'flex',
        ));
        $items[] = new TextareaItemForm(array(
            'is_require'=>0,
            'title'=>'字段说明',
            'name'=>"desc",
            'value'=>$this->desc,
            'tips'=>"请填写字段的说明的相关信息",
            'format'=>'flex',
            'data'=>'6',//这个是额外加入的数据，目前为显示的行数
        ));
        foreach ($items as $item){
            $stream .= $item->toView();
        }
        //赠送一个javascript的内容，助力默认的内容
        $stream .= $this->joinJavascript($id);
        return $stream;
    }

    //展示表单的内容
    public function toCustom($data){
        return "暂未配置【".$this->type."】表单显示";
    }

    /**
     * 展示custom的内容
     */
    public function toCustomWithFlex($data = ""){
        \Kuoqi::error("暂未配置【".$this->type."】-flex表单显示");
    }

    //展示custom
    public function toCustomWithCard($data = ""){
        \Kuoqi::error("暂未配置[".$this->title."]【".$this->type."】-card表单显示");
    }
    //展示最简单的布局方式
    public function toCustomWithSimple($data = ""){
        return trim(sprintf('<div class="row-flex">
                <div class="col-auto"><span class="label pdl-none pdr-none">%s：</span></div>
                <div class="col-other"><span class="label pdl-none pdr-none">%s</span></div>
                </div>'
            ,$this->title
            ,$data
        ));
    }

    /**
     * toExtview 2024/4/29 获取某格式下的额外参数的配置信息
     * @param $format 使用的字段类型
     * @param $config 在类型下初始的配置信息
     * @return string
     */
    public function toExtview($format,$config){
        $stream = "";
        $items = array();
        if($format=="select"){
            $items[] = new ListItemForm(array(
                'is_require'=>0,
                'title'=>'选项内容',
                'name'=>"data",
                'value'=>$config['data'],
                'tips'=>"表单项的默认值",
                'format'=>'flex',
            ));
        }else if($format=="date"){
            $items[] = new RadioItemForm(array(
                'is_require'=>0,
                'title'=>'可选规则',
                'name'=>"data",
                'value'=>$config['data'],
                'format'=>'flex',
                'data'=>array(
                    array('name'=>"所有时段",'value'=>0),
                    array('name'=>"向后选择",'value'=>1),
                    array('name'=>"向前选择",'value'=>2),
                ),
            ));
        }else if($format=="textarea"){
            $items[] = new IntItemForm(array(
                'is_require'=>0,
                'title'=>'显示行数',
                'name'=>"data",
                'value'=>$config['data'],
                'format'=>'flex',
            ));
        }else if($format=="radio"){
            $items[] = new ListItemForm(array(
                'is_require'=>0,
                'title'=>'选项内容',
                'name'=>"data",
                'value'=>$config['data'],
                'tips'=>"表单项的默认值",
                'format'=>'flex',
            ));
        }else if($format=="image"){
            $items[] = new TextItemForm(array(
                'is_require'=>0,
                'title'=>'上传路径',
                'name'=>"data",
                'value'=>$config['data'],
                'tips'=>"请填写图片上传路径",
                'format'=>'flex',
            ));
        }else if($format=="images"){
            $items[] = new TextItemForm(array(
                'is_require'=>0,
                'title'=>'上传路径',
                'name'=>"data",
                'value'=>$config['data'],
                'tips'=>"请填写图片上传路径",
                'format'=>'flex',
            ));
        }else if($format=="file"){
            $items[] = new TextItemForm(array(
                'is_require'=>0,
                'title'=>'上传路径',
                'name'=>"data",
                'value'=>$config['data'],
                'tips'=>"请填写文件上传路径",
                'format'=>'flex',
            ));
        }else if($format=="files"){
            $items[] = new TextItemForm(array(
                'is_require'=>0,
                'title'=>'上传路径',
                'name'=>"data",
                'value'=>$config['data'],
                'tips'=>"请填写多文件上传路径",
                'format'=>'flex',
            ));
        }
        foreach ($items as $item){
            $stream .= $item->toView();
        }
        return trim($stream);
    }

    //返回字符串是关于当前接收到参数，然后进行一些处理的方式
    final function joinJavascript($id){
        $stream = '$Q("#'.$id.'-select").on("change",function(){
            var item = $Q(this).parents("div.row-flex");
            $Q.ajax({
                url:"",
                type:"post",
                dataType:"json",
                data:{
                    scene:"ext_item",format:$Q(this).val()
                },
                success:function(result){
                    if(result.status==1){
                        $Q(item).next().html(result.data);
                        app.sync();
                    }else{
                        $Q.msg(result.message);
                    }
                },
                error:function(){
                    $Q.msg("系统错误，请联系开发人员");
                }
            })
        })';
        return sprintf('<script>%s</script>',$stream);
    }

    //验证表单中当前是否某个字段满足要求
    public function valiForm($datas){
        if(isset($datas[$this->name])){
            //这个是字段存在的情况
            if($this->is_require){
                //如果当前是必填的时候
                if(strlen($datas[$this->name])==0){
                    $this->setError($this->title." 为必填项");
                    return false;
                }
            }
        }else{
            //这个是字段不存在的情况
            $this->setError("请提交[".$this->title."]的内容");
            return false;
        }
        return true;
    }

    /**
     * 设置当前节点的数据
     * @param $data 设置当前的item的数据内容
     * @return $this 返回当前itemForm的对象
     */
    public function setValue($value){
        $this->value = $value;
        return $this;
    }

    //设置当前的格式
    public function setFormat($format = ''){
        $list = self::$FORMAT_LIST_TIPS;
        $arr = array();
        foreach ($list as $item){
            $arr[] = $item['value'];
        }
        if(in_array($format,$arr)){
            $this->format = $format;
        }
        return $this;
    }

    static public function createByconfig($config){
        $config = \Kuoqi::opts($config,array(
            'is_require'=>0,//是否必填，必填为1，非必填为0
            'type'=>'text',//使用的基础格式内容，需要符合基础格式的信息
            'format'=>'flex',//使用flex的布局方式
            'title'=>"",//当前字段默认使用as外部显示的名称，如果没有显示当前的name
            'name'=>"",//当前字段的保存名称
            'value'=>'',//当前字段的值
            'desc'=>'',//当前字段的描述信息，这个是用于单独的描述，与自动填补在表单的默认数据不同
            'tips'=>'',//这个是默认提示在内容选项区域的提示
            'data'=>'',//额外填充的数据的相关信息
            'id'=>'',//绑定在当前上面的唯一ID
            'sort'=>9,//
            'status'=>1,//
        ));
        $classname = '\Kuoqi\Kuoqiui\Form\Item\\'.ucwords($config['type']).'ItemForm';
        return new $classname($config);
    }
}