<?php
namespace Kuoqi\Model;
use Kuoqi\FileDriver;
use Kuoqi\Format;
use Kuoqi\Config;
use Kuoqi\Model\Data\FieldDataModel;

/**
 * Class DataModel 数据模块的模型
 * Create By Kuoqi on 2024-07-23 10:24
 * @package Kuoqi\Model
 */
class DataModel extends \Kuoqi\Model{

    protected $_head = "";
    protected $_context = "";
    protected $path = "";
    protected $names = array();//字段的名称信息
    protected $fields = array();//字段的详细信息
    protected $lineseparator_char = 10;//行分隔符
    protected $wordseparator_char = 32;//字段分隔符
    protected $size = 10485760;//切片的大小,默认10M的文件
    public function __construct()
    {
        $this->path = \Kuoqi\Params::$MODEL_DATA_ROOT.'/'.\Kuoqi\Route::hump2code(get_called_class());
        $this->_head = $this->path.'.khead';
        $this->_context = $this->path.'.kcontext';

        //判断头文件是否存在
        $config = new FileDriver\ConfigFileDriver($this->_head);
        if(!$config->isValied()){
            $this->create();//如果头文件不存在则尝试自动创建文件信息
            $config->sync();//同步数据，也就是重载
        }
        //判断存储文件的内容大小，如果太大则直接进行备份
        $fileDriver = new FileDriver($this->_context);
        if($fileDriver->getSize()>$this->size){
            $nfile = $this->path.'.'.date('YmdHis').'.kcontext';
            $fileDriver->moveTo(dirname($nfile),basename($nfile));
        }
        $list = $config->toArray();
        foreach ($list as $k=>$v){
            $this->names[] = $k;
            $this->fields[$v['sort']] = new FieldDataModel($this->_head,$v);
        }
        ksort($this->fields);
    }

    /**
     * 尝试创建头文件的信息,在调用的过程中补全内容信息
     * 这里是如果配置文件不存在的情况下，调用的时候触发
     * create 2024-07-23
     */
    public function create(){}

    /**
     * 常用的增删改查,增加数据信息
     * insert 2024-07-23
     */
    public function insert($data = array()){
        $line = "";
        foreach ($this->fields as $field){
            $val = isset($data[$field->getName()])?$data[$field->getName()]:"";
            $field->setValue($val);
            if($field->valiValue()){
                if($line){
                    $line .= chr($this->wordseparator_char).$field->getValue();
                }else{
                    $line = $field->getValue();
                }
            }else{
                $line .=  chr($this->wordseparator_char)."";
            }
        }
        $fileDriver = new FileDriver($this->_context);
        $fileDriver->append($line.chr($this->lineseparator_char));
    }

    /**
     * 增加一个字段到现在已经有的信息中
     * addField 2024-07-23
     * @param $field_name
     * @return FieldDataModel
     * @throws \Kuoqi\Error
     */
    public function addField($field_name){
        return new Data\FieldDataModel($this->_head,$field_name);
    }

}