<?php
namespace Kuoqi\Model;
/**
 * 关于一个数据表的字段的相关配置
 * Class DbFieldModel
 * @package Kuoqi\Model
 */
class DbFieldModel extends \Kuoqi\Model{
    //创建表的信息，如果是更新字段的话也请在这里面进行操作
    public $table = null;//数据表的相关信息
    protected $_name = '';//字段名称
    protected $_type = '';//类型
    protected $_comment = '';//备注
    protected $_default = "NULL";//默认的值
    protected $_isnull = true;//是否可以为空
    protected $_auto_increment = false;//是否自动递增，当为索引的时候拥有自动递增功能
    protected $_unsigned = false;//当为数字的时候是否无符号
    public $stream = '';//成品的字段信息

    /**
     * DbFieldModel constructor.
     * @param $name 字段名
     * @param array $options 字段参数属性配置
     */
    public function __construct($name,$options = array())
    {
        $this->_name = $name;
    }
    /**
     * 配置自动增加
     * @param int $isTruelen 如果有值则设置递增数据的长度，默认int类型，可使用setType修改，等同于快速设置
     * @return $this
     */
    public function autoIncrement($isTruelen = 10)
    {
        $isTruelen = abs(intval($isTruelen));
        if($isTruelen){
            $this->_auto_increment = true;
            $this->_type = sprintf('int(%s)',$isTruelen);
            $this->_unsigned = true;
            $this->_isnull = false;
        }else{
            $this->auto_increment = false;
        }
        return $this;
    }
    /**
     * 获取type
     * @return string
     */
    public function getType()
    {
        return $this->_type;
    }
    /**
     * 设置type
     * @param $v type的值
     * @return $this
     */
    public function setType($v)
    {
        $this->_type = $v; return $this;
    }
    /**
     * 设置默认的值
     * @param $default 默认值
     * @return $this
     */
    public function Def($default = NULL){
        if($default===NULL){
            $this->_default = 'NULL';
        }else if($default===""){
            $this->_default = "''";
        }else{
            $this->_default = $default;
        }
        return $this;
    }
    /**
     * 设置类型为varchar
     * @param int $len
     * @return $this
     */
    public function varchar($len = 255){
        $this->_type = sprintf('varchar(%s)',intval($len));
        return $this;
    }
    /**
     * 设置类型为char
     * @param int $len
     * @return $this
     */
    public function char($len = 255){
        $this->_type = sprintf('char(%s)',intval($len));
        return $this;
    }
    /**
     * 设置类型为int
     * @param int $len
     * @return $this
     */
    public function int($len = 11){
        $this->_type = sprintf('int(%s)',intval($len));
        return $this;
    }
    /**
     * 设置类型为float
     * @param int $len
     * @param int $ispointlen
     * @return $this
     */
    public function float($len = 6,$ispointlen = 2){
        $this->_type = sprintf('float(%s,%s)',intval($len),intval($ispointlen));
        return $this;
    }

    /**
     * 判定为text
     * text 2024-07-30
     * @return $this
     */
    public function text(){
        $this->_type = 'text';
        return $this;
    }
    /**
     * 设置是否无符号，type为int系列时有效
     * @param bool $isUnsigned
     * @return $this
     */
    public function unsigned($isUnsigned = true){
        $this->_unsigned = $isUnsigned;
        return $this;
    }
    /**
     * 设置备注信息
     * @param $comm
     * @return $this
     */
    public function comment($comm){
        $this->_comment = $comm;
        return $this;
    }
    //这个已经作为基础的结束方法，使用这个没有任何作用，自是当做一个标识符
    public function create(){
        return $this;

    }

    //创建表格的时候的sql
    public function sqlCreate(){
        if($this->_isnull){
            $default = "DEFAULT {$this->_default}";
        }else{
            $default = 'NOT NULL';
        }
        $stream = trim(sprintf("`%s` %s %s %s %s %s"
            ,$this->_name
            ,$this->_type
            ,$this->_unsigned?'unsigned':''
            ,$default
            ,$this->_auto_increment?'AUTO_INCREMENT':""
            ,$this->_comment?"COMMENT '{$this->_comment}'":""
        ));
        $stream = preg_replace("/\s+/"," ",$stream);
        return $stream;
    }

    //返回单独写入的sql语句
    //ALTER TABLE <表名> ADD <新字段名> <数据类型> [约束条件] AFTER <已经存在的字段名>;
    public function sqlInsert($after = ''){
        if($this->_isnull){
            $default = "DEFAULT {$this->_default}";
        }else{
            $default = 'NOT NULL';
        }
        $stream = trim(sprintf("`%s` %s %s %s %s %s"
            ,$this->_name
            ,$this->_type
            ,$this->_unsigned?'unsigned':''
            ,$default
            ,$this->_auto_increment?'AUTO_INCREMENT':""
            ,$this->_comment?"COMMENT '{$this->_comment}'":""
        ));
        $stream = preg_replace("/\s+/"," ",$stream);
        $sql = sprintf('ALTER TABLE `%s` ADD %s%s'
            ,\Kuoqi\Config::get("DB_PREFIX").$this->table->name
            ,$stream
            ,$after?(' AFTER `'.$after.'`'):''
        );
        return $sql;
    }

}