<?php
namespace Kuoqi\Model;
use Kuoqi\Config;
use Kuoqi\Error;

/**
 * 关于配置一个数据表，放到系统中就直接可以使用的那种
 * Class DbTableModel
 * @package Kuoqi\Model
 */
class DbTableModel extends \Kuoqi\Model{
    public $name = '';//数据表名，排除前缀的部分
    //public $field = array();//数据表的字段列表
    public $_field = array();//数据表的字段列表-的对象信息
    public $engine = '';//默认的排序引擎
    public $charset = '';//字符串信息
    public $collate = '';//核对编码
    public $comment = '';//说明信息
    protected $primary_key = '';//索引信息
    protected $db = null;//当前链接的DB
    protected $dbname = '';//绑定的当前数据库主机名

    public function __construct()
    {

        $this->engine = Config::get("DBTABLE_ENGINE");
        $this->charset = Config::get("DBTABLE_CHARSET");
        $this->collate = Config::get("DBTABLE_COLLATE");
        $dbDriver = new \Kuoqi\DbDriver();//使用链接数据库对象信息
        $this->dbname = $dbDriver->dbname;
        $this->db = $dbDriver->connect();
        //将字段的信息写入到表的字段中
        $this->_field = $this->getAttrlist();
        parent::__construct();
    }

    //尝试获取属性的字段的相关信息
    protected function getAttrlist(){ return array();}

    //设置字段
    public function setField(){
        //todo--完善设置字段的方法
    }

    /**
     * 增加字段
     * @param $field 字段名
     * @param array $options 字段的参数信息
     * @return DbFieldModel 字段对象信息
     */
    public function addField($field,$options = array()){
        $this->_field[$field] = new DbFieldModel($field,$options);
        $this->_field[$field]->table = $this;
        return $this->_field[$field];
    }
    //获取字段对象
    public function getField(){
        //todo--完善获取字段
        throw new Error("todo--完善增加再短");
    }
    //删除字段
    public function delField(){
        //todo--完善删除字段
        throw new Error("todo--完善增加再短");
    }
    /**
     * 设置索引信息，支持字符串与数组输入
     * @param $primaryKey 索引设置
     */
    public function PrimaryKey($primaryKey){

        $this->primary_key = $primaryKey;
    }

    //判断当前数据表是否存在
    public function exist(){
        $sql = sprintf('SELECT count(*) FROM `INFORMATION_SCHEMA`.`TABLES` WHERE (`TABLE_SCHEMA` = "%s" and `TABLE_NAME` = "%s")'
            ,$this->dbname
            ,Config::get("DB_PREFIX").$this->name
        );
        $query = $this->db->query($sql);
        if($query){
            $count = intval($query->fetchColumn());
            return $count?true:false;
        }else{
            //查询失败的情况下
            $this->errmsg = "查询失败";
            return false;
        }
    }

    //数据表的删除
    //这个操作方法相当的危险，慎用慎用
    public function delete(){
        $sql = sprintf('DROP TABLE IF EXISTS `%s`',Config::get("DB_PREFIX").$this->name);
        $this->db->query($sql);
    }

    //创建表的信息，如果是更新字段的话也请在这里面进行操作
    public function create(){
        $list = array();
        foreach ($this->_field as $v){
            $list[] = $v->sqlCreate();
        }
        if(is_array($this->primary_key)){
            if(count($this->primary_key)>1){
                $tmp = "";
                foreach ($this->primary_key as $k=>$v){
                    $tmp .= $tmp?",`{$v}``":"`{$v}`";
                }
                $list[] = "PRIMARY KEY ({$tmp})";
            }else if(count($this->primary_key)==1){
                $list[] = "PRIMARY KEY (`{$this->primary_key[0]}`) USING BTREE";
            }
        }else if($this->primary_key){
            $list[] = "PRIMARY KEY (`{$this->primary_key}`) USING BTREE";
        }
        $stream = implode(",".PHP_EOL,$list);
        //判定索引信息
        $stream = trim(sprintf("CREATE TABLE `%s` (%s) ENGINE=%s CHARSET=%s COLLATE=%s %s"
            ,Config::get("DB_PREFIX").$this->name
            ,$stream
            ,$this->engine
            ,$this->charset
            ,$this->collate
            ,$this->comment?"COMMENT='{$this->comment}'":""
        ));
        //进行一个反馈
        $rs = $this->db->query($stream);
        if($rs===false){
            throw new \Kuoqi\Error("创建失败：");
        }
    }

    /**
     * 判断数据表是否存在
     * isExist 2024-07-30
     * @return bool
     */
    static public function isExist(){
        $clsname = get_called_class();
        $table = new $clsname();
        $sql = sprintf('SELECT count(*) FROM `INFORMATION_SCHEMA`.`TABLES` WHERE (`TABLE_SCHEMA` = "%s" and `TABLE_NAME` = "%s")'
            ,$table->dbname
            ,Config::get("DB_PREFIX").$table->name
        );
        $query = $table->db->query($sql);
        if($query){
            $count = intval($query->fetchColumn());
            return $count?true:false;
        }else{
            //查询失败的情况下
            $table->errmsg = "查询失败";
            return false;
        }
    }

    //自动完善一个table的相关信息,目前暂时没有完善的日志系统追踪
    static public function perfect(){
        $clsname = get_called_class();
        $table = new $clsname();
        //对比现在是否已经存在数据表
        $sql = sprintf('SELECT COLUMN_NAME as colname FROM INFORMATION_SCHEMA.COLUMNS WHERE TABLE_SCHEMA = \'%s\' and TABLE_NAME = \'%s\'',
            $table->dbname,
            Config::get("DB_PREFIX").$table->name
        );
        $query = $table->db->query($sql);
        if($query){
            //数据存在
            $data = $query->fetchAll(\PDO::FETCH_ASSOC);
            if($data){
                $cols = array();//重数据库查询出来的结果
                foreach ($data as $item){
                    $cols[] = $item['colname'];
                }
                //获取最新的字段的相关信息
                $lcode = '';
                foreach ($table->_field as $colk=>$col){
                    if(in_array($colk,$cols)){
                        //字段存在，不做任何处理
                    }else{
                        //字段不存在，需要新增的部分
                        $table->db->query($col->sqlInsert($lcode));
                    }
                    $lcode = $colk;
                }
            }else{
                //数据表不存在
                $table->create();
            }
        }else{
            //数据不存在
        }
    }

}