<?php
namespace Kuoqi\Pack;
use Kuoqi\Pack;
use Kuoqi\PathDriver;
use ZipArchive;
use Kuoqi\Error;

/**
 * Class Phpr 一个常规的php软件包
 * @package Kuoqi
 */
class Zip extends Pack {
    protected $active = null;
    protected $active_file = "";//当前绑定的文件
    protected $file_driver = "";//
    static protected $_ERR_CODE = array(
        \ZipArchive::ER_SEEK=>'Seek error',
        \ZipArchive::ER_READ=>'Read error',
        \ZipArchive::ER_NOENT=>'No such file',
        \ZipArchive::ER_OPEN=>"Can't open file",
        \ZipArchive::ER_EXISTS=>'File already exists',
        \ZipArchive::ER_MEMORY=>'Malloc failure',
        \ZipArchive::ER_INVAL=>'Invalid argument',
        \ZipArchive::ER_NOZIP=>'Not a zip archive',
        \ZipArchive::ER_INCONS=>'Zip archive inconsistent',
    );
    function ZipStatusString( $status )
    {
        switch( (int) $status )
        {
            case ZipArchive::ER_OK           : return 'N No error';
            case ZipArchive::ER_MULTIDISK    : return 'N Multi-disk zip archives not supported';
            case ZipArchive::ER_RENAME       : return 'S Renaming temporary file failed';
            case ZipArchive::ER_CLOSE        : return 'S Closing zip archive failed';
            case ZipArchive::ER_SEEK         : return 'S Seek error';
            case ZipArchive::ER_READ         : return 'S Read error';
            case ZipArchive::ER_WRITE        : return 'S Write error';
            case ZipArchive::ER_CRC          : return 'N CRC error';
            case ZipArchive::ER_ZIPCLOSED    : return 'N Containing zip archive was closed';
            case ZipArchive::ER_NOENT        : return 'N No such file';
            case ZipArchive::ER_EXISTS       : return 'N File already exists';
            case ZipArchive::ER_OPEN         : return 'S Can\'t open file';
            case ZipArchive::ER_TMPOPEN      : return 'S Failure to create temporary file';
            case ZipArchive::ER_ZLIB         : return 'Z Zlib error';
            case ZipArchive::ER_MEMORY       : return 'N Malloc failure';
            case ZipArchive::ER_CHANGED      : return 'N Entry has been changed';
            case ZipArchive::ER_COMPNOTSUPP  : return 'N Compression method not supported';
            case ZipArchive::ER_EOF          : return 'N Premature EOF';
            case ZipArchive::ER_INVAL        : return 'N Invalid argument';
            case ZipArchive::ER_NOZIP        : return 'N Not a zip archive';
            case ZipArchive::ER_INTERNAL     : return 'N Internal error';
            case ZipArchive::ER_INCONS       : return 'N Zip archive inconsistent';
            case ZipArchive::ER_REMOVE       : return 'S Can\'t remove file';
            case ZipArchive::ER_DELETED      : return 'N Entry has been deleted';

            default: return sprintf('Unknown status %s', $status );
        }
    }
    public function __construct($filename = "")
    {
        parent::__construct();
        $this->active = new \ZipArchive();
        if($filename){
            $this->active_file = $filename;
        }else{
            $this->active_file = "{@APP}/runtime/tmpfile/".time().rand(1000,9999).".zip";
        }
        $this->file_driver = new \Kuoqi\FileDriver($this->active_file);
        $this->file_driver->checkPath();
        if($this->file_driver->exist()){
            $res = $this->active->open(\Kuoqi\Route::str2rootpath($this->active_file),\ZipArchive::FL_COMPRESSED );
            if($res===true){
            }else{
                throw new Error('\Kuoqi\Package\Zip : failed, code is ' . $res .', error is '.self::$_ERR_CODE[$res]);
            }
        }else{
            $res = $this->active->open(\Kuoqi\Route::str2rootpath($this->active_file),\ZipArchive::CREATE );
            if($res===true){
            }else{
                throw new Error('\Kuoqi\Package\Zip : failed, code is ' . $res .', error is '.self::$_ERR_CODE[$res]);
            }
        }

    }

    /**
     * 清空ZIP包的所有内容
     * clear 2024-07-30
     */
    public function clear()
    {
        for($i=0;$i<$this->active->numFiles;$i++){
            $this->active->deleteIndex($i);
        }
    }

    //创建文件信息
    public function create(){
        $fileDriver = (new \Kuoqi\FileDriver($this->active_file));
        if(!$fileDriver->exist()){
            var_dump($this->active->getStream());exit();
            //$fileDriver->save();
        }
    }

    /**
     * 以文件的方式增加
     * @param $lcoalname
     * @param $file
     */
    public function putFile($lcoalname,$file){
        $this->active->addFile(\Kuoqi\Route::str2rootpath($file),$lcoalname);
    }

    /**
     * 增加目录，或者将内容压缩到目录中
     * putPath 2024-07-30
     * @param $localname
     * @param string $orddir
     */
    public function putPath($localname,$orddir = ''){
        if($orddir){
            $pathDriver = new PathDriver($orddir);
            $files = $pathDriver->getFiles();
            foreach ($files as $file){
                $nfile = str_replace($pathDriver->getDir(),$localname,$file);
                $this->putFile($nfile,$file);
            }
        }else{
            $this->active->addEmptyDir($localname);
        }
    }

    /**
     * 文件流输入到压缩包中
     * putText 2024-09-06
     * @param $localname
     * @param $context
     */
    public function putText($localname,$context){
        $this->active->addFromString($localname,$context);
    }

    /**
     * 获取文件
     * @return string
     */
    public function getFile(){
        return $this->active_file;
    }

    /**
     * 判定是否存在
     * @return bool
     */
    public function exise(){
        return (bool)$this->active->numFiles;
    }

    /**
     * 关闭ZIP文档
     * close 2024-07-30
     */
    public function close(){
        if($this->active){
            $this->active->close();
            $this->active = null;
        }
    }

    public function toFile(){
        return $this->file_driver;
    }

    public function __destruct()
    {
        if($this->active){
            $this->active->close();
            $this->active = null;
        }
//        if($this->active_file){
//            @unlink(\Kuoqi\Route::str2rootpath($this->active_file));
//            $this->active_file = "";
//        }
    }
}