<?php
namespace Kuoqi\Process;
use Kuoqi\Error;

/**
 * 使用进程的执行文件
 * Class GuardProcess
 * @package Kuoqi
 */
//守护进程使用主备模式进行守护，
//守护进程下可开很多很多的业务进程，业务进程使用PHP脚本的方式进行，不加渲染KuoqiPHP的色彩
class GuardProcess extends \Kuoqi\Process {
    public $period_time = 60;
    //已经需要激活的进程的相关信息
    protected $process_list = array();
    protected $pid = 'guard.pid';
    //每一个进程都有一个守护文件，根据守护文件判定该进程是否存活，守护进程执行start则为守护进程启动，收回进程执行end则关闭当前守护进程
    public function __construct($pid = '')
    {
        if($pid){
            $this->pid = $pid;
        }
        parent::__construct();
    }

    //开启守护进程
    public function start(){
        \Kuoqi::warn("welcome to use this");
        \Kuoqi::warn(sprintf("pid is %s",$this->pid));
        \Kuoqi::warn(sprintf("period timelen is %s",$this->period_time));
        while (true){
            \Kuoqi::warn(sprintf('guard start with %s and %s',date('Y-m-d H:i:s'),memory_get_usage()));
            $this->configRead();
            $this->guardProcessall();
            $this->guardProcessAdd();
            $this->guardProcessCancel();
            $this->configSync();
            \Kuoqi::warn(sprintf('guard return with %s and %s',date('Y-m-d H:i:s'),memory_get_usage()));
            sleep($this->period_time);
        }
    }
    //增加待守护的进程内容
    public function add($file,$opts = array()){
        $process = new ItemProcess($file,$opts);
        $this->process_list[] = $process;
    }
    //删除一个进程信息

    /**
     * 根据PID关闭一个进程信息
     * @param $pid
     */
    public function cancelByid($id){
        foreach ($this->process_list as $k=>$item){
            //触发ITEM下的守护进程代码
            if($id == $item->getId()){
                //删除进程信息，然后
                $item->cancelProcess();
                unset($this->process_list[$k]);
                foreach ($this->config['process'] as $pc=>$pv){
                    if($pv['id']==$id){
                        unset($this->config['process'][$pc]);
                    }
                }
            }
        }
    }

    //保护所有的进程的相关信息
    protected function guardProcessall(){
        $this->config['process'] = array();
        foreach ($this->process_list as $k=>$item){
            //触发ITEM下的守护进程代码
            $item->guardProcess($item);
            $this->config['process'][$item->getID()] = $item->getInfo();
        }
    }
    //进程的新增
    protected function guardProcessAdd(){
        if(isset($this->config['process_add'])&&is_array($this->config['process_add'])){
            foreach ($this->config['process_add'] as $k=>$v){
                $this->add($v['file'],$v['opts']);
            }
            unset($this->config['process_add']);
        }
    }
    //进程的删除
    protected function guardProcessCancel(){
        if(isset($this->config['process_cancel'])&&is_array($this->config['process_cancel'])){
            foreach ($this->config['process_cancel'] as $k=>$v){
                $this->cancelByid($v['id']);
            }
            unset($this->config['process_cancel']);
        }
    }

    //静态方法

    /**
     * 根据pid文件新增一个
     * @param $file
     * @param array $opts
     */
    public function pidWithadd($file,$opts = array()){
        $this->configRead();
        if(isset($this->config['process_add'])&&is_array($this->config['process_add'])){
            $this->config['process_add'][] = array(
                'file'=>$file,
                'opts'=>$opts,
            );
        }else{
            $this->config['process_add'] = array();
            $this->config['process_add'][] = array(
                'file'=>$file,
                'opts'=>$opts,
            );
        }
        $this->configSync();
    }

    /**
     * 根据PID文件删除一个
     * @param $id
     * @param array $opts
     */
    public function pidWithcancel($id){
        $this->configRead();
        if(isset($this->config['process_cancel'])&&is_array($this->config['process_cancel'])){
            $this->config['process_cancel'][] = array(
                'id'=>$id,
            );
        }else{
            $this->config['process_cancel'] = array();
            $this->config['process_cancel'][] = array(
                'id'=>$id,
            );
        }
        $this->configSync();
    }
}