<?php
namespace Kuoqi\Socket;

/**
 * 行为基础脚本信息
 * Class WebSocket
 * @package Kuoqi\Worker
 */
class Behavior extends  \Kuoqi\KuoqiException {
    public static $MODE_TEXT = 129;
    public static $MODE_BIN = 130;
    protected $token = "123456";
    protected $package_size = 8196;//每一个包的长度
    protected $server_addr = '0.0.0.0';
    protected $server_port = '';
    protected $master = null;//当前绑定的master的链接信息
    protected $debug = true;
    protected $mode = 130;//使用模式，129：
    public function __construct($address,$port,$option = array()){
        $this->mode = self::$MODE_BIN;
        $this->server_addr = $address;
        $this->server_port = $port;
        set_time_limit(0);
        ignore_user_abort(0);
        //$this->lister->serverConstruct();
    }

    /**
     * @return bool 创建一个通讯节点
     */
    protected function initCreate(){
        // resource socket_create( int $domain, int $type, int $protocol)
        //Domain
        //描述
        //AF_INET IPv4 网络协议。TCP 和 UDP 都可使用此协议。
        //AF_INET6 IPv6 网络协议。TCP 和 UDP 都可使用此协议。
        //AF_UNIX 本地通讯协议。具有高性能和低成本的 IPC（进程间通讯）。
        //type
        //SOCK_STREAM 提供一个顺序化的、可靠的、全双工的、基于连接的字节流。支持数据传送流量控制机制。TCP 协议即基于这种流式套接字。
        //SOCK_DGRAM 提供数据报文的支持。(无连接，不可靠、固定最大长度).UDP协议即基于这种数据报文套接字。
        //SOCK_SEQPACKET 提供一个顺序化的、可靠的、全双工的、面向连接的、固定最大长度的数据通信；数据端通过接收每一个数据段来读取整个数据包。
        //SOCK_RAW 提供读取原始的网络协议。这种特殊的套接字可用于手工构建任意类型的协议。一般使用这个套接字来实现 ICMP 请求（例如 ping）。
        //SOCK_RDM 提供一个可靠的数据层，但不保证到达顺序。一般的操作系统都未实现此功能。
        //protocal
        //icmp Internet Control Message Protocol 主要用于网关和主机报告错误的数据通信。例如"ping"命令（在目前大部分的操作系统中）就是使用 ICMP 协议实现的。
        //udp User Datagram Protocol 是一个无连接的、不可靠的、具有固定最大长度的报文协议。由于这些特性，UDP 协议拥有最小的协议开销。
        //tcp Transmission Control Protocol 是一个可靠的、基于连接的、面向数据流的全双工协议。TCP 能够保障所有的数据包是按照其发送顺序而接收的。如果任意数据包在通讯时丢失，TCP 将自动重发数据包直到目标主机应答已接收。因为可靠性和性能的原因，TCP 在数据传输层使用 8bit 字节边界。因此，TCP 应用程序必须允许传送部分报文的可能。
        if(!($this->master = socket_create(AF_INET,SOCK_STREAM,SOL_TCP))){
            $this->setError("socket_create() failed");
            return false;
        }

        return true;
    }


    //监听一个节点
    protected function initListen($backlog = 0){

        //bool socket_bind( resource $socket, string $address[, int $port = 0] )
        //绑定 address 到 socket。该操作必须是在使用 socket_connect() 或者 socket_listen() 建立一个连接之前。
        //socket
        //用 socket_create() 创建的一个有效的套接字资源。
        //address
        //如果套接字是 AF_INET 族，那么 address 必须是一个四点分法的 IP 地址（例如 127.0.0.1 ）。
        //
        //如果套接字是 AF_UNIX 族，那么 address 是 Unix 套接字一部分（例如 /tmp/my.sock ）。
        //port （可选）
        //参数 port 仅仅用于 AF_INET 套接字连接的时候，并且指定连接中需要监听的端口号。
        if(!socket_bind($this->master,$this->server_addr,$this->server_port)){
            $this->setError(sprintf('socket_bind failed on %s:%s with %s'."\n",
                $this->server_addr,
                $this->server_port,
                iconv('GBK','UTF-8',socket_strerror(socket_last_error()))));
            return false;
        }
        socket_set_option($this->master, SOL_SOCKET, SO_REUSEADDR, 1);
        //TCP_NODELAY报告Nagle TCP算法是否被禁用。
        socket_set_option($this->master, SOL_SOCKET, TCP_NODELAY, 1);
        //Rep报告输入操作的超时值.
        socket_set_option($this->master, SOL_SOCKET, SO_RCVTIMEO, array('sec' => 5, 'usec' => 0));

        socket_set_option($this->master, SOL_SOCKET, SO_LINGER, array('l_linger' => 0, 'l_onoff' => 1));
        //报告超时值，该超时值指定由于流控制阻止发送数据而导致输出功能阻塞的时间.
        socket_set_option($this->master, SOL_SOCKET, SO_SNDTIMEO, array('sec' => 5, 'l_onoff' => 0));
        //报告发送缓冲区的大小。.
        socket_set_option($this->master, SOL_SOCKET, SO_SNDBUF, 4196);
        //报告接收缓冲区的大小.
        socket_set_option($this->master, SOL_SOCKET, SO_RCVBUF, 4196);
        //报告有关错误状态的信息并清除它
        socket_set_option($this->master, SOL_SOCKET, SO_ERROR, 1);
        if(!socket_listen($this->master,$backlog)){
            $this->setError("socket_listen() failed");
            return false;
        }
        return true;
    }

    //链接一个节点
    protected function initConnect(){
        //bool socket_connect( resource $socket, string $address[, int $port = 0] )
        if(!socket_connect($this->master,$this->server_addr,$this->server_port)){
            $this->setError(sprintf('socket_connect failed on %s:%s with %s'."\n",
                $this->server_addr,
                $this->server_port,
                iconv('GBK','UTF-8',socket_strerror(socket_last_error()))));
            return false;
        }
    }

    protected $event = array(
        'login'=>'',//客户端链接的时候执行本方法内容
        'logout'=>'',//关闭时候执行
        'listen'=>'',//关联listen函数做一次数据处理的信息返回
        'message'=>'',//接收到消息的时候执行
    );

    public function bind($event,$function){
        $this->event[$event] = $function;
    }

    /**
     * 开始执行
     */
    public function start(){
       //请初始化开始方法
    }
    /**
     * 执行运行
     */
    public function run(){
        while(true){
            $this->listen();
        }
    }

    public function listen($time_sec = null){}

    public function debug($msg){
        echo date('Y-m-d H:i:s').'['.memory_get_usage().']'.$msg.PHP_EOL;
    }
    //握手协议的相关信息
    protected function handshake(){
        $this->debug("请自定义握手协议的内容");
    }

    /**
     * @param $context 推送字符串的相关信息
     */
    public function postContext($context){
        socket_write($this->master,$context,strlen($context));
    }

    //设置mode
    public function setMode($code){
        $this->mode = $code;
    }

}