<?php
namespace Kuoqi\Socket;

/**
 * 服务端基础脚本
 * Class WebSocket
 * @package Kuoqi\Worker
 */
class Socket extends  \Kuoqi\Socket\Behavior {
    protected $sockets = array();
    protected $_block_timesec = null;//设置当前循环的时间间隔
    protected $_bind_message = array();
    protected $_bind_connect = array();
    public function __construct($address,$port,$option = array()){
        parent::__construct($address,$port,$option);
    }

    protected function init(){
        //创建并返回一个套接字，也称作一个通讯节点。一个典型的网络连接由 2 个套接字构成，一个运行在客户端，另一个运行在服务器端。
       $this->initCreate()&&$this->initListen();
       $this->sockets[] = $this->master;
    }

    /**
     * @param null $timelen 设置listen的是否阻塞，或者设置阻塞的时间
     */
    public function setBlocktime($timelen = null){
        $this->_block_timesec = $timelen;
    }

    /**
     * 开始执行
     */
    public function start(){
        $this->init();
        if(!$this->isValied()){
            throw new \Kuoqi\Error("异常:".$this->getError());
        }
        $this->run();
    }
    /**
     * 执行运行
     */
    public function run(){
        while(true){
            $this->listen();
        }
    }

    /**
     * 运行监听
     */
    public function listen($time_sec = null){
        if(is_callable($this->event['listen'])){
            call_user_func($this->event['listen'],$this);
        }
        $socketArr = $this->sockets;$write = NULL;$except = NULL;
        //自动选择来消息的socket 如果是握手 自动选择主机
        //int socket_select ( array &$read , array &$write , array &$except , int $tv_sec [, int $tv_usec = 0 ] )
        //无线阻塞
        //tv_sec : 0 为非阻塞模式
        //tv_sec : null 为完全非阻塞模式
        if(false ===socket_select($socketArr, $write, $except, $this->_block_timesec)){
            \Kuoqi::warn(sprintf("socket_select failed: %s"
                ,iconv('GBK','UTF-8',socket_strerror(socket_last_error()))));
        }
        foreach ($socketArr as $k=>$socket){
            if ($socket == $this->master){
                //如果是关于主机的链接
                $client = socket_accept($this->master);
                //准备链接的时候已经断了，这个不管
                if ($client === false){
                    \Kuoqi::warn("客户端断开");
                    continue;
                }
                //执行链接一个客户端的操作
                $this->doConnect($client);
            }else{
                //$this->debug("收到消息");
                //如果是关于客户端的一些链接发送信息类
                $bytes = socket_recv($socket,$buffer,$this->package_size,0);
                if ($bytes == 0){
                    $this->doDisconnect($socket);//这个客户端突然发送一些为空的数据，标识自动断开了，GG
                }else{
                    $this->onMessage($buffer,$socket);
                }
            }
        }
    }

    /**
     * 发送给客户端的相关信息
     * @param $client
     * @param $msg
     * @return int
     */
    public function postMessage($client,$msg){
        if(is_array($msg)){
            $msg = \Kuoqi\Helper::jsonencode($msg);
        }
        $msg = \Kuoqi\Secret::datapass_encode($msg,$this->mode);
        if(socket_write($client, $msg, strlen($msg))){
            return true;
        }else{
            $this->debug(sprintf("发送到%s失败:%s",$client,iconv('GBK','UTF-8',socket_strerror(socket_last_error()))));
            return false;
        }
    }

    /**
     * 执行客户端链接的相关操作
     * @param $socket
     * @return bool|void
     */
    function doConnect($client){
        $bytes = socket_recv($client,$buffer,$this->package_size,0);
        if($bytes==0){
            $this->debug("soekct_connect:未发送任何链接信息");
            return;
        }else{
            //这里是握手协议的头部信息
            $buffer = \Kuoqi\Secret::datapass_decode($buffer,$this->mode);
            if($buffer){
                $buffer = $buffer['stream'];
            }else{
                $this->debug("soekct_connect:错误,无法解码字符串");
                return;
            }
            $protocal = new \Kuoqi\Input\ProtocolInput($buffer);
            if($protocal->valihost('handshake')){
                $token = $protocal->get("token");
                if($token==$this->token){
                    array_push($this->sockets, $client);
                    $this->onConnect($client);
                }else{
                    $this->debug("客户端 ".$client . "尝试登入，失败,token验证失败:token=".$token);
                }
            }else{
                $this->debug("客户端 ".$client . "登录失败,协议解读失败:".$protocal->getError());
            }
        }
    }

    /**
     * 断开链接信息
     * @param $socket
     */
    function doDisconnect($socket){
        $index = array_search($socket, $this->sockets);
        socket_close($socket);
        if ($index >= 0){
            unset($this->sockets[$index]);
            if(is_callable($this->event['logout'])){
                call_user_func($this->event['logout'],$this);
            }else{
                $this->debug("客户端 ".$socket . " 于 ".date("Y-n-d H:i:s").' 登出,内存大小为:'.memory_get_usage());
            }
        }
    }

    /**
     * 获取sockets列表
     * @return array
     */
    public function getSockets(){
        return $this->sockets;
    }

    /**
     * 获取sockets的列表，只能排除某一个客户端
     * @param $exclude_client
     * @return array
     */
    public function getClients($exclude_client = null){
        $array = array();
        foreach ($this->sockets as $item){
            //排除当前进程
            if($item == $exclude_client){ continue;}
            //排除主进程信息
            if($item==$this->master){ continue;}
            $array[] = $item;
        }
        return $array;
    }

    //获取客户端的数据数量列表
    public function countClient(){
        return count($this->getClients());
    }

    /**
     * @param $callfunc 收到消息的函数
     */
    public function onMessage($callfunc,$_client = '',$options = array()){
        if(is_callable($callfunc)){
            $this->_bind_message[] = $callfunc;
        }else{
            $buffer = \Kuoqi\Secret::datapass_decode($callfunc,$this->mode);
            $stream = $buffer['stream'];
            $len = $buffer['len'];
            while ($len>strlen($stream)){
                $stream .= socket_read($_client,$this->package_size);
            }
            //\Kuoqi::warn("收到文件，长度为：".strlen($stream));
            foreach ($this->_bind_message as $item){
                call_user_func($item,$stream,$_client,$this,$options);
            }
        }
    }

    //客户端登录成功
    public function onConnect($callfunc){
        if(is_callable($callfunc)){
            $this->_bind_connect[] = $callfunc;
        }else{
            foreach ($this->_bind_connect as $item){
                call_user_func($item,$callfunc,$this);
            }
        }
    }
}