<?php
namespace Kuoqi\Worker\Httpd;
use Kuoqi\Url;

/**
 * 对于请求的数据进行的同意回复
 * Class Request
 * @package Kuoqi\Worker
 */
class Response extends \Kuoqi\KuoqiException {
    public $request = null;//请求的对象
    public $httpd = null;//当前的httpd对象
    public $mime_types=array(
        'avi'=>'video/x-msvideo',
        'bmp'=>'image/bmp',
        'css'=>'text/css',
        'doc'=>'application/msword',
        'gif'=>'image/gif',
        'htm'=>'text/html',
        'html'=>'text/html',
        'ico'=>'image/x-icon',
        'jpe'=>'image/jpeg',
        'jpeg'=>'image/jpeg',
        'jpg'=>'image/jpeg',
        'js'=>'application/x-javascript',
        'mpeg'=>'video/mpeg',
        'ogg'=>'application/ogg',
        'png'=>'image/png',
        'rtf'=>'text/rtf',
        'rtx'=>'text/richtext',
        'swf'=>'application/x-shockwave-flash',
        'wav'=>'audio/x-wav',
        'wbmp'=>'image/vnd.wap.wbmp',
        'zip'=>'application/zip',
    );
    public function __construct($request,$options = array())
    {
        $this->request = $request;
    }
    //返回字符串信息
    public function responseText(){
        //判定如果是文件的情况先
        if($this->request->is_file){
            //如果是文件，直接使用文件返回的方式进行，当然，前提是不是php文件
            return $this->ResponseWithOther();
        }else{
            //非文件,那么这个就有趣了
            //判定有没有文件结尾，如果有文件结尾就直接交给文件处理，没有的话启动路由模式
            if($this->request->is_ext){
                return $this->ResponseWithOther();
            }else{
                //这里是路由相关的数据了，这里启动我们的拍黄片的核心信息
                return $this->ResponseWithPhp();
            }

        }
    }
    final function ResponseWithPhp(){

        $_SERVER['REQUEST_URI'] = $this->request->request;
        $app = new \Kuoqi\App();
        $app->__initCommonConfig();
        $app->__initInput();//输入部分的初始化
        $app->__initBehavior();//初始化用户行为部分behavior
        $app->__initPrivateConfig();
        //将当前app的信息注入到控制器中，让后面可以使用
        //本操作只会将本app信息写入到 \Kuoqi\Controller的静态中备用，不进行任何其他操作
        \Kuoqi\Controller::useApp($app);
        $app->view = new \Kuoqi\App\View($app);
        $app->response = new \Kuoqi\Response($app);
        ob_start();//开启输出
        //这里进入behavior的部分
        //这里执行一些app的方法，给APP安排上在这上面的显示和初始化这些
        $app->behavior = new \Kuoqi\Behavior($app);
        //行为的解析，使用多一个函数的目的是让这个信息可以被手动变更，而不是直接一次性固定死了
        $app->behavior->exec();
        $stream = ob_get_clean();
        $header = $app->response->flush(true);
        $header = $header.$stream;
        return $header;
    }
    final function ResponseWithHtml(){
        $fileDriver = new \Kuoqi\FileDriver('{@HOME}/'.$this->request->file);
        if($fileDriver->exist()){
            $content = $fileDriver->getContent();
            return $this->httpd->GetHeaders(200,"OK",$content,$this->getMime($this->request->file));
        }else{
            return $this->httpd->GetHeaders(404,"NotFound");
        }
    }
    final function ResponseWithOther(){
        $fileDriver = new \Kuoqi\FileDriver('{@HOME}/'.$this->request->path);
        if($fileDriver->exist()){
            $content = $fileDriver->getContent();
            return $this->httpd->GetHeaders(200,"OK",$content,$this->getMime($this->request->path));
        }else{
            return $this->httpd->GetHeaders(404,"NotFound");
        }
    }
    /**
     * 获取资源的类型
     * @param $path
     * @return mixed
     */
    final function getMime($path){
        $type=explode(".",$path);
        $ext = isset($type[1])?$type[1]:'html';
        $mime=isset($this->mime_types[$ext])?$this->mime_types[$ext]:'*/*';
        return $mime;
    }

}